/***********************************************************************************************************************
 * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
 *
 * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
 * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
 * sold pursuant to Renesas terms and conditions of sale.  Purchasers are solely responsible for the selection and use
 * of Renesas products and Renesas assumes no liability.  No license, express or implied, to any intellectual property
 * right is granted by Renesas. This software is protected under all applicable laws, including copyright laws. Renesas
 * reserves the right to change or discontinue this software and/or this documentation. THE SOFTWARE AND DOCUMENTATION
 * IS DELIVERED TO YOU "AS IS," AND RENESAS MAKES NO REPRESENTATIONS OR WARRANTIES, AND TO THE FULLEST EXTENT
 * PERMISSIBLE UNDER APPLICABLE LAW, DISCLAIMS ALL WARRANTIES, WHETHER EXPLICITLY OR IMPLICITLY, INCLUDING WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, AND NONINFRINGEMENT, WITH RESPECT TO THE SOFTWARE OR
 * DOCUMENTATION.  RENESAS SHALL HAVE NO LIABILITY ARISING OUT OF ANY SECURITY VULNERABILITY OR BREACH.  TO THE MAXIMUM
 * EXTENT PERMITTED BY LAW, IN NO EVENT WILL RENESAS BE LIABLE TO YOU IN CONNECTION WITH THE SOFTWARE OR DOCUMENTATION
 * (OR ANY PERSON OR ENTITY CLAIMING RIGHTS DERIVED FROM YOU) FOR ANY LOSS, DAMAGES, OR CLAIMS WHATSOEVER, INCLUDING,
 * WITHOUT LIMITATION, ANY DIRECT, CONSEQUENTIAL, SPECIAL, INDIRECT, PUNITIVE, OR INCIDENTAL DAMAGES; ANY LOST PROFITS,
 * OTHER ECONOMIC DAMAGE, PROPERTY DAMAGE, OR PERSONAL INJURY; AND EVEN IF RENESAS HAS BEEN ADVISED OF THE POSSIBILITY
 * OF SUCH LOSS, DAMAGES, CLAIMS OR COSTS.
 **********************************************************************************************************************/

#ifndef  __SCE_ProcCommon_h__
#define  __SCE_ProcCommon_h__

#include  <stdint.h>

#include "bsp_api.h"                   /* For Crypto Error codes */
#include "SCE_module.h"

/* ================================================================================ */
/* ================                      SCE                      ================ */
/* ================================================================================ */

/**
 * @brief Trusted Security IP (SCE)
 */
typedef struct
{
    union
    {
        __IOM uint32_t REG_00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_00H_b;
    };
    union
    {
        __IOM uint32_t REG_04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_04H_b;
    };
    union
    {
        __IOM uint32_t REG_08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_08H_b;
    };
    union
    {
        __IOM uint32_t REG_0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_0CH_b;
    };
    union
    {
        __IOM uint32_t REG_10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10H_b;
    };
    union
    {
        __IOM uint32_t REG_14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14H_b;
    };
    union
    {
        __IOM uint32_t REG_18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18H_b;
    };
    union
    {
        __IOM uint32_t REG_1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CH_b;
    };
    union
    {
        __IOM uint32_t REG_20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20H_b;
    };
    union
    {
        __IOM uint32_t REG_24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24H_b;
    };
    union
    {
        __IOM uint32_t REG_28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_28H_b;
    };
    union
    {
        __IOM uint32_t REG_2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2CH_b;
    };
    union
    {
        __IOM uint32_t REG_30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_30H_b;
    };
    union
    {
        __IOM uint32_t REG_34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_34H_b;
    };
    union
    {
        __IOM uint32_t REG_38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_38H_b;
    };
    union
    {
        __IOM uint32_t REG_3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3CH_b;
    };
    union
    {
        __IOM uint32_t REG_40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_40H_b;
    };
    union
    {
        __IOM uint32_t REG_44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_44H_b;
    };
    union
    {
        __IOM uint32_t REG_48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_48H_b;
    };
    union
    {
        __IOM uint32_t REG_4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4CH_b;
    };
    union
    {
        __IOM uint32_t REG_50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_50H_b;
    };
    union
    {
        __IOM uint32_t REG_54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_54H_b;
    };
    union
    {
        __IOM uint32_t REG_58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_58H_b;
    };
    union
    {
        __IOM uint32_t REG_5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5CH_b;
    };
    union
    {
        __IOM uint32_t REG_60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_60H_b;
    };
    union
    {
        __IOM uint32_t REG_64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_64H_b;
    };
    union
    {
        __IOM uint32_t REG_68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_68H_b;
    };
    union
    {
        __IOM uint32_t REG_6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6CH_b;
    };
    union
    {
        __IOM uint32_t REG_70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_70H_b;
    };
    union
    {
        __IOM uint32_t REG_74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_74H_b;
    };
    union
    {
        __IOM uint32_t REG_78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_78H_b;
    };
    union
    {
        __IOM uint32_t REG_7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7CH_b;
    };
    union
    {
        __IOM uint32_t REG_80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_80H_b;
    };
    union
    {
        __IOM uint32_t REG_84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_84H_b;
    };
    union
    {
        __IOM uint32_t REG_88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_88H_b;
    };
    union
    {
        __IOM uint32_t REG_8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8CH_b;
    };
    union
    {
        __IOM uint32_t REG_90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_90H_b;
    };
    union
    {
        __IOM uint32_t REG_94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_94H_b;
    };
    union
    {
        __IOM uint32_t REG_98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_98H_b;
    };
    union
    {
        __IOM uint32_t REG_9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9CH_b;
    };
    union
    {
        __IOM uint32_t REG_A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A0H_b;
    };
    union
    {
        __IOM uint32_t REG_A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A4H_b;
    };
    union
    {
        __IOM uint32_t REG_A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A8H_b;
    };
    union
    {
        __IOM uint32_t REG_ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ACH_b;
    };
    union
    {
        __IOM uint32_t REG_B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B0H_b;
    };
    union
    {
        __IOM uint32_t REG_B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B4H_b;
    };
    union
    {
        __IOM uint32_t REG_B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B8H_b;
    };
    union
    {
        __IOM uint32_t REG_BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BCH_b;
    };
    union
    {
        __IOM uint32_t REG_C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C0H_b;
    };
    union
    {
        __IOM uint32_t REG_C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C4H_b;
    };
    union
    {
        __IOM uint32_t REG_C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C8H_b;
    };
    union
    {
        __IOM uint32_t REG_CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CCH_b;
    };
    union
    {
        __IOM uint32_t REG_D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D0H_b;
    };
    union
    {
        __IOM uint32_t REG_D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D4H_b;
    };
    union
    {
        __IOM uint32_t REG_D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D8H_b;
    };
    union
    {
        __IOM uint32_t REG_DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DCH_b;
    };
    union
    {
        __IOM uint32_t REG_E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E0H_b;
    };
    union
    {
        __IOM uint32_t REG_E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E4H_b;
    };
    union
    {
        __IOM uint32_t REG_E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E8H_b;
    };
    union
    {
        __IOM uint32_t REG_ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ECH_b;
    };
    union
    {
        __IOM uint32_t REG_F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F0H_b;
    };
    union
    {
        __IOM uint32_t REG_F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F4H_b;
    };
    union
    {
        __IOM uint32_t REG_F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F8H_b;
    };
    union
    {
        __IOM uint32_t REG_FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FCH_b;
    };
    union
    {
        __IOM uint32_t REG_100H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_100H_b;
    };
    union
    {
        __IOM uint32_t REG_104H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_104H_b;
    };
    union
    {
        __IOM uint32_t REG_108H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_108H_b;
    };
    union
    {
        __IOM uint32_t REG_10CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10CH_b;
    };
    union
    {
        __IOM uint32_t REG_110H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_110H_b;
    };
    union
    {
        __IOM uint32_t REG_114H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_114H_b;
    };
    union
    {
        __IOM uint32_t REG_118H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_118H_b;
    };
    union
    {
        __IOM uint32_t REG_11CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11CH_b;
    };
    union
    {
        __IOM uint32_t REG_120H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_120H_b;
    };
    union
    {
        __IOM uint32_t REG_124H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_124H_b;
    };
    union
    {
        __IOM uint32_t REG_128H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_128H_b;
    };
    union
    {
        __IOM uint32_t REG_12CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12CH_b;
    };
    union
    {
        __IOM uint32_t REG_130H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_130H_b;
    };
    union
    {
        __IOM uint32_t REG_134H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_134H_b;
    };
    union
    {
        __IOM uint32_t REG_138H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_138H_b;
    };
    union
    {
        __IOM uint32_t REG_13CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13CH_b;
    };
    union
    {
        __IOM uint32_t REG_140H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_140H_b;
    };
    union
    {
        __IOM uint32_t REG_144H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_144H_b;
    };
    union
    {
        __IOM uint32_t REG_148H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_148H_b;
    };
    union
    {
        __IOM uint32_t REG_14CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14CH_b;
    };
    union
    {
        __IOM uint32_t REG_150H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_150H_b;
    };
    union
    {
        __IOM uint32_t REG_154H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_154H_b;
    };
    union
    {
        __IOM uint32_t REG_158H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_158H_b;
    };
    union
    {
        __IOM uint32_t REG_15CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15CH_b;
    };
    union
    {
        __IOM uint32_t REG_160H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_160H_b;
    };
    union
    {
        __IOM uint32_t REG_164H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_164H_b;
    };
    union
    {
        __IOM uint32_t REG_168H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_168H_b;
    };
    union
    {
        __IOM uint32_t REG_16CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16CH_b;
    };
    union
    {
        __IOM uint32_t REG_170H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_170H_b;
    };
    union
    {
        __IOM uint32_t REG_174H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_174H_b;
    };
    union
    {
        __IOM uint32_t REG_178H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_178H_b;
    };
    union
    {
        __IOM uint32_t REG_17CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17CH_b;
    };
    union
    {
        __IOM uint32_t REG_180H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_180H_b;
    };
    union
    {
        __IOM uint32_t REG_184H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_184H_b;
    };
    union
    {
        __IOM uint32_t REG_188H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_188H_b;
    };
    union
    {
        __IOM uint32_t REG_18CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18CH_b;
    };
    union
    {
        __IOM uint32_t REG_190H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_190H_b;
    };
    union
    {
        __IOM uint32_t REG_194H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_194H_b;
    };
    union
    {
        __IOM uint32_t REG_198H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_198H_b;
    };
    union
    {
        __IOM uint32_t REG_19CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A0H_b;
    };
    union
    {
        __IOM uint32_t REG_1A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A4H_b;
    };
    union
    {
        __IOM uint32_t REG_1A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A8H_b;
    };
    union
    {
        __IOM uint32_t REG_1ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ACH_b;
    };
    union
    {
        __IOM uint32_t REG_1B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B0H_b;
    };
    union
    {
        __IOM uint32_t REG_1B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B4H_b;
    };
    union
    {
        __IOM uint32_t REG_1B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BCH_b;
    };
    union
    {
        __IOM uint32_t REG_1C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C0H_b;
    };
    union
    {
        __IOM uint32_t REG_1C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C4H_b;
    };
    union
    {
        __IOM uint32_t REG_1C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CCH_b;
    };
    union
    {
        __IOM uint32_t REG_1D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D0H_b;
    };
    union
    {
        __IOM uint32_t REG_1D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D4H_b;
    };
    union
    {
        __IOM uint32_t REG_1D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DCH_b;
    };
    union
    {
        __IOM uint32_t REG_1E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E0H_b;
    };
    union
    {
        __IOM uint32_t REG_1E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E4H_b;
    };
    union
    {
        __IOM uint32_t REG_1E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E8H_b;
    };
    union
    {
        __IOM uint32_t REG_1ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ECH_b;
    };
    union
    {
        __IOM uint32_t REG_1F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F0H_b;
    };
    union
    {
        __IOM uint32_t REG_1F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F4H_b;
    };
    union
    {
        __IOM uint32_t REG_1F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FCH_b;
    };
    union
    {
        __IOM uint32_t REG_200H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_200H_b;
    };
    union
    {
        __IOM uint32_t REG_204H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_204H_b;
    };
    union
    {
        __IOM uint32_t REG_208H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_208H_b;
    };
    union
    {
        __IOM uint32_t REG_20CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20CH_b;
    };
    union
    {
        __IOM uint32_t REG_210H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_210H_b;
    };
    union
    {
        __IOM uint32_t REG_214H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_214H_b;
    };
    union
    {
        __IOM uint32_t REG_218H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_218H_b;
    };
    union
    {
        __IOM uint32_t REG_21CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21CH_b;
    };
    union
    {
        __IOM uint32_t REG_220H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_220H_b;
    };
    union
    {
        __IOM uint32_t REG_224H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_224H_b;
    };
    union
    {
        __IOM uint32_t REG_228H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_228H_b;
    };
    union
    {
        __IOM uint32_t REG_22CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22CH_b;
    };
    union
    {
        __IOM uint32_t REG_230H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_230H_b;
    };
    union
    {
        __IOM uint32_t REG_234H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_234H_b;
    };
    union
    {
        __IOM uint32_t REG_238H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_238H_b;
    };
    union
    {
        __IOM uint32_t REG_23CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23CH_b;
    };
    union
    {
        __IOM uint32_t REG_240H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_240H_b;
    };
    union
    {
        __IOM uint32_t REG_244H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_244H_b;
    };
    union
    {
        __IOM uint32_t REG_248H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_248H_b;
    };
    union
    {
        __IOM uint32_t REG_24CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24CH_b;
    };
    union
    {
        __IOM uint32_t REG_250H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_250H_b;
    };
    union
    {
        __IOM uint32_t REG_254H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_254H_b;
    };
    union
    {
        __IOM uint32_t REG_258H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_258H_b;
    };
    union
    {
        __IOM uint32_t REG_25CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_25CH_b;
    };
    union
    {
        __IOM uint32_t REG_260H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_260H_b;
    };
    union
    {
        __IOM uint32_t REG_264H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_264H_b;
    };
    union
    {
        __IOM uint32_t REG_268H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_268H_b;
    };
    union
    {
        __IOM uint32_t REG_26CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_26CH_b;
    };
    union
    {
        __IOM uint32_t REG_270H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_270H_b;
    };
    union
    {
        __IOM uint32_t REG_274H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_274H_b;
    };
    union
    {
        __IOM uint32_t REG_278H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_278H_b;
    };
    union
    {
        __IOM uint32_t REG_27CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_27CH_b;
    };
    union
    {
        __IOM uint32_t REG_280H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_280H_b;
    };
    union
    {
        __IOM uint32_t REG_284H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_284H_b;
    };
    union
    {
        __IOM uint32_t REG_288H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_288H_b;
    };
    union
    {
        __IOM uint32_t REG_28CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_28CH_b;
    };
    union
    {
        __IOM uint32_t REG_290H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_290H_b;
    };
    union
    {
        __IOM uint32_t REG_294H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_294H_b;
    };
    union
    {
        __IOM uint32_t REG_298H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_298H_b;
    };
    union
    {
        __IOM uint32_t REG_29CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_29CH_b;
    };
    union
    {
        __IOM uint32_t REG_2A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2A0H_b;
    };
    union
    {
        __IOM uint32_t REG_2A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2A4H_b;
    };
    union
    {
        __IOM uint32_t REG_2A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2A8H_b;
    };
    union
    {
        __IOM uint32_t REG_2ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2ACH_b;
    };
    union
    {
        __IOM uint32_t REG_2B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2B0H_b;
    };
    union
    {
        __IOM uint32_t REG_2B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2B4H_b;
    };
    union
    {
        __IOM uint32_t REG_2B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2B8H_b;
    };
    union
    {
        __IOM uint32_t REG_2BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2BCH_b;
    };
    union
    {
        __IOM uint32_t REG_2C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2C0H_b;
    };
    union
    {
        __IOM uint32_t REG_2C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2C4H_b;
    };
    union
    {
        __IOM uint32_t REG_2C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2C8H_b;
    };
    union
    {
        __IOM uint32_t REG_2CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2CCH_b;
    };
    union
    {
        __IOM uint32_t REG_2D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2D0H_b;
    };
    union
    {
        __IOM uint32_t REG_2D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2D4H_b;
    };
    union
    {
        __IOM uint32_t REG_2D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2D8H_b;
    };
    union
    {
        __IOM uint32_t REG_2DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2DCH_b;
    };
    union
    {
        __IOM uint32_t REG_2E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2E0H_b;
    };
    union
    {
        __IOM uint32_t REG_2E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2E4H_b;
    };
    union
    {
        __IOM uint32_t REG_2E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2E8H_b;
    };
    union
    {
        __IOM uint32_t REG_2ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2ECH_b;
    };
    union
    {
        __IOM uint32_t REG_2F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2F0H_b;
    };
    union
    {
        __IOM uint32_t REG_2F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2F4H_b;
    };
    union
    {
        __IOM uint32_t REG_2F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2F8H_b;
    };
    union
    {
        __IOM uint32_t REG_2FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2FCH_b;
    };
    union
    {
        __IOM uint32_t REG_300H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_300H_b;
    };
    union
    {
        __IOM uint32_t REG_304H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_304H_b;
    };
    union
    {
        __IOM uint32_t REG_308H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_308H_b;
    };
    union
    {
        __IOM uint32_t REG_30CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_30CH_b;
    };
    union
    {
        __IOM uint32_t REG_310H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_310H_b;
    };
    union
    {
        __IOM uint32_t REG_314H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_314H_b;
    };
    union
    {
        __IOM uint32_t REG_318H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_318H_b;
    };
    union
    {
        __IOM uint32_t REG_31CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_31CH_b;
    };
    union
    {
        __IOM uint32_t REG_320H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_320H_b;
    };
    union
    {
        __IOM uint32_t REG_324H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_324H_b;
    };
    union
    {
        __IOM uint32_t REG_328H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_328H_b;
    };
    union
    {
        __IOM uint32_t REG_32CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_32CH_b;
    };
    union
    {
        __IOM uint32_t REG_330H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_330H_b;
    };
    union
    {
        __IOM uint32_t REG_334H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_334H_b;
    };
    union
    {
        __IOM uint32_t REG_338H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_338H_b;
    };
    union
    {
        __IOM uint32_t REG_33CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_33CH_b;
    };
    union
    {
        __IOM uint32_t REG_340H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_340H_b;
    };
    union
    {
        __IOM uint32_t REG_344H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_344H_b;
    };
    union
    {
        __IOM uint32_t REG_348H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_348H_b;
    };
    union
    {
        __IOM uint32_t REG_34CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_34CH_b;
    };
    union
    {
        __IOM uint32_t REG_350H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_350H_b;
    };
    union
    {
        __IOM uint32_t REG_354H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_354H_b;
    };
    union
    {
        __IOM uint32_t REG_358H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_358H_b;
    };
    union
    {
        __IOM uint32_t REG_35CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_35CH_b;
    };
    union
    {
        __IOM uint32_t REG_360H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_360H_b;
    };
    union
    {
        __IOM uint32_t REG_364H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_364H_b;
    };
    union
    {
        __IOM uint32_t REG_368H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_368H_b;
    };
    union
    {
        __IOM uint32_t REG_36CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_36CH_b;
    };
    union
    {
        __IOM uint32_t REG_370H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_370H_b;
    };
    union
    {
        __IOM uint32_t REG_374H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_374H_b;
    };
    union
    {
        __IOM uint32_t REG_378H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_378H_b;
    };
    union
    {
        __IOM uint32_t REG_37CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_37CH_b;
    };
    union
    {
        __IOM uint32_t REG_380H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_380H_b;
    };
    union
    {
        __IOM uint32_t REG_384H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_384H_b;
    };
    union
    {
        __IOM uint32_t REG_388H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_388H_b;
    };
    union
    {
        __IOM uint32_t REG_38CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_38CH_b;
    };
    union
    {
        __IOM uint32_t REG_390H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_390H_b;
    };
    union
    {
        __IOM uint32_t REG_394H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_394H_b;
    };
    union
    {
        __IOM uint32_t REG_398H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_398H_b;
    };
    union
    {
        __IOM uint32_t REG_39CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_39CH_b;
    };
    union
    {
        __IOM uint32_t REG_3A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3A0H_b;
    };
    union
    {
        __IOM uint32_t REG_3A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3A4H_b;
    };
    union
    {
        __IOM uint32_t REG_3A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3A8H_b;
    };
    union
    {
        __IOM uint32_t REG_3ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3ACH_b;
    };
    union
    {
        __IOM uint32_t REG_3B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3B0H_b;
    };
    union
    {
        __IOM uint32_t REG_3B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3B4H_b;
    };
    union
    {
        __IOM uint32_t REG_3B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3B8H_b;
    };
    union
    {
        __IOM uint32_t REG_3BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3BCH_b;
    };
    union
    {
        __IOM uint32_t REG_3C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3C0H_b;
    };
    union
    {
        __IOM uint32_t REG_3C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3C4H_b;
    };
    union
    {
        __IOM uint32_t REG_3C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3C8H_b;
    };
    union
    {
        __IOM uint32_t REG_3CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3CCH_b;
    };
    union
    {
        __IOM uint32_t REG_3D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3D0H_b;
    };
    union
    {
        __IOM uint32_t REG_3D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3D4H_b;
    };
    union
    {
        __IOM uint32_t REG_3D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3D8H_b;
    };
    union
    {
        __IOM uint32_t REG_3DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3DCH_b;
    };
    union
    {
        __IOM uint32_t REG_3E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3E0H_b;
    };
    union
    {
        __IOM uint32_t REG_3E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3E4H_b;
    };
    union
    {
        __IOM uint32_t REG_3E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3E8H_b;
    };
    union
    {
        __IOM uint32_t REG_3ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3ECH_b;
    };
    union
    {
        __IOM uint32_t REG_3F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3F0H_b;
    };
    union
    {
        __IOM uint32_t REG_3F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3F4H_b;
    };
    union
    {
        __IOM uint32_t REG_3F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3F8H_b;
    };
    union
    {
        __IOM uint32_t REG_3FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_3FCH_b;
    };
    union
    {
        __IOM uint32_t REG_400H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_400H_b;
    };
    union
    {
        __IOM uint32_t REG_404H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_404H_b;
    };
    union
    {
        __IOM uint32_t REG_408H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_408H_b;
    };
    union
    {
        __IOM uint32_t REG_40CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_40CH_b;
    };
    union
    {
        __IOM uint32_t REG_410H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_410H_b;
    };
    union
    {
        __IOM uint32_t REG_414H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_414H_b;
    };
    union
    {
        __IOM uint32_t REG_418H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_418H_b;
    };
    union
    {
        __IOM uint32_t REG_41CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_41CH_b;
    };
    union
    {
        __IOM uint32_t REG_420H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_420H_b;
    };
    union
    {
        __IOM uint32_t REG_424H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_424H_b;
    };
    union
    {
        __IOM uint32_t REG_428H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_428H_b;
    };
    union
    {
        __IOM uint32_t REG_42CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_42CH_b;
    };
    union
    {
        __IOM uint32_t REG_430H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_430H_b;
    };
    union
    {
        __IOM uint32_t REG_434H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_434H_b;
    };
    union
    {
        __IOM uint32_t REG_438H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_438H_b;
    };
    union
    {
        __IOM uint32_t REG_43CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_43CH_b;
    };
    union
    {
        __IOM uint32_t REG_440H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_440H_b;
    };
    union
    {
        __IOM uint32_t REG_444H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_444H_b;
    };
    union
    {
        __IOM uint32_t REG_448H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_448H_b;
    };
    union
    {
        __IOM uint32_t REG_44CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_44CH_b;
    };
    union
    {
        __IOM uint32_t REG_450H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_450H_b;
    };
    union
    {
        __IOM uint32_t REG_454H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_454H_b;
    };
    union
    {
        __IOM uint32_t REG_458H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_458H_b;
    };
    union
    {
        __IOM uint32_t REG_45CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_45CH_b;
    };
    union
    {
        __IOM uint32_t REG_460H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_460H_b;
    };
    union
    {
        __IOM uint32_t REG_464H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_464H_b;
    };
    union
    {
        __IOM uint32_t REG_468H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_468H_b;
    };
    union
    {
        __IOM uint32_t REG_46CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_46CH_b;
    };
    union
    {
        __IOM uint32_t REG_470H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_470H_b;
    };
    union
    {
        __IOM uint32_t REG_474H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_474H_b;
    };
    union
    {
        __IOM uint32_t REG_478H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_478H_b;
    };
    union
    {
        __IOM uint32_t REG_47CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_47CH_b;
    };
    union
    {
        __IOM uint32_t REG_480H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_480H_b;
    };
    union
    {
        __IOM uint32_t REG_484H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_484H_b;
    };
    union
    {
        __IOM uint32_t REG_488H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_488H_b;
    };
    union
    {
        __IOM uint32_t REG_48CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_48CH_b;
    };
    union
    {
        __IOM uint32_t REG_490H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_490H_b;
    };
    union
    {
        __IOM uint32_t REG_494H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_494H_b;
    };
    union
    {
        __IOM uint32_t REG_498H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_498H_b;
    };
    union
    {
        __IOM uint32_t REG_49CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_49CH_b;
    };
    union
    {
        __IOM uint32_t REG_4A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4A0H_b;
    };
    union
    {
        __IOM uint32_t REG_4A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4A4H_b;
    };
    union
    {
        __IOM uint32_t REG_4A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4A8H_b;
    };
    union
    {
        __IOM uint32_t REG_4ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4ACH_b;
    };
    union
    {
        __IOM uint32_t REG_4B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4B0H_b;
    };
    union
    {
        __IOM uint32_t REG_4B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4B4H_b;
    };
    union
    {
        __IOM uint32_t REG_4B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4B8H_b;
    };
    union
    {
        __IOM uint32_t REG_4BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4BCH_b;
    };
    union
    {
        __IOM uint32_t REG_4C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4C0H_b;
    };
    union
    {
        __IOM uint32_t REG_4C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4C4H_b;
    };
    union
    {
        __IOM uint32_t REG_4C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4C8H_b;
    };
    union
    {
        __IOM uint32_t REG_4CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4CCH_b;
    };
    union
    {
        __IOM uint32_t REG_4D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4D0H_b;
    };
    union
    {
        __IOM uint32_t REG_4D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4D4H_b;
    };
    union
    {
        __IOM uint32_t REG_4D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4D8H_b;
    };
    union
    {
        __IOM uint32_t REG_4DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4DCH_b;
    };
    union
    {
        __IOM uint32_t REG_4E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4E0H_b;
    };
    union
    {
        __IOM uint32_t REG_4E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4E4H_b;
    };
    union
    {
        __IOM uint32_t REG_4E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4E8H_b;
    };
    union
    {
        __IOM uint32_t REG_4ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4ECH_b;
    };
    union
    {
        __IOM uint32_t REG_4F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4F0H_b;
    };
    union
    {
        __IOM uint32_t REG_4F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4F4H_b;
    };
    union
    {
        __IOM uint32_t REG_4F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4F8H_b;
    };
    union
    {
        __IOM uint32_t REG_4FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_4FCH_b;
    };
    union
    {
        __IOM uint32_t REG_500H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_500H_b;
    };
    union
    {
        __IOM uint32_t REG_504H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_504H_b;
    };
    union
    {
        __IOM uint32_t REG_508H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_508H_b;
    };
    union
    {
        __IOM uint32_t REG_50CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_50CH_b;
    };
    union
    {
        __IOM uint32_t REG_510H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_510H_b;
    };
    union
    {
        __IOM uint32_t REG_514H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_514H_b;
    };
    union
    {
        __IOM uint32_t REG_518H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_518H_b;
    };
    union
    {
        __IOM uint32_t REG_51CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_51CH_b;
    };
    union
    {
        __IOM uint32_t REG_520H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_520H_b;
    };
    union
    {
        __IOM uint32_t REG_524H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_524H_b;
    };
    union
    {
        __IOM uint32_t REG_528H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_528H_b;
    };
    union
    {
        __IOM uint32_t REG_52CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_52CH_b;
    };
    union
    {
        __IOM uint32_t REG_530H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_530H_b;
    };
    union
    {
        __IOM uint32_t REG_534H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_534H_b;
    };
    union
    {
        __IOM uint32_t REG_538H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_538H_b;
    };
    union
    {
        __IOM uint32_t REG_53CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_53CH_b;
    };
    union
    {
        __IOM uint32_t REG_540H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_540H_b;
    };
    union
    {
        __IOM uint32_t REG_544H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_544H_b;
    };
    union
    {
        __IOM uint32_t REG_548H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_548H_b;
    };
    union
    {
        __IOM uint32_t REG_54CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_54CH_b;
    };
    union
    {
        __IOM uint32_t REG_550H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_550H_b;
    };
    union
    {
        __IOM uint32_t REG_554H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_554H_b;
    };
    union
    {
        __IOM uint32_t REG_558H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_558H_b;
    };
    union
    {
        __IOM uint32_t REG_55CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_55CH_b;
    };
    union
    {
        __IOM uint32_t REG_560H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_560H_b;
    };
    union
    {
        __IOM uint32_t REG_564H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_564H_b;
    };
    union
    {
        __IOM uint32_t REG_568H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_568H_b;
    };
    union
    {
        __IOM uint32_t REG_56CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_56CH_b;
    };
    union
    {
        __IOM uint32_t REG_570H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_570H_b;
    };
    union
    {
        __IOM uint32_t REG_574H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_574H_b;
    };
    union
    {
        __IOM uint32_t REG_578H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_578H_b;
    };
    union
    {
        __IOM uint32_t REG_57CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_57CH_b;
    };
    union
    {
        __IOM uint32_t REG_580H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_580H_b;
    };
    union
    {
        __IOM uint32_t REG_584H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_584H_b;
    };
    union
    {
        __IOM uint32_t REG_588H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_588H_b;
    };
    union
    {
        __IOM uint32_t REG_58CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_58CH_b;
    };
    union
    {
        __IOM uint32_t REG_590H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_590H_b;
    };
    union
    {
        __IOM uint32_t REG_594H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_594H_b;
    };
    union
    {
        __IOM uint32_t REG_598H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_598H_b;
    };
    union
    {
        __IOM uint32_t REG_59CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_59CH_b;
    };
    union
    {
        __IOM uint32_t REG_5A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5A0H_b;
    };
    union
    {
        __IOM uint32_t REG_5A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5A4H_b;
    };
    union
    {
        __IOM uint32_t REG_5A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5A8H_b;
    };
    union
    {
        __IOM uint32_t REG_5ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5ACH_b;
    };
    union
    {
        __IOM uint32_t REG_5B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5B0H_b;
    };
    union
    {
        __IOM uint32_t REG_5B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5B4H_b;
    };
    union
    {
        __IOM uint32_t REG_5B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5B8H_b;
    };
    union
    {
        __IOM uint32_t REG_5BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5BCH_b;
    };
    union
    {
        __IOM uint32_t REG_5C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5C0H_b;
    };
    union
    {
        __IOM uint32_t REG_5C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5C4H_b;
    };
    union
    {
        __IOM uint32_t REG_5C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5C8H_b;
    };
    union
    {
        __IOM uint32_t REG_5CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5CCH_b;
    };
    union
    {
        __IOM uint32_t REG_5D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5D0H_b;
    };
    union
    {
        __IOM uint32_t REG_5D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5D4H_b;
    };
    union
    {
        __IOM uint32_t REG_5D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5D8H_b;
    };
    union
    {
        __IOM uint32_t REG_5DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5DCH_b;
    };
    union
    {
        __IOM uint32_t REG_5E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5E0H_b;
    };
    union
    {
        __IOM uint32_t REG_5E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5E4H_b;
    };
    union
    {
        __IOM uint32_t REG_5E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5E8H_b;
    };
    union
    {
        __IOM uint32_t REG_5ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5ECH_b;
    };
    union
    {
        __IOM uint32_t REG_5F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5F0H_b;
    };
    union
    {
        __IOM uint32_t REG_5F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5F4H_b;
    };
    union
    {
        __IOM uint32_t REG_5F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5F8H_b;
    };
    union
    {
        __IOM uint32_t REG_5FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_5FCH_b;
    };
    union
    {
        __IOM uint32_t REG_600H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_600H_b;
    };
    union
    {
        __IOM uint32_t REG_604H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_604H_b;
    };
    union
    {
        __IOM uint32_t REG_608H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_608H_b;
    };
    union
    {
        __IOM uint32_t REG_60CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_60CH_b;
    };
    union
    {
        __IOM uint32_t REG_610H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_610H_b;
    };
    union
    {
        __IOM uint32_t REG_614H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_614H_b;
    };
    union
    {
        __IOM uint32_t REG_618H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_618H_b;
    };
    union
    {
        __IOM uint32_t REG_61CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_61CH_b;
    };
    union
    {
        __IOM uint32_t REG_620H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_620H_b;
    };
    union
    {
        __IOM uint32_t REG_624H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_624H_b;
    };
    union
    {
        __IOM uint32_t REG_628H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_628H_b;
    };
    union
    {
        __IOM uint32_t REG_62CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_62CH_b;
    };
    union
    {
        __IOM uint32_t REG_630H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_630H_b;
    };
    union
    {
        __IOM uint32_t REG_634H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_634H_b;
    };
    union
    {
        __IOM uint32_t REG_638H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_638H_b;
    };
    union
    {
        __IOM uint32_t REG_63CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_63CH_b;
    };
    union
    {
        __IOM uint32_t REG_640H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_640H_b;
    };
    union
    {
        __IOM uint32_t REG_644H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_644H_b;
    };
    union
    {
        __IOM uint32_t REG_648H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_648H_b;
    };
    union
    {
        __IOM uint32_t REG_64CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_64CH_b;
    };
    union
    {
        __IOM uint32_t REG_650H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_650H_b;
    };
    union
    {
        __IOM uint32_t REG_654H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_654H_b;
    };
    union
    {
        __IOM uint32_t REG_658H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_658H_b;
    };
    union
    {
        __IOM uint32_t REG_65CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_65CH_b;
    };
    union
    {
        __IOM uint32_t REG_660H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_660H_b;
    };
    union
    {
        __IOM uint32_t REG_664H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_664H_b;
    };
    union
    {
        __IOM uint32_t REG_668H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_668H_b;
    };
    union
    {
        __IOM uint32_t REG_66CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_66CH_b;
    };
    union
    {
        __IOM uint32_t REG_670H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_670H_b;
    };
    union
    {
        __IOM uint32_t REG_674H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_674H_b;
    };
    union
    {
        __IOM uint32_t REG_678H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_678H_b;
    };
    union
    {
        __IOM uint32_t REG_67CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_67CH_b;
    };
    union
    {
        __IOM uint32_t REG_680H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_680H_b;
    };
    union
    {
        __IOM uint32_t REG_684H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_684H_b;
    };
    union
    {
        __IOM uint32_t REG_688H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_688H_b;
    };
    union
    {
        __IOM uint32_t REG_68CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_68CH_b;
    };
    union
    {
        __IOM uint32_t REG_690H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_690H_b;
    };
    union
    {
        __IOM uint32_t REG_694H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_694H_b;
    };
    union
    {
        __IOM uint32_t REG_698H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_698H_b;
    };
    union
    {
        __IOM uint32_t REG_69CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_69CH_b;
    };
    union
    {
        __IOM uint32_t REG_6A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6A0H_b;
    };
    union
    {
        __IOM uint32_t REG_6A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6A4H_b;
    };
    union
    {
        __IOM uint32_t REG_6A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6A8H_b;
    };
    union
    {
        __IOM uint32_t REG_6ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6ACH_b;
    };
    union
    {
        __IOM uint32_t REG_6B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6B0H_b;
    };
    union
    {
        __IOM uint32_t REG_6B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6B4H_b;
    };
    union
    {
        __IOM uint32_t REG_6B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6B8H_b;
    };
    union
    {
        __IOM uint32_t REG_6BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6BCH_b;
    };
    union
    {
        __IOM uint32_t REG_6C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6C0H_b;
    };
    union
    {
        __IOM uint32_t REG_6C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6C4H_b;
    };
    union
    {
        __IOM uint32_t REG_6C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6C8H_b;
    };
    union
    {
        __IOM uint32_t REG_6CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6CCH_b;
    };
    union
    {
        __IOM uint32_t REG_6D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6D0H_b;
    };
    union
    {
        __IOM uint32_t REG_6D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6D4H_b;
    };
    union
    {
        __IOM uint32_t REG_6D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6D8H_b;
    };
    union
    {
        __IOM uint32_t REG_6DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6DCH_b;
    };
    union
    {
        __IOM uint32_t REG_6E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6E0H_b;
    };
    union
    {
        __IOM uint32_t REG_6E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6E4H_b;
    };
    union
    {
        __IOM uint32_t REG_6E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6E8H_b;
    };
    union
    {
        __IOM uint32_t REG_6ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6ECH_b;
    };
    union
    {
        __IOM uint32_t REG_6F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6F0H_b;
    };
    union
    {
        __IOM uint32_t REG_6F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6F4H_b;
    };
    union
    {
        __IOM uint32_t REG_6F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6F8H_b;
    };
    union
    {
        __IOM uint32_t REG_6FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_6FCH_b;
    };
    union
    {
        __IOM uint32_t REG_700H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_700H_b;
    };
    union
    {
        __IOM uint32_t REG_704H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_704H_b;
    };
    union
    {
        __IOM uint32_t REG_708H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_708H_b;
    };
    union
    {
        __IOM uint32_t REG_70CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_70CH_b;
    };
    union
    {
        __IOM uint32_t REG_710H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_710H_b;
    };
    union
    {
        __IOM uint32_t REG_714H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_714H_b;
    };
    union
    {
        __IOM uint32_t REG_718H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_718H_b;
    };
    union
    {
        __IOM uint32_t REG_71CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_71CH_b;
    };
    union
    {
        __IOM uint32_t REG_720H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_720H_b;
    };
    union
    {
        __IOM uint32_t REG_724H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_724H_b;
    };
    union
    {
        __IOM uint32_t REG_728H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_728H_b;
    };
    union
    {
        __IOM uint32_t REG_72CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_72CH_b;
    };
    union
    {
        __IOM uint32_t REG_730H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_730H_b;
    };
    union
    {
        __IOM uint32_t REG_734H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_734H_b;
    };
    union
    {
        __IOM uint32_t REG_738H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_738H_b;
    };
    union
    {
        __IOM uint32_t REG_73CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_73CH_b;
    };
    union
    {
        __IOM uint32_t REG_740H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_740H_b;
    };
    union
    {
        __IOM uint32_t REG_744H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_744H_b;
    };
    union
    {
        __IOM uint32_t REG_748H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_748H_b;
    };
    union
    {
        __IOM uint32_t REG_74CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_74CH_b;
    };
    union
    {
        __IOM uint32_t REG_750H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_750H_b;
    };
    union
    {
        __IOM uint32_t REG_754H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_754H_b;
    };
    union
    {
        __IOM uint32_t REG_758H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_758H_b;
    };
    union
    {
        __IOM uint32_t REG_75CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_75CH_b;
    };
    union
    {
        __IOM uint32_t REG_760H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_760H_b;
    };
    union
    {
        __IOM uint32_t REG_764H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_764H_b;
    };
    union
    {
        __IOM uint32_t REG_768H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_768H_b;
    };
    union
    {
        __IOM uint32_t REG_76CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_76CH_b;
    };
    union
    {
        __IOM uint32_t REG_770H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_770H_b;
    };
    union
    {
        __IOM uint32_t REG_774H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_774H_b;
    };
    union
    {
        __IOM uint32_t REG_778H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_778H_b;
    };
    union
    {
        __IOM uint32_t REG_77CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_77CH_b;
    };
    union
    {
        __IOM uint32_t REG_780H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_780H_b;
    };
    union
    {
        __IOM uint32_t REG_784H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_784H_b;
    };
    union
    {
        __IOM uint32_t REG_788H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_788H_b;
    };
    union
    {
        __IOM uint32_t REG_78CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_78CH_b;
    };
    union
    {
        __IOM uint32_t REG_790H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_790H_b;
    };
    union
    {
        __IOM uint32_t REG_794H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_794H_b;
    };
    union
    {
        __IOM uint32_t REG_798H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_798H_b;
    };
    union
    {
        __IOM uint32_t REG_79CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_79CH_b;
    };
    union
    {
        __IOM uint32_t REG_7A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7A0H_b;
    };
    union
    {
        __IOM uint32_t REG_7A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7A4H_b;
    };
    union
    {
        __IOM uint32_t REG_7A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7A8H_b;
    };
    union
    {
        __IOM uint32_t REG_7ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7ACH_b;
    };
    union
    {
        __IOM uint32_t REG_7B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7B0H_b;
    };
    union
    {
        __IOM uint32_t REG_7B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7B4H_b;
    };
    union
    {
        __IOM uint32_t REG_7B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7B8H_b;
    };
    union
    {
        __IOM uint32_t REG_7BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7BCH_b;
    };
    union
    {
        __IOM uint32_t REG_7C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7C0H_b;
    };
    union
    {
        __IOM uint32_t REG_7C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7C4H_b;
    };
    union
    {
        __IOM uint32_t REG_7C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7C8H_b;
    };
    union
    {
        __IOM uint32_t REG_7CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7CCH_b;
    };
    union
    {
        __IOM uint32_t REG_7D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7D0H_b;
    };
    union
    {
        __IOM uint32_t REG_7D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7D4H_b;
    };
    union
    {
        __IOM uint32_t REG_7D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7D8H_b;
    };
    union
    {
        __IOM uint32_t REG_7DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7DCH_b;
    };
    union
    {
        __IOM uint32_t REG_7E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7E0H_b;
    };
    union
    {
        __IOM uint32_t REG_7E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7E4H_b;
    };
    union
    {
        __IOM uint32_t REG_7E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7E8H_b;
    };
    union
    {
        __IOM uint32_t REG_7ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7ECH_b;
    };
    union
    {
        __IOM uint32_t REG_7F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7F0H_b;
    };
    union
    {
        __IOM uint32_t REG_7F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7F4H_b;
    };
    union
    {
        __IOM uint32_t REG_7F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7F8H_b;
    };
    union
    {
        __IOM uint32_t REG_7FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_7FCH_b;
    };
    union
    {
        __IOM uint32_t REG_800H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_800H_b;
    };
    union
    {
        __IOM uint32_t REG_804H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_804H_b;
    };
    union
    {
        __IOM uint32_t REG_808H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_808H_b;
    };
    union
    {
        __IOM uint32_t REG_80CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_80CH_b;
    };
    union
    {
        __IOM uint32_t REG_810H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_810H_b;
    };
    union
    {
        __IOM uint32_t REG_814H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_814H_b;
    };
    union
    {
        __IOM uint32_t REG_818H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_818H_b;
    };
    union
    {
        __IOM uint32_t REG_81CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_81CH_b;
    };
    union
    {
        __IOM uint32_t REG_820H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_820H_b;
    };
    union
    {
        __IOM uint32_t REG_824H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_824H_b;
    };
    union
    {
        __IOM uint32_t REG_828H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_828H_b;
    };
    union
    {
        __IOM uint32_t REG_82CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_82CH_b;
    };
    union
    {
        __IOM uint32_t REG_830H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_830H_b;
    };
    union
    {
        __IOM uint32_t REG_834H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_834H_b;
    };
    union
    {
        __IOM uint32_t REG_838H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_838H_b;
    };
    union
    {
        __IOM uint32_t REG_83CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_83CH_b;
    };
    union
    {
        __IOM uint32_t REG_840H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_840H_b;
    };
    union
    {
        __IOM uint32_t REG_844H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_844H_b;
    };
    union
    {
        __IOM uint32_t REG_848H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_848H_b;
    };
    union
    {
        __IOM uint32_t REG_84CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_84CH_b;
    };
    union
    {
        __IOM uint32_t REG_850H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_850H_b;
    };
    union
    {
        __IOM uint32_t REG_854H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_854H_b;
    };
    union
    {
        __IOM uint32_t REG_858H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_858H_b;
    };
    union
    {
        __IOM uint32_t REG_85CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_85CH_b;
    };
    union
    {
        __IOM uint32_t REG_860H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_860H_b;
    };
    union
    {
        __IOM uint32_t REG_864H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_864H_b;
    };
    union
    {
        __IOM uint32_t REG_868H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_868H_b;
    };
    union
    {
        __IOM uint32_t REG_86CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_86CH_b;
    };
    union
    {
        __IOM uint32_t REG_870H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_870H_b;
    };
    union
    {
        __IOM uint32_t REG_874H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_874H_b;
    };
    union
    {
        __IOM uint32_t REG_878H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_878H_b;
    };
    union
    {
        __IOM uint32_t REG_87CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_87CH_b;
    };
    union
    {
        __IOM uint32_t REG_880H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_880H_b;
    };
    union
    {
        __IOM uint32_t REG_884H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_884H_b;
    };
    union
    {
        __IOM uint32_t REG_888H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_888H_b;
    };
    union
    {
        __IOM uint32_t REG_88CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_88CH_b;
    };
    union
    {
        __IOM uint32_t REG_890H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_890H_b;
    };
    union
    {
        __IOM uint32_t REG_894H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_894H_b;
    };
    union
    {
        __IOM uint32_t REG_898H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_898H_b;
    };
    union
    {
        __IOM uint32_t REG_89CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_89CH_b;
    };
    union
    {
        __IOM uint32_t REG_8A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8A0H_b;
    };
    union
    {
        __IOM uint32_t REG_8A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8A4H_b;
    };
    union
    {
        __IOM uint32_t REG_8A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8A8H_b;
    };
    union
    {
        __IOM uint32_t REG_8ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8ACH_b;
    };
    union
    {
        __IOM uint32_t REG_8B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8B0H_b;
    };
    union
    {
        __IOM uint32_t REG_8B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8B4H_b;
    };
    union
    {
        __IOM uint32_t REG_8B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8B8H_b;
    };
    union
    {
        __IOM uint32_t REG_8BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8BCH_b;
    };
    union
    {
        __IOM uint32_t REG_8C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8C0H_b;
    };
    union
    {
        __IOM uint32_t REG_8C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8C4H_b;
    };
    union
    {
        __IOM uint32_t REG_8C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8C8H_b;
    };
    union
    {
        __IOM uint32_t REG_8CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8CCH_b;
    };
    union
    {
        __IOM uint32_t REG_8D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8D0H_b;
    };
    union
    {
        __IOM uint32_t REG_8D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8D4H_b;
    };
    union
    {
        __IOM uint32_t REG_8D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8D8H_b;
    };
    union
    {
        __IOM uint32_t REG_8DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8DCH_b;
    };
    union
    {
        __IOM uint32_t REG_8E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8E0H_b;
    };
    union
    {
        __IOM uint32_t REG_8E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8E4H_b;
    };
    union
    {
        __IOM uint32_t REG_8E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8E8H_b;
    };
    union
    {
        __IOM uint32_t REG_8ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8ECH_b;
    };
    union
    {
        __IOM uint32_t REG_8F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8F0H_b;
    };
    union
    {
        __IOM uint32_t REG_8F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8F4H_b;
    };
    union
    {
        __IOM uint32_t REG_8F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8F8H_b;
    };
    union
    {
        __IOM uint32_t REG_8FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_8FCH_b;
    };
    union
    {
        __IOM uint32_t REG_900H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_900H_b;
    };
    union
    {
        __IOM uint32_t REG_904H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_904H_b;
    };
    union
    {
        __IOM uint32_t REG_908H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_908H_b;
    };
    union
    {
        __IOM uint32_t REG_90CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_90CH_b;
    };
    union
    {
        __IOM uint32_t REG_910H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_910H_b;
    };
    union
    {
        __IOM uint32_t REG_914H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_914H_b;
    };
    union
    {
        __IOM uint32_t REG_918H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_918H_b;
    };
    union
    {
        __IOM uint32_t REG_91CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_91CH_b;
    };
    union
    {
        __IOM uint32_t REG_920H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_920H_b;
    };
    union
    {
        __IOM uint32_t REG_924H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_924H_b;
    };
    union
    {
        __IOM uint32_t REG_928H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_928H_b;
    };
    union
    {
        __IOM uint32_t REG_92CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_92CH_b;
    };
    union
    {
        __IOM uint32_t REG_930H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_930H_b;
    };
    union
    {
        __IOM uint32_t REG_934H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_934H_b;
    };
    union
    {
        __IOM uint32_t REG_938H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_938H_b;
    };
    union
    {
        __IOM uint32_t REG_93CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_93CH_b;
    };
    union
    {
        __IOM uint32_t REG_940H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_940H_b;
    };
    union
    {
        __IOM uint32_t REG_944H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_944H_b;
    };
    union
    {
        __IOM uint32_t REG_948H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_948H_b;
    };
    union
    {
        __IOM uint32_t REG_94CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_94CH_b;
    };
    union
    {
        __IOM uint32_t REG_950H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_950H_b;
    };
    union
    {
        __IOM uint32_t REG_954H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_954H_b;
    };
    union
    {
        __IOM uint32_t REG_958H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_958H_b;
    };
    union
    {
        __IOM uint32_t REG_95CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_95CH_b;
    };
    union
    {
        __IOM uint32_t REG_960H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_960H_b;
    };
    union
    {
        __IOM uint32_t REG_964H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_964H_b;
    };
    union
    {
        __IOM uint32_t REG_968H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_968H_b;
    };
    union
    {
        __IOM uint32_t REG_96CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_96CH_b;
    };
    union
    {
        __IOM uint32_t REG_970H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_970H_b;
    };
    union
    {
        __IOM uint32_t REG_974H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_974H_b;
    };
    union
    {
        __IOM uint32_t REG_978H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_978H_b;
    };
    union
    {
        __IOM uint32_t REG_97CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_97CH_b;
    };
    union
    {
        __IOM uint32_t REG_980H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_980H_b;
    };
    union
    {
        __IOM uint32_t REG_984H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_984H_b;
    };
    union
    {
        __IOM uint32_t REG_988H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_988H_b;
    };
    union
    {
        __IOM uint32_t REG_98CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_98CH_b;
    };
    union
    {
        __IOM uint32_t REG_990H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_990H_b;
    };
    union
    {
        __IOM uint32_t REG_994H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_994H_b;
    };
    union
    {
        __IOM uint32_t REG_998H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_998H_b;
    };
    union
    {
        __IOM uint32_t REG_99CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_99CH_b;
    };
    union
    {
        __IOM uint32_t REG_9A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9A0H_b;
    };
    union
    {
        __IOM uint32_t REG_9A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9A4H_b;
    };
    union
    {
        __IOM uint32_t REG_9A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9A8H_b;
    };
    union
    {
        __IOM uint32_t REG_9ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9ACH_b;
    };
    union
    {
        __IOM uint32_t REG_9B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9B0H_b;
    };
    union
    {
        __IOM uint32_t REG_9B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9B4H_b;
    };
    union
    {
        __IOM uint32_t REG_9B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9B8H_b;
    };
    union
    {
        __IOM uint32_t REG_9BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9BCH_b;
    };
    union
    {
        __IOM uint32_t REG_9C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9C0H_b;
    };
    union
    {
        __IOM uint32_t REG_9C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9C4H_b;
    };
    union
    {
        __IOM uint32_t REG_9C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9C8H_b;
    };
    union
    {
        __IOM uint32_t REG_9CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9CCH_b;
    };
    union
    {
        __IOM uint32_t REG_9D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9D0H_b;
    };
    union
    {
        __IOM uint32_t REG_9D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9D4H_b;
    };
    union
    {
        __IOM uint32_t REG_9D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9D8H_b;
    };
    union
    {
        __IOM uint32_t REG_9DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9DCH_b;
    };
    union
    {
        __IOM uint32_t REG_9E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9E0H_b;
    };
    union
    {
        __IOM uint32_t REG_9E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9E4H_b;
    };
    union
    {
        __IOM uint32_t REG_9E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9E8H_b;
    };
    union
    {
        __IOM uint32_t REG_9ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9ECH_b;
    };
    union
    {
        __IOM uint32_t REG_9F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9F0H_b;
    };
    union
    {
        __IOM uint32_t REG_9F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9F4H_b;
    };
    union
    {
        __IOM uint32_t REG_9F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9F8H_b;
    };
    union
    {
        __IOM uint32_t REG_9FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_9FCH_b;
    };
    union
    {
        __IOM uint32_t REG_A00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A00H_b;
    };
    union
    {
        __IOM uint32_t REG_A04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A04H_b;
    };
    union
    {
        __IOM uint32_t REG_A08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A08H_b;
    };
    union
    {
        __IOM uint32_t REG_A0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A0CH_b;
    };
    union
    {
        __IOM uint32_t REG_A10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A10H_b;
    };
    union
    {
        __IOM uint32_t REG_A14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A14H_b;
    };
    union
    {
        __IOM uint32_t REG_A18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A18H_b;
    };
    union
    {
        __IOM uint32_t REG_A1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A1CH_b;
    };
    union
    {
        __IOM uint32_t REG_A20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A20H_b;
    };
    union
    {
        __IOM uint32_t REG_A24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A24H_b;
    };
    union
    {
        __IOM uint32_t REG_A28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A28H_b;
    };
    union
    {
        __IOM uint32_t REG_A2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A2CH_b;
    };
    union
    {
        __IOM uint32_t REG_A30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A30H_b;
    };
    union
    {
        __IOM uint32_t REG_A34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A34H_b;
    };
    union
    {
        __IOM uint32_t REG_A38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A38H_b;
    };
    union
    {
        __IOM uint32_t REG_A3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A3CH_b;
    };
    union
    {
        __IOM uint32_t REG_A40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A40H_b;
    };
    union
    {
        __IOM uint32_t REG_A44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A44H_b;
    };
    union
    {
        __IOM uint32_t REG_A48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A48H_b;
    };
    union
    {
        __IOM uint32_t REG_A4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A4CH_b;
    };
    union
    {
        __IOM uint32_t REG_A50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A50H_b;
    };
    union
    {
        __IOM uint32_t REG_A54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A54H_b;
    };
    union
    {
        __IOM uint32_t REG_A58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A58H_b;
    };
    union
    {
        __IOM uint32_t REG_A5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A5CH_b;
    };
    union
    {
        __IOM uint32_t REG_A60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A60H_b;
    };
    union
    {
        __IOM uint32_t REG_A64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A64H_b;
    };
    union
    {
        __IOM uint32_t REG_A68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A68H_b;
    };
    union
    {
        __IOM uint32_t REG_A6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A6CH_b;
    };
    union
    {
        __IOM uint32_t REG_A70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A70H_b;
    };
    union
    {
        __IOM uint32_t REG_A74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A74H_b;
    };
    union
    {
        __IOM uint32_t REG_A78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A78H_b;
    };
    union
    {
        __IOM uint32_t REG_A7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A7CH_b;
    };
    union
    {
        __IOM uint32_t REG_A80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A80H_b;
    };
    union
    {
        __IOM uint32_t REG_A84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A84H_b;
    };
    union
    {
        __IOM uint32_t REG_A88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A88H_b;
    };
    union
    {
        __IOM uint32_t REG_A8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A8CH_b;
    };
    union
    {
        __IOM uint32_t REG_A90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A90H_b;
    };
    union
    {
        __IOM uint32_t REG_A94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A94H_b;
    };
    union
    {
        __IOM uint32_t REG_A98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A98H_b;
    };
    union
    {
        __IOM uint32_t REG_A9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_A9CH_b;
    };
    union
    {
        __IOM uint32_t REG_AA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AA0H_b;
    };
    union
    {
        __IOM uint32_t REG_AA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AA4H_b;
    };
    union
    {
        __IOM uint32_t REG_AA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AA8H_b;
    };
    union
    {
        __IOM uint32_t REG_AACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AACH_b;
    };
    union
    {
        __IOM uint32_t REG_AB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AB0H_b;
    };
    union
    {
        __IOM uint32_t REG_AB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AB4H_b;
    };
    union
    {
        __IOM uint32_t REG_AB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AB8H_b;
    };
    union
    {
        __IOM uint32_t REG_ABCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ABCH_b;
    };
    union
    {
        __IOM uint32_t REG_AC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AC0H_b;
    };
    union
    {
        __IOM uint32_t REG_AC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AC4H_b;
    };
    union
    {
        __IOM uint32_t REG_AC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AC8H_b;
    };
    union
    {
        __IOM uint32_t REG_ACCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ACCH_b;
    };
    union
    {
        __IOM uint32_t REG_AD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AD0H_b;
    };
    union
    {
        __IOM uint32_t REG_AD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AD4H_b;
    };
    union
    {
        __IOM uint32_t REG_AD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AD8H_b;
    };
    union
    {
        __IOM uint32_t REG_ADCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ADCH_b;
    };
    union
    {
        __IOM uint32_t REG_AE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AE0H_b;
    };
    union
    {
        __IOM uint32_t REG_AE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AE4H_b;
    };
    union
    {
        __IOM uint32_t REG_AE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AE8H_b;
    };
    union
    {
        __IOM uint32_t REG_AECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AECH_b;
    };
    union
    {
        __IOM uint32_t REG_AF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AF0H_b;
    };
    union
    {
        __IOM uint32_t REG_AF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AF4H_b;
    };
    union
    {
        __IOM uint32_t REG_AF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AF8H_b;
    };
    union
    {
        __IOM uint32_t REG_AFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_AFCH_b;
    };
    union
    {
        __IOM uint32_t REG_B00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B00H_b;
    };
    union
    {
        __IOM uint32_t REG_B04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B04H_b;
    };
    union
    {
        __IOM uint32_t REG_B08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B08H_b;
    };
    union
    {
        __IOM uint32_t REG_B0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B0CH_b;
    };
    union
    {
        __IOM uint32_t REG_B10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B10H_b;
    };
    union
    {
        __IOM uint32_t REG_B14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B14H_b;
    };
    union
    {
        __IOM uint32_t REG_B18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B18H_b;
    };
    union
    {
        __IOM uint32_t REG_B1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B1CH_b;
    };
    union
    {
        __IOM uint32_t REG_B20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B20H_b;
    };
    union
    {
        __IOM uint32_t REG_B24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B24H_b;
    };
    union
    {
        __IOM uint32_t REG_B28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B28H_b;
    };
    union
    {
        __IOM uint32_t REG_B2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B2CH_b;
    };
    union
    {
        __IOM uint32_t REG_B30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B30H_b;
    };
    union
    {
        __IOM uint32_t REG_B34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B34H_b;
    };
    union
    {
        __IOM uint32_t REG_B38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B38H_b;
    };
    union
    {
        __IOM uint32_t REG_B3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B3CH_b;
    };
    union
    {
        __IOM uint32_t REG_B40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B40H_b;
    };
    union
    {
        __IOM uint32_t REG_B44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B44H_b;
    };
    union
    {
        __IOM uint32_t REG_B48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B48H_b;
    };
    union
    {
        __IOM uint32_t REG_B4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B4CH_b;
    };
    union
    {
        __IOM uint32_t REG_B50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B50H_b;
    };
    union
    {
        __IOM uint32_t REG_B54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B54H_b;
    };
    union
    {
        __IOM uint32_t REG_B58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B58H_b;
    };
    union
    {
        __IOM uint32_t REG_B5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B5CH_b;
    };
    union
    {
        __IOM uint32_t REG_B60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B60H_b;
    };
    union
    {
        __IOM uint32_t REG_B64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B64H_b;
    };
    union
    {
        __IOM uint32_t REG_B68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B68H_b;
    };
    union
    {
        __IOM uint32_t REG_B6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B6CH_b;
    };
    union
    {
        __IOM uint32_t REG_B70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B70H_b;
    };
    union
    {
        __IOM uint32_t REG_B74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B74H_b;
    };
    union
    {
        __IOM uint32_t REG_B78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B78H_b;
    };
    union
    {
        __IOM uint32_t REG_B7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B7CH_b;
    };
    union
    {
        __IOM uint32_t REG_B80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B80H_b;
    };
    union
    {
        __IOM uint32_t REG_B84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B84H_b;
    };
    union
    {
        __IOM uint32_t REG_B88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B88H_b;
    };
    union
    {
        __IOM uint32_t REG_B8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B8CH_b;
    };
    union
    {
        __IOM uint32_t REG_B90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B90H_b;
    };
    union
    {
        __IOM uint32_t REG_B94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B94H_b;
    };
    union
    {
        __IOM uint32_t REG_B98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B98H_b;
    };
    union
    {
        __IOM uint32_t REG_B9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_B9CH_b;
    };
    union
    {
        __IOM uint32_t REG_BA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BA0H_b;
    };
    union
    {
        __IOM uint32_t REG_BA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BA4H_b;
    };
    union
    {
        __IOM uint32_t REG_BA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BA8H_b;
    };
    union
    {
        __IOM uint32_t REG_BACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BACH_b;
    };
    union
    {
        __IOM uint32_t REG_BB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BB0H_b;
    };
    union
    {
        __IOM uint32_t REG_BB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BB4H_b;
    };
    union
    {
        __IOM uint32_t REG_BB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BB8H_b;
    };
    union
    {
        __IOM uint32_t REG_BBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BBCH_b;
    };
    union
    {
        __IOM uint32_t REG_BC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BC0H_b;
    };
    union
    {
        __IOM uint32_t REG_BC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BC4H_b;
    };
    union
    {
        __IOM uint32_t REG_BC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BC8H_b;
    };
    union
    {
        __IOM uint32_t REG_BCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BCCH_b;
    };
    union
    {
        __IOM uint32_t REG_BD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BD0H_b;
    };
    union
    {
        __IOM uint32_t REG_BD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BD4H_b;
    };
    union
    {
        __IOM uint32_t REG_BD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BD8H_b;
    };
    union
    {
        __IOM uint32_t REG_BDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BDCH_b;
    };
    union
    {
        __IOM uint32_t REG_BE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BE0H_b;
    };
    union
    {
        __IOM uint32_t REG_BE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BE4H_b;
    };
    union
    {
        __IOM uint32_t REG_BE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BE8H_b;
    };
    union
    {
        __IOM uint32_t REG_BECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BECH_b;
    };
    union
    {
        __IOM uint32_t REG_BF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BF0H_b;
    };
    union
    {
        __IOM uint32_t REG_BF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BF4H_b;
    };
    union
    {
        __IOM uint32_t REG_BF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BF8H_b;
    };
    union
    {
        __IOM uint32_t REG_BFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_BFCH_b;
    };
    union
    {
        __IOM uint32_t REG_C00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C00H_b;
    };
    union
    {
        __IOM uint32_t REG_C04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C04H_b;
    };
    union
    {
        __IOM uint32_t REG_C08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C08H_b;
    };
    union
    {
        __IOM uint32_t REG_C0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C0CH_b;
    };
    union
    {
        __IOM uint32_t REG_C10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C10H_b;
    };
    union
    {
        __IOM uint32_t REG_C14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C14H_b;
    };
    union
    {
        __IOM uint32_t REG_C18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C18H_b;
    };
    union
    {
        __IOM uint32_t REG_C1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C1CH_b;
    };
    union
    {
        __IOM uint32_t REG_C20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C20H_b;
    };
    union
    {
        __IOM uint32_t REG_C24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C24H_b;
    };
    union
    {
        __IOM uint32_t REG_C28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C28H_b;
    };
    union
    {
        __IOM uint32_t REG_C2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C2CH_b;
    };
    union
    {
        __IOM uint32_t REG_C30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C30H_b;
    };
    union
    {
        __IOM uint32_t REG_C34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C34H_b;
    };
    union
    {
        __IOM uint32_t REG_C38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C38H_b;
    };
    union
    {
        __IOM uint32_t REG_C3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C3CH_b;
    };
    union
    {
        __IOM uint32_t REG_C40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C40H_b;
    };
    union
    {
        __IOM uint32_t REG_C44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C44H_b;
    };
    union
    {
        __IOM uint32_t REG_C48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C48H_b;
    };
    union
    {
        __IOM uint32_t REG_C4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C4CH_b;
    };
    union
    {
        __IOM uint32_t REG_C50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C50H_b;
    };
    union
    {
        __IOM uint32_t REG_C54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C54H_b;
    };
    union
    {
        __IOM uint32_t REG_C58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C58H_b;
    };
    union
    {
        __IOM uint32_t REG_C5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C5CH_b;
    };
    union
    {
        __IOM uint32_t REG_C60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C60H_b;
    };
    union
    {
        __IOM uint32_t REG_C64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C64H_b;
    };
    union
    {
        __IOM uint32_t REG_C68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C68H_b;
    };
    union
    {
        __IOM uint32_t REG_C6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C6CH_b;
    };
    union
    {
        __IOM uint32_t REG_C70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C70H_b;
    };
    union
    {
        __IOM uint32_t REG_C74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C74H_b;
    };
    union
    {
        __IOM uint32_t REG_C78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C78H_b;
    };
    union
    {
        __IOM uint32_t REG_C7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C7CH_b;
    };
    union
    {
        __IOM uint32_t REG_C80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C80H_b;
    };
    union
    {
        __IOM uint32_t REG_C84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C84H_b;
    };
    union
    {
        __IOM uint32_t REG_C88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C88H_b;
    };
    union
    {
        __IOM uint32_t REG_C8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C8CH_b;
    };
    union
    {
        __IOM uint32_t REG_C90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C90H_b;
    };
    union
    {
        __IOM uint32_t REG_C94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C94H_b;
    };
    union
    {
        __IOM uint32_t REG_C98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C98H_b;
    };
    union
    {
        __IOM uint32_t REG_C9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_C9CH_b;
    };
    union
    {
        __IOM uint32_t REG_CA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CA0H_b;
    };
    union
    {
        __IOM uint32_t REG_CA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CA4H_b;
    };
    union
    {
        __IOM uint32_t REG_CA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CA8H_b;
    };
    union
    {
        __IOM uint32_t REG_CACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CACH_b;
    };
    union
    {
        __IOM uint32_t REG_CB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CB0H_b;
    };
    union
    {
        __IOM uint32_t REG_CB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CB4H_b;
    };
    union
    {
        __IOM uint32_t REG_CB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CB8H_b;
    };
    union
    {
        __IOM uint32_t REG_CBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CBCH_b;
    };
    union
    {
        __IOM uint32_t REG_CC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CC0H_b;
    };
    union
    {
        __IOM uint32_t REG_CC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CC4H_b;
    };
    union
    {
        __IOM uint32_t REG_CC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CC8H_b;
    };
    union
    {
        __IOM uint32_t REG_CCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CCCH_b;
    };
    union
    {
        __IOM uint32_t REG_CD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CD0H_b;
    };
    union
    {
        __IOM uint32_t REG_CD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CD4H_b;
    };
    union
    {
        __IOM uint32_t REG_CD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CD8H_b;
    };
    union
    {
        __IOM uint32_t REG_CDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CDCH_b;
    };
    union
    {
        __IOM uint32_t REG_CE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CE0H_b;
    };
    union
    {
        __IOM uint32_t REG_CE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CE4H_b;
    };
    union
    {
        __IOM uint32_t REG_CE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CE8H_b;
    };
    union
    {
        __IOM uint32_t REG_CECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CECH_b;
    };
    union
    {
        __IOM uint32_t REG_CF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CF0H_b;
    };
    union
    {
        __IOM uint32_t REG_CF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CF4H_b;
    };
    union
    {
        __IOM uint32_t REG_CF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CF8H_b;
    };
    union
    {
        __IOM uint32_t REG_CFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_CFCH_b;
    };
    union
    {
        __IOM uint32_t REG_D00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D00H_b;
    };
    union
    {
        __IOM uint32_t REG_D04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D04H_b;
    };
    union
    {
        __IOM uint32_t REG_D08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D08H_b;
    };
    union
    {
        __IOM uint32_t REG_D0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D0CH_b;
    };
    union
    {
        __IOM uint32_t REG_D10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D10H_b;
    };
    union
    {
        __IOM uint32_t REG_D14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D14H_b;
    };
    union
    {
        __IOM uint32_t REG_D18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D18H_b;
    };
    union
    {
        __IOM uint32_t REG_D1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D1CH_b;
    };
    union
    {
        __IOM uint32_t REG_D20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D20H_b;
    };
    union
    {
        __IOM uint32_t REG_D24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D24H_b;
    };
    union
    {
        __IOM uint32_t REG_D28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D28H_b;
    };
    union
    {
        __IOM uint32_t REG_D2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D2CH_b;
    };
    union
    {
        __IOM uint32_t REG_D30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D30H_b;
    };
    union
    {
        __IOM uint32_t REG_D34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D34H_b;
    };
    union
    {
        __IOM uint32_t REG_D38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D38H_b;
    };
    union
    {
        __IOM uint32_t REG_D3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D3CH_b;
    };
    union
    {
        __IOM uint32_t REG_D40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D40H_b;
    };
    union
    {
        __IOM uint32_t REG_D44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D44H_b;
    };
    union
    {
        __IOM uint32_t REG_D48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D48H_b;
    };
    union
    {
        __IOM uint32_t REG_D4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D4CH_b;
    };
    union
    {
        __IOM uint32_t REG_D50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D50H_b;
    };
    union
    {
        __IOM uint32_t REG_D54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D54H_b;
    };
    union
    {
        __IOM uint32_t REG_D58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D58H_b;
    };
    union
    {
        __IOM uint32_t REG_D5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D5CH_b;
    };
    union
    {
        __IOM uint32_t REG_D60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D60H_b;
    };
    union
    {
        __IOM uint32_t REG_D64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D64H_b;
    };
    union
    {
        __IOM uint32_t REG_D68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D68H_b;
    };
    union
    {
        __IOM uint32_t REG_D6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D6CH_b;
    };
    union
    {
        __IOM uint32_t REG_D70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D70H_b;
    };
    union
    {
        __IOM uint32_t REG_D74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D74H_b;
    };
    union
    {
        __IOM uint32_t REG_D78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D78H_b;
    };
    union
    {
        __IOM uint32_t REG_D7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D7CH_b;
    };
    union
    {
        __IOM uint32_t REG_D80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D80H_b;
    };
    union
    {
        __IOM uint32_t REG_D84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D84H_b;
    };
    union
    {
        __IOM uint32_t REG_D88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D88H_b;
    };
    union
    {
        __IOM uint32_t REG_D8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D8CH_b;
    };
    union
    {
        __IOM uint32_t REG_D90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D90H_b;
    };
    union
    {
        __IOM uint32_t REG_D94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D94H_b;
    };
    union
    {
        __IOM uint32_t REG_D98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D98H_b;
    };
    union
    {
        __IOM uint32_t REG_D9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_D9CH_b;
    };
    union
    {
        __IOM uint32_t REG_DA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DA0H_b;
    };
    union
    {
        __IOM uint32_t REG_DA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DA4H_b;
    };
    union
    {
        __IOM uint32_t REG_DA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DA8H_b;
    };
    union
    {
        __IOM uint32_t REG_DACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DACH_b;
    };
    union
    {
        __IOM uint32_t REG_DB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DB0H_b;
    };
    union
    {
        __IOM uint32_t REG_DB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DB4H_b;
    };
    union
    {
        __IOM uint32_t REG_DB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DB8H_b;
    };
    union
    {
        __IOM uint32_t REG_DBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DBCH_b;
    };
    union
    {
        __IOM uint32_t REG_DC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DC0H_b;
    };
    union
    {
        __IOM uint32_t REG_DC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DC4H_b;
    };
    union
    {
        __IOM uint32_t REG_DC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DC8H_b;
    };
    union
    {
        __IOM uint32_t REG_DCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DCCH_b;
    };
    union
    {
        __IOM uint32_t REG_DD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DD0H_b;
    };
    union
    {
        __IOM uint32_t REG_DD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DD4H_b;
    };
    union
    {
        __IOM uint32_t REG_DD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DD8H_b;
    };
    union
    {
        __IOM uint32_t REG_DDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DDCH_b;
    };
    union
    {
        __IOM uint32_t REG_DE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DE0H_b;
    };
    union
    {
        __IOM uint32_t REG_DE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DE4H_b;
    };
    union
    {
        __IOM uint32_t REG_DE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DE8H_b;
    };
    union
    {
        __IOM uint32_t REG_DECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DECH_b;
    };
    union
    {
        __IOM uint32_t REG_DF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DF0H_b;
    };
    union
    {
        __IOM uint32_t REG_DF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DF4H_b;
    };
    union
    {
        __IOM uint32_t REG_DF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DF8H_b;
    };
    union
    {
        __IOM uint32_t REG_DFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_DFCH_b;
    };
    union
    {
        __IOM uint32_t REG_E00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E00H_b;
    };
    union
    {
        __IOM uint32_t REG_E04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E04H_b;
    };
    union
    {
        __IOM uint32_t REG_E08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E08H_b;
    };
    union
    {
        __IOM uint32_t REG_E0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E0CH_b;
    };
    union
    {
        __IOM uint32_t REG_E10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E10H_b;
    };
    union
    {
        __IOM uint32_t REG_E14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E14H_b;
    };
    union
    {
        __IOM uint32_t REG_E18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E18H_b;
    };
    union
    {
        __IOM uint32_t REG_E1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E1CH_b;
    };
    union
    {
        __IOM uint32_t REG_E20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E20H_b;
    };
    union
    {
        __IOM uint32_t REG_E24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E24H_b;
    };
    union
    {
        __IOM uint32_t REG_E28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E28H_b;
    };
    union
    {
        __IOM uint32_t REG_E2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E2CH_b;
    };
    union
    {
        __IOM uint32_t REG_E30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E30H_b;
    };
    union
    {
        __IOM uint32_t REG_E34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E34H_b;
    };
    union
    {
        __IOM uint32_t REG_E38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E38H_b;
    };
    union
    {
        __IOM uint32_t REG_E3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E3CH_b;
    };
    union
    {
        __IOM uint32_t REG_E40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E40H_b;
    };
    union
    {
        __IOM uint32_t REG_E44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E44H_b;
    };
    union
    {
        __IOM uint32_t REG_E48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E48H_b;
    };
    union
    {
        __IOM uint32_t REG_E4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E4CH_b;
    };
    union
    {
        __IOM uint32_t REG_E50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E50H_b;
    };
    union
    {
        __IOM uint32_t REG_E54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E54H_b;
    };
    union
    {
        __IOM uint32_t REG_E58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E58H_b;
    };
    union
    {
        __IOM uint32_t REG_E5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E5CH_b;
    };
    union
    {
        __IOM uint32_t REG_E60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E60H_b;
    };
    union
    {
        __IOM uint32_t REG_E64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E64H_b;
    };
    union
    {
        __IOM uint32_t REG_E68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E68H_b;
    };
    union
    {
        __IOM uint32_t REG_E6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E6CH_b;
    };
    union
    {
        __IOM uint32_t REG_E70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E70H_b;
    };
    union
    {
        __IOM uint32_t REG_E74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E74H_b;
    };
    union
    {
        __IOM uint32_t REG_E78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E78H_b;
    };
    union
    {
        __IOM uint32_t REG_E7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E7CH_b;
    };
    union
    {
        __IOM uint32_t REG_E80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E80H_b;
    };
    union
    {
        __IOM uint32_t REG_E84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E84H_b;
    };
    union
    {
        __IOM uint32_t REG_E88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E88H_b;
    };
    union
    {
        __IOM uint32_t REG_E8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E8CH_b;
    };
    union
    {
        __IOM uint32_t REG_E90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E90H_b;
    };
    union
    {
        __IOM uint32_t REG_E94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E94H_b;
    };
    union
    {
        __IOM uint32_t REG_E98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E98H_b;
    };
    union
    {
        __IOM uint32_t REG_E9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_E9CH_b;
    };
    union
    {
        __IOM uint32_t REG_EA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EA0H_b;
    };
    union
    {
        __IOM uint32_t REG_EA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EA4H_b;
    };
    union
    {
        __IOM uint32_t REG_EA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EA8H_b;
    };
    union
    {
        __IOM uint32_t REG_EACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EACH_b;
    };
    union
    {
        __IOM uint32_t REG_EB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EB0H_b;
    };
    union
    {
        __IOM uint32_t REG_EB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EB4H_b;
    };
    union
    {
        __IOM uint32_t REG_EB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EB8H_b;
    };
    union
    {
        __IOM uint32_t REG_EBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EBCH_b;
    };
    union
    {
        __IOM uint32_t REG_EC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EC0H_b;
    };
    union
    {
        __IOM uint32_t REG_EC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EC4H_b;
    };
    union
    {
        __IOM uint32_t REG_EC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EC8H_b;
    };
    union
    {
        __IOM uint32_t REG_ECCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ECCH_b;
    };
    union
    {
        __IOM uint32_t REG_ED0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ED0H_b;
    };
    union
    {
        __IOM uint32_t REG_ED4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ED4H_b;
    };
    union
    {
        __IOM uint32_t REG_ED8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_ED8H_b;
    };
    union
    {
        __IOM uint32_t REG_EDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EDCH_b;
    };
    union
    {
        __IOM uint32_t REG_EE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EE0H_b;
    };
    union
    {
        __IOM uint32_t REG_EE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EE4H_b;
    };
    union
    {
        __IOM uint32_t REG_EE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EE8H_b;
    };
    union
    {
        __IOM uint32_t REG_EECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EECH_b;
    };
    union
    {
        __IOM uint32_t REG_EF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EF0H_b;
    };
    union
    {
        __IOM uint32_t REG_EF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EF4H_b;
    };
    union
    {
        __IOM uint32_t REG_EF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EF8H_b;
    };
    union
    {
        __IOM uint32_t REG_EFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_EFCH_b;
    };
    union
    {
        __IOM uint32_t REG_F00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F00H_b;
    };
    union
    {
        __IOM uint32_t REG_F04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F04H_b;
    };
    union
    {
        __IOM uint32_t REG_F08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F08H_b;
    };
    union
    {
        __IOM uint32_t REG_F0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F0CH_b;
    };
    union
    {
        __IOM uint32_t REG_F10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F10H_b;
    };
    union
    {
        __IOM uint32_t REG_F14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F14H_b;
    };
    union
    {
        __IOM uint32_t REG_F18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F18H_b;
    };
    union
    {
        __IOM uint32_t REG_F1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F1CH_b;
    };
    union
    {
        __IOM uint32_t REG_F20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F20H_b;
    };
    union
    {
        __IOM uint32_t REG_F24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F24H_b;
    };
    union
    {
        __IOM uint32_t REG_F28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F28H_b;
    };
    union
    {
        __IOM uint32_t REG_F2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F2CH_b;
    };
    union
    {
        __IOM uint32_t REG_F30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F30H_b;
    };
    union
    {
        __IOM uint32_t REG_F34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F34H_b;
    };
    union
    {
        __IOM uint32_t REG_F38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F38H_b;
    };
    union
    {
        __IOM uint32_t REG_F3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F3CH_b;
    };
    union
    {
        __IOM uint32_t REG_F40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F40H_b;
    };
    union
    {
        __IOM uint32_t REG_F44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F44H_b;
    };
    union
    {
        __IOM uint32_t REG_F48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F48H_b;
    };
    union
    {
        __IOM uint32_t REG_F4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F4CH_b;
    };
    union
    {
        __IOM uint32_t REG_F50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F50H_b;
    };
    union
    {
        __IOM uint32_t REG_F54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F54H_b;
    };
    union
    {
        __IOM uint32_t REG_F58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F58H_b;
    };
    union
    {
        __IOM uint32_t REG_F5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F5CH_b;
    };
    union
    {
        __IOM uint32_t REG_F60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F60H_b;
    };
    union
    {
        __IOM uint32_t REG_F64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F64H_b;
    };
    union
    {
        __IOM uint32_t REG_F68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F68H_b;
    };
    union
    {
        __IOM uint32_t REG_F6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F6CH_b;
    };
    union
    {
        __IOM uint32_t REG_F70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F70H_b;
    };
    union
    {
        __IOM uint32_t REG_F74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F74H_b;
    };
    union
    {
        __IOM uint32_t REG_F78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F78H_b;
    };
    union
    {
        __IOM uint32_t REG_F7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F7CH_b;
    };
    union
    {
        __IOM uint32_t REG_F80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F80H_b;
    };
    union
    {
        __IOM uint32_t REG_F84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F84H_b;
    };
    union
    {
        __IOM uint32_t REG_F88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F88H_b;
    };
    union
    {
        __IOM uint32_t REG_F8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F8CH_b;
    };
    union
    {
        __IOM uint32_t REG_F90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F90H_b;
    };
    union
    {
        __IOM uint32_t REG_F94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F94H_b;
    };
    union
    {
        __IOM uint32_t REG_F98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F98H_b;
    };
    union
    {
        __IOM uint32_t REG_F9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_F9CH_b;
    };
    union
    {
        __IOM uint32_t REG_FA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FA0H_b;
    };
    union
    {
        __IOM uint32_t REG_FA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FA4H_b;
    };
    union
    {
        __IOM uint32_t REG_FA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FA8H_b;
    };
    union
    {
        __IOM uint32_t REG_FACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FACH_b;
    };
    union
    {
        __IOM uint32_t REG_FB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FB0H_b;
    };
    union
    {
        __IOM uint32_t REG_FB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FB4H_b;
    };
    union
    {
        __IOM uint32_t REG_FB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FB8H_b;
    };
    union
    {
        __IOM uint32_t REG_FBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FBCH_b;
    };
    union
    {
        __IOM uint32_t REG_FC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FC0H_b;
    };
    union
    {
        __IOM uint32_t REG_FC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FC4H_b;
    };
    union
    {
        __IOM uint32_t REG_FC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FC8H_b;
    };
    union
    {
        __IOM uint32_t REG_FCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FCCH_b;
    };
    union
    {
        __IOM uint32_t REG_FD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FD0H_b;
    };
    union
    {
        __IOM uint32_t REG_FD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FD4H_b;
    };
    union
    {
        __IOM uint32_t REG_FD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FD8H_b;
    };
    union
    {
        __IOM uint32_t REG_FDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FDCH_b;
    };
    union
    {
        __IOM uint32_t REG_FE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FE0H_b;
    };
    union
    {
        __IOM uint32_t REG_FE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FE4H_b;
    };
    union
    {
        __IOM uint32_t REG_FE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FE8H_b;
    };
    union
    {
        __IOM uint32_t REG_FECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FECH_b;
    };
    union
    {
        __IOM uint32_t REG_FF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FF0H_b;
    };
    union
    {
        __IOM uint32_t REG_FF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FF4H_b;
    };
    union
    {
        __IOM uint32_t REG_FF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FF8H_b;
    };
    union
    {
        __IOM uint32_t REG_FFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_FFCH_b;
    };
    union
    {
        __IOM uint32_t REG_1000H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1000H_b;
    };
    union
    {
        __IOM uint32_t REG_1004H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1004H_b;
    };
    union
    {
        __IOM uint32_t REG_1008H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1008H_b;
    };
    union
    {
        __IOM uint32_t REG_100CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_100CH_b;
    };
    union
    {
        __IOM uint32_t REG_1010H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1010H_b;
    };
    union
    {
        __IOM uint32_t REG_1014H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1014H_b;
    };
    union
    {
        __IOM uint32_t REG_1018H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1018H_b;
    };
    union
    {
        __IOM uint32_t REG_101CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_101CH_b;
    };
    union
    {
        __IOM uint32_t REG_1020H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1020H_b;
    };
    union
    {
        __IOM uint32_t REG_1024H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1024H_b;
    };
    union
    {
        __IOM uint32_t REG_1028H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1028H_b;
    };
    union
    {
        __IOM uint32_t REG_102CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_102CH_b;
    };
    union
    {
        __IOM uint32_t REG_1030H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1030H_b;
    };
    union
    {
        __IOM uint32_t REG_1034H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1034H_b;
    };
    union
    {
        __IOM uint32_t REG_1038H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1038H_b;
    };
    union
    {
        __IOM uint32_t REG_103CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_103CH_b;
    };
    union
    {
        __IOM uint32_t REG_1040H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1040H_b;
    };
    union
    {
        __IOM uint32_t REG_1044H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1044H_b;
    };
    union
    {
        __IOM uint32_t REG_1048H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1048H_b;
    };
    union
    {
        __IOM uint32_t REG_104CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_104CH_b;
    };
    union
    {
        __IOM uint32_t REG_1050H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1050H_b;
    };
    union
    {
        __IOM uint32_t REG_1054H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1054H_b;
    };
    union
    {
        __IOM uint32_t REG_1058H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1058H_b;
    };
    union
    {
        __IOM uint32_t REG_105CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_105CH_b;
    };
    union
    {
        __IOM uint32_t REG_1060H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1060H_b;
    };
    union
    {
        __IOM uint32_t REG_1064H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1064H_b;
    };
    union
    {
        __IOM uint32_t REG_1068H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1068H_b;
    };
    union
    {
        __IOM uint32_t REG_106CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_106CH_b;
    };
    union
    {
        __IOM uint32_t REG_1070H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1070H_b;
    };
    union
    {
        __IOM uint32_t REG_1074H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1074H_b;
    };
    union
    {
        __IOM uint32_t REG_1078H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1078H_b;
    };
    union
    {
        __IOM uint32_t REG_107CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_107CH_b;
    };
    union
    {
        __IOM uint32_t REG_1080H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1080H_b;
    };
    union
    {
        __IOM uint32_t REG_1084H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1084H_b;
    };
    union
    {
        __IOM uint32_t REG_1088H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1088H_b;
    };
    union
    {
        __IOM uint32_t REG_108CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_108CH_b;
    };
    union
    {
        __IOM uint32_t REG_1090H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1090H_b;
    };
    union
    {
        __IOM uint32_t REG_1094H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1094H_b;
    };
    union
    {
        __IOM uint32_t REG_1098H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1098H_b;
    };
    union
    {
        __IOM uint32_t REG_109CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_109CH_b;
    };
    union
    {
        __IOM uint32_t REG_10A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10A0H_b;
    };
    union
    {
        __IOM uint32_t REG_10A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10A4H_b;
    };
    union
    {
        __IOM uint32_t REG_10A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10A8H_b;
    };
    union
    {
        __IOM uint32_t REG_10ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10ACH_b;
    };
    union
    {
        __IOM uint32_t REG_10B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10B0H_b;
    };
    union
    {
        __IOM uint32_t REG_10B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10B4H_b;
    };
    union
    {
        __IOM uint32_t REG_10B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10B8H_b;
    };
    union
    {
        __IOM uint32_t REG_10BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10BCH_b;
    };
    union
    {
        __IOM uint32_t REG_10C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10C0H_b;
    };
    union
    {
        __IOM uint32_t REG_10C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10C4H_b;
    };
    union
    {
        __IOM uint32_t REG_10C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10C8H_b;
    };
    union
    {
        __IOM uint32_t REG_10CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10CCH_b;
    };
    union
    {
        __IOM uint32_t REG_10D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10D0H_b;
    };
    union
    {
        __IOM uint32_t REG_10D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10D4H_b;
    };
    union
    {
        __IOM uint32_t REG_10D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10D8H_b;
    };
    union
    {
        __IOM uint32_t REG_10DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10DCH_b;
    };
    union
    {
        __IOM uint32_t REG_10E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10E0H_b;
    };
    union
    {
        __IOM uint32_t REG_10E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10E4H_b;
    };
    union
    {
        __IOM uint32_t REG_10E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10E8H_b;
    };
    union
    {
        __IOM uint32_t REG_10ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10ECH_b;
    };
    union
    {
        __IOM uint32_t REG_10F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10F0H_b;
    };
    union
    {
        __IOM uint32_t REG_10F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10F4H_b;
    };
    union
    {
        __IOM uint32_t REG_10F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10F8H_b;
    };
    union
    {
        __IOM uint32_t REG_10FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_10FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1100H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1100H_b;
    };
    union
    {
        __IOM uint32_t REG_1104H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1104H_b;
    };
    union
    {
        __IOM uint32_t REG_1108H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1108H_b;
    };
    union
    {
        __IOM uint32_t REG_110CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_110CH_b;
    };
    union
    {
        __IOM uint32_t REG_1110H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1110H_b;
    };
    union
    {
        __IOM uint32_t REG_1114H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1114H_b;
    };
    union
    {
        __IOM uint32_t REG_1118H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1118H_b;
    };
    union
    {
        __IOM uint32_t REG_111CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_111CH_b;
    };
    union
    {
        __IOM uint32_t REG_1120H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1120H_b;
    };
    union
    {
        __IOM uint32_t REG_1124H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1124H_b;
    };
    union
    {
        __IOM uint32_t REG_1128H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1128H_b;
    };
    union
    {
        __IOM uint32_t REG_112CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_112CH_b;
    };
    union
    {
        __IOM uint32_t REG_1130H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1130H_b;
    };
    union
    {
        __IOM uint32_t REG_1134H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1134H_b;
    };
    union
    {
        __IOM uint32_t REG_1138H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1138H_b;
    };
    union
    {
        __IOM uint32_t REG_113CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_113CH_b;
    };
    union
    {
        __IOM uint32_t REG_1140H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1140H_b;
    };
    union
    {
        __IOM uint32_t REG_1144H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1144H_b;
    };
    union
    {
        __IOM uint32_t REG_1148H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1148H_b;
    };
    union
    {
        __IOM uint32_t REG_114CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_114CH_b;
    };
    union
    {
        __IOM uint32_t REG_1150H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1150H_b;
    };
    union
    {
        __IOM uint32_t REG_1154H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1154H_b;
    };
    union
    {
        __IOM uint32_t REG_1158H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1158H_b;
    };
    union
    {
        __IOM uint32_t REG_115CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_115CH_b;
    };
    union
    {
        __IOM uint32_t REG_1160H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1160H_b;
    };
    union
    {
        __IOM uint32_t REG_1164H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1164H_b;
    };
    union
    {
        __IOM uint32_t REG_1168H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1168H_b;
    };
    union
    {
        __IOM uint32_t REG_116CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_116CH_b;
    };
    union
    {
        __IOM uint32_t REG_1170H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1170H_b;
    };
    union
    {
        __IOM uint32_t REG_1174H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1174H_b;
    };
    union
    {
        __IOM uint32_t REG_1178H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1178H_b;
    };
    union
    {
        __IOM uint32_t REG_117CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_117CH_b;
    };
    union
    {
        __IOM uint32_t REG_1180H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1180H_b;
    };
    union
    {
        __IOM uint32_t REG_1184H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1184H_b;
    };
    union
    {
        __IOM uint32_t REG_1188H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1188H_b;
    };
    union
    {
        __IOM uint32_t REG_118CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_118CH_b;
    };
    union
    {
        __IOM uint32_t REG_1190H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1190H_b;
    };
    union
    {
        __IOM uint32_t REG_1194H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1194H_b;
    };
    union
    {
        __IOM uint32_t REG_1198H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1198H_b;
    };
    union
    {
        __IOM uint32_t REG_119CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_119CH_b;
    };
    union
    {
        __IOM uint32_t REG_11A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11A0H_b;
    };
    union
    {
        __IOM uint32_t REG_11A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11A4H_b;
    };
    union
    {
        __IOM uint32_t REG_11A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11A8H_b;
    };
    union
    {
        __IOM uint32_t REG_11ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11ACH_b;
    };
    union
    {
        __IOM uint32_t REG_11B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11B0H_b;
    };
    union
    {
        __IOM uint32_t REG_11B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11B4H_b;
    };
    union
    {
        __IOM uint32_t REG_11B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11B8H_b;
    };
    union
    {
        __IOM uint32_t REG_11BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11BCH_b;
    };
    union
    {
        __IOM uint32_t REG_11C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11C0H_b;
    };
    union
    {
        __IOM uint32_t REG_11C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11C4H_b;
    };
    union
    {
        __IOM uint32_t REG_11C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11C8H_b;
    };
    union
    {
        __IOM uint32_t REG_11CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11CCH_b;
    };
    union
    {
        __IOM uint32_t REG_11D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11D0H_b;
    };
    union
    {
        __IOM uint32_t REG_11D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11D4H_b;
    };
    union
    {
        __IOM uint32_t REG_11D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11D8H_b;
    };
    union
    {
        __IOM uint32_t REG_11DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11DCH_b;
    };
    union
    {
        __IOM uint32_t REG_11E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11E0H_b;
    };
    union
    {
        __IOM uint32_t REG_11E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11E4H_b;
    };
    union
    {
        __IOM uint32_t REG_11E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11E8H_b;
    };
    union
    {
        __IOM uint32_t REG_11ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11ECH_b;
    };
    union
    {
        __IOM uint32_t REG_11F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11F0H_b;
    };
    union
    {
        __IOM uint32_t REG_11F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11F4H_b;
    };
    union
    {
        __IOM uint32_t REG_11F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11F8H_b;
    };
    union
    {
        __IOM uint32_t REG_11FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_11FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1200H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1200H_b;
    };
    union
    {
        __IOM uint32_t REG_1204H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1204H_b;
    };
    union
    {
        __IOM uint32_t REG_1208H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1208H_b;
    };
    union
    {
        __IOM uint32_t REG_120CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_120CH_b;
    };
    union
    {
        __IOM uint32_t REG_1210H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1210H_b;
    };
    union
    {
        __IOM uint32_t REG_1214H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1214H_b;
    };
    union
    {
        __IOM uint32_t REG_1218H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1218H_b;
    };
    union
    {
        __IOM uint32_t REG_121CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_121CH_b;
    };
    union
    {
        __IOM uint32_t REG_1220H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1220H_b;
    };
    union
    {
        __IOM uint32_t REG_1224H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1224H_b;
    };
    union
    {
        __IOM uint32_t REG_1228H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1228H_b;
    };
    union
    {
        __IOM uint32_t REG_122CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_122CH_b;
    };
    union
    {
        __IOM uint32_t REG_1230H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1230H_b;
    };
    union
    {
        __IOM uint32_t REG_1234H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1234H_b;
    };
    union
    {
        __IOM uint32_t REG_1238H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1238H_b;
    };
    union
    {
        __IOM uint32_t REG_123CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_123CH_b;
    };
    union
    {
        __IOM uint32_t REG_1240H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1240H_b;
    };
    union
    {
        __IOM uint32_t REG_1244H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1244H_b;
    };
    union
    {
        __IOM uint32_t REG_1248H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1248H_b;
    };
    union
    {
        __IOM uint32_t REG_124CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_124CH_b;
    };
    union
    {
        __IOM uint32_t REG_1250H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1250H_b;
    };
    union
    {
        __IOM uint32_t REG_1254H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1254H_b;
    };
    union
    {
        __IOM uint32_t REG_1258H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1258H_b;
    };
    union
    {
        __IOM uint32_t REG_125CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_125CH_b;
    };
    union
    {
        __IOM uint32_t REG_1260H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1260H_b;
    };
    union
    {
        __IOM uint32_t REG_1264H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1264H_b;
    };
    union
    {
        __IOM uint32_t REG_1268H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1268H_b;
    };
    union
    {
        __IOM uint32_t REG_126CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_126CH_b;
    };
    union
    {
        __IOM uint32_t REG_1270H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1270H_b;
    };
    union
    {
        __IOM uint32_t REG_1274H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1274H_b;
    };
    union
    {
        __IOM uint32_t REG_1278H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1278H_b;
    };
    union
    {
        __IOM uint32_t REG_127CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_127CH_b;
    };
    union
    {
        __IOM uint32_t REG_1280H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1280H_b;
    };
    union
    {
        __IOM uint32_t REG_1284H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1284H_b;
    };
    union
    {
        __IOM uint32_t REG_1288H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1288H_b;
    };
    union
    {
        __IOM uint32_t REG_128CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_128CH_b;
    };
    union
    {
        __IOM uint32_t REG_1290H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1290H_b;
    };
    union
    {
        __IOM uint32_t REG_1294H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1294H_b;
    };
    union
    {
        __IOM uint32_t REG_1298H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1298H_b;
    };
    union
    {
        __IOM uint32_t REG_129CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_129CH_b;
    };
    union
    {
        __IOM uint32_t REG_12A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12A0H_b;
    };
    union
    {
        __IOM uint32_t REG_12A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12A4H_b;
    };
    union
    {
        __IOM uint32_t REG_12A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12A8H_b;
    };
    union
    {
        __IOM uint32_t REG_12ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12ACH_b;
    };
    union
    {
        __IOM uint32_t REG_12B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12B0H_b;
    };
    union
    {
        __IOM uint32_t REG_12B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12B4H_b;
    };
    union
    {
        __IOM uint32_t REG_12B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12B8H_b;
    };
    union
    {
        __IOM uint32_t REG_12BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12BCH_b;
    };
    union
    {
        __IOM uint32_t REG_12C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12C0H_b;
    };
    union
    {
        __IOM uint32_t REG_12C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12C4H_b;
    };
    union
    {
        __IOM uint32_t REG_12C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12C8H_b;
    };
    union
    {
        __IOM uint32_t REG_12CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12CCH_b;
    };
    union
    {
        __IOM uint32_t REG_12D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12D0H_b;
    };
    union
    {
        __IOM uint32_t REG_12D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12D4H_b;
    };
    union
    {
        __IOM uint32_t REG_12D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12D8H_b;
    };
    union
    {
        __IOM uint32_t REG_12DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12DCH_b;
    };
    union
    {
        __IOM uint32_t REG_12E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12E0H_b;
    };
    union
    {
        __IOM uint32_t REG_12E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12E4H_b;
    };
    union
    {
        __IOM uint32_t REG_12E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12E8H_b;
    };
    union
    {
        __IOM uint32_t REG_12ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12ECH_b;
    };
    union
    {
        __IOM uint32_t REG_12F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12F0H_b;
    };
    union
    {
        __IOM uint32_t REG_12F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12F4H_b;
    };
    union
    {
        __IOM uint32_t REG_12F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12F8H_b;
    };
    union
    {
        __IOM uint32_t REG_12FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_12FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1300H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1300H_b;
    };
    union
    {
        __IOM uint32_t REG_1304H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1304H_b;
    };
    union
    {
        __IOM uint32_t REG_1308H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1308H_b;
    };
    union
    {
        __IOM uint32_t REG_130CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_130CH_b;
    };
    union
    {
        __IOM uint32_t REG_1310H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1310H_b;
    };
    union
    {
        __IOM uint32_t REG_1314H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1314H_b;
    };
    union
    {
        __IOM uint32_t REG_1318H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1318H_b;
    };
    union
    {
        __IOM uint32_t REG_131CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_131CH_b;
    };
    union
    {
        __IOM uint32_t REG_1320H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1320H_b;
    };
    union
    {
        __IOM uint32_t REG_1324H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1324H_b;
    };
    union
    {
        __IOM uint32_t REG_1328H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1328H_b;
    };
    union
    {
        __IOM uint32_t REG_132CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_132CH_b;
    };
    union
    {
        __IOM uint32_t REG_1330H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1330H_b;
    };
    union
    {
        __IOM uint32_t REG_1334H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1334H_b;
    };
    union
    {
        __IOM uint32_t REG_1338H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1338H_b;
    };
    union
    {
        __IOM uint32_t REG_133CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_133CH_b;
    };
    union
    {
        __IOM uint32_t REG_1340H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1340H_b;
    };
    union
    {
        __IOM uint32_t REG_1344H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1344H_b;
    };
    union
    {
        __IOM uint32_t REG_1348H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1348H_b;
    };
    union
    {
        __IOM uint32_t REG_134CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_134CH_b;
    };
    union
    {
        __IOM uint32_t REG_1350H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1350H_b;
    };
    union
    {
        __IOM uint32_t REG_1354H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1354H_b;
    };
    union
    {
        __IOM uint32_t REG_1358H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1358H_b;
    };
    union
    {
        __IOM uint32_t REG_135CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_135CH_b;
    };
    union
    {
        __IOM uint32_t REG_1360H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1360H_b;
    };
    union
    {
        __IOM uint32_t REG_1364H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1364H_b;
    };
    union
    {
        __IOM uint32_t REG_1368H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1368H_b;
    };
    union
    {
        __IOM uint32_t REG_136CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_136CH_b;
    };
    union
    {
        __IOM uint32_t REG_1370H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1370H_b;
    };
    union
    {
        __IOM uint32_t REG_1374H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1374H_b;
    };
    union
    {
        __IOM uint32_t REG_1378H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1378H_b;
    };
    union
    {
        __IOM uint32_t REG_137CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_137CH_b;
    };
    union
    {
        __IOM uint32_t REG_1380H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1380H_b;
    };
    union
    {
        __IOM uint32_t REG_1384H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1384H_b;
    };
    union
    {
        __IOM uint32_t REG_1388H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1388H_b;
    };
    union
    {
        __IOM uint32_t REG_138CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_138CH_b;
    };
    union
    {
        __IOM uint32_t REG_1390H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1390H_b;
    };
    union
    {
        __IOM uint32_t REG_1394H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1394H_b;
    };
    union
    {
        __IOM uint32_t REG_1398H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1398H_b;
    };
    union
    {
        __IOM uint32_t REG_139CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_139CH_b;
    };
    union
    {
        __IOM uint32_t REG_13A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13A0H_b;
    };
    union
    {
        __IOM uint32_t REG_13A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13A4H_b;
    };
    union
    {
        __IOM uint32_t REG_13A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13A8H_b;
    };
    union
    {
        __IOM uint32_t REG_13ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13ACH_b;
    };
    union
    {
        __IOM uint32_t REG_13B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13B0H_b;
    };
    union
    {
        __IOM uint32_t REG_13B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13B4H_b;
    };
    union
    {
        __IOM uint32_t REG_13B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13B8H_b;
    };
    union
    {
        __IOM uint32_t REG_13BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13BCH_b;
    };
    union
    {
        __IOM uint32_t REG_13C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13C0H_b;
    };
    union
    {
        __IOM uint32_t REG_13C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13C4H_b;
    };
    union
    {
        __IOM uint32_t REG_13C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13C8H_b;
    };
    union
    {
        __IOM uint32_t REG_13CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13CCH_b;
    };
    union
    {
        __IOM uint32_t REG_13D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13D0H_b;
    };
    union
    {
        __IOM uint32_t REG_13D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13D4H_b;
    };
    union
    {
        __IOM uint32_t REG_13D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13D8H_b;
    };
    union
    {
        __IOM uint32_t REG_13DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13DCH_b;
    };
    union
    {
        __IOM uint32_t REG_13E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13E0H_b;
    };
    union
    {
        __IOM uint32_t REG_13E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13E4H_b;
    };
    union
    {
        __IOM uint32_t REG_13E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13E8H_b;
    };
    union
    {
        __IOM uint32_t REG_13ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13ECH_b;
    };
    union
    {
        __IOM uint32_t REG_13F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13F0H_b;
    };
    union
    {
        __IOM uint32_t REG_13F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13F4H_b;
    };
    union
    {
        __IOM uint32_t REG_13F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13F8H_b;
    };
    union
    {
        __IOM uint32_t REG_13FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_13FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1400H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1400H_b;
    };
    union
    {
        __IOM uint32_t REG_1404H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1404H_b;
    };
    union
    {
        __IOM uint32_t REG_1408H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1408H_b;
    };
    union
    {
        __IOM uint32_t REG_140CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_140CH_b;
    };
    union
    {
        __IOM uint32_t REG_1410H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1410H_b;
    };
    union
    {
        __IOM uint32_t REG_1414H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1414H_b;
    };
    union
    {
        __IOM uint32_t REG_1418H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1418H_b;
    };
    union
    {
        __IOM uint32_t REG_141CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_141CH_b;
    };
    union
    {
        __IOM uint32_t REG_1420H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1420H_b;
    };
    union
    {
        __IOM uint32_t REG_1424H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1424H_b;
    };
    union
    {
        __IOM uint32_t REG_1428H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1428H_b;
    };
    union
    {
        __IOM uint32_t REG_142CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_142CH_b;
    };
    union
    {
        __IOM uint32_t REG_1430H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1430H_b;
    };
    union
    {
        __IOM uint32_t REG_1434H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1434H_b;
    };
    union
    {
        __IOM uint32_t REG_1438H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1438H_b;
    };
    union
    {
        __IOM uint32_t REG_143CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_143CH_b;
    };
    union
    {
        __IOM uint32_t REG_1440H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1440H_b;
    };
    union
    {
        __IOM uint32_t REG_1444H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1444H_b;
    };
    union
    {
        __IOM uint32_t REG_1448H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1448H_b;
    };
    union
    {
        __IOM uint32_t REG_144CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_144CH_b;
    };
    union
    {
        __IOM uint32_t REG_1450H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1450H_b;
    };
    union
    {
        __IOM uint32_t REG_1454H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1454H_b;
    };
    union
    {
        __IOM uint32_t REG_1458H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1458H_b;
    };
    union
    {
        __IOM uint32_t REG_145CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_145CH_b;
    };
    union
    {
        __IOM uint32_t REG_1460H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1460H_b;
    };
    union
    {
        __IOM uint32_t REG_1464H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1464H_b;
    };
    union
    {
        __IOM uint32_t REG_1468H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1468H_b;
    };
    union
    {
        __IOM uint32_t REG_146CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_146CH_b;
    };
    union
    {
        __IOM uint32_t REG_1470H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1470H_b;
    };
    union
    {
        __IOM uint32_t REG_1474H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1474H_b;
    };
    union
    {
        __IOM uint32_t REG_1478H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1478H_b;
    };
    union
    {
        __IOM uint32_t REG_147CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_147CH_b;
    };
    union
    {
        __IOM uint32_t REG_1480H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1480H_b;
    };
    union
    {
        __IOM uint32_t REG_1484H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1484H_b;
    };
    union
    {
        __IOM uint32_t REG_1488H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1488H_b;
    };
    union
    {
        __IOM uint32_t REG_148CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_148CH_b;
    };
    union
    {
        __IOM uint32_t REG_1490H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1490H_b;
    };
    union
    {
        __IOM uint32_t REG_1494H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1494H_b;
    };
    union
    {
        __IOM uint32_t REG_1498H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1498H_b;
    };
    union
    {
        __IOM uint32_t REG_149CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_149CH_b;
    };
    union
    {
        __IOM uint32_t REG_14A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14A0H_b;
    };
    union
    {
        __IOM uint32_t REG_14A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14A4H_b;
    };
    union
    {
        __IOM uint32_t REG_14A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14A8H_b;
    };
    union
    {
        __IOM uint32_t REG_14ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14ACH_b;
    };
    union
    {
        __IOM uint32_t REG_14B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14B0H_b;
    };
    union
    {
        __IOM uint32_t REG_14B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14B4H_b;
    };
    union
    {
        __IOM uint32_t REG_14B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14B8H_b;
    };
    union
    {
        __IOM uint32_t REG_14BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14BCH_b;
    };
    union
    {
        __IOM uint32_t REG_14C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14C0H_b;
    };
    union
    {
        __IOM uint32_t REG_14C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14C4H_b;
    };
    union
    {
        __IOM uint32_t REG_14C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14C8H_b;
    };
    union
    {
        __IOM uint32_t REG_14CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14CCH_b;
    };
    union
    {
        __IOM uint32_t REG_14D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14D0H_b;
    };
    union
    {
        __IOM uint32_t REG_14D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14D4H_b;
    };
    union
    {
        __IOM uint32_t REG_14D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14D8H_b;
    };
    union
    {
        __IOM uint32_t REG_14DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14DCH_b;
    };
    union
    {
        __IOM uint32_t REG_14E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14E0H_b;
    };
    union
    {
        __IOM uint32_t REG_14E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14E4H_b;
    };
    union
    {
        __IOM uint32_t REG_14E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14E8H_b;
    };
    union
    {
        __IOM uint32_t REG_14ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14ECH_b;
    };
    union
    {
        __IOM uint32_t REG_14F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14F0H_b;
    };
    union
    {
        __IOM uint32_t REG_14F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14F4H_b;
    };
    union
    {
        __IOM uint32_t REG_14F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14F8H_b;
    };
    union
    {
        __IOM uint32_t REG_14FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_14FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1500H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1500H_b;
    };
    union
    {
        __IOM uint32_t REG_1504H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1504H_b;
    };
    union
    {
        __IOM uint32_t REG_1508H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1508H_b;
    };
    union
    {
        __IOM uint32_t REG_150CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_150CH_b;
    };
    union
    {
        __IOM uint32_t REG_1510H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1510H_b;
    };
    union
    {
        __IOM uint32_t REG_1514H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1514H_b;
    };
    union
    {
        __IOM uint32_t REG_1518H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1518H_b;
    };
    union
    {
        __IOM uint32_t REG_151CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_151CH_b;
    };
    union
    {
        __IOM uint32_t REG_1520H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1520H_b;
    };
    union
    {
        __IOM uint32_t REG_1524H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1524H_b;
    };
    union
    {
        __IOM uint32_t REG_1528H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1528H_b;
    };
    union
    {
        __IOM uint32_t REG_152CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_152CH_b;
    };
    union
    {
        __IOM uint32_t REG_1530H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1530H_b;
    };
    union
    {
        __IOM uint32_t REG_1534H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1534H_b;
    };
    union
    {
        __IOM uint32_t REG_1538H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1538H_b;
    };
    union
    {
        __IOM uint32_t REG_153CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_153CH_b;
    };
    union
    {
        __IOM uint32_t REG_1540H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1540H_b;
    };
    union
    {
        __IOM uint32_t REG_1544H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1544H_b;
    };
    union
    {
        __IOM uint32_t REG_1548H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1548H_b;
    };
    union
    {
        __IOM uint32_t REG_154CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_154CH_b;
    };
    union
    {
        __IOM uint32_t REG_1550H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1550H_b;
    };
    union
    {
        __IOM uint32_t REG_1554H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1554H_b;
    };
    union
    {
        __IOM uint32_t REG_1558H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1558H_b;
    };
    union
    {
        __IOM uint32_t REG_155CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_155CH_b;
    };
    union
    {
        __IOM uint32_t REG_1560H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1560H_b;
    };
    union
    {
        __IOM uint32_t REG_1564H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1564H_b;
    };
    union
    {
        __IOM uint32_t REG_1568H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1568H_b;
    };
    union
    {
        __IOM uint32_t REG_156CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_156CH_b;
    };
    union
    {
        __IOM uint32_t REG_1570H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1570H_b;
    };
    union
    {
        __IOM uint32_t REG_1574H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1574H_b;
    };
    union
    {
        __IOM uint32_t REG_1578H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1578H_b;
    };
    union
    {
        __IOM uint32_t REG_157CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_157CH_b;
    };
    union
    {
        __IOM uint32_t REG_1580H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1580H_b;
    };
    union
    {
        __IOM uint32_t REG_1584H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1584H_b;
    };
    union
    {
        __IOM uint32_t REG_1588H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1588H_b;
    };
    union
    {
        __IOM uint32_t REG_158CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_158CH_b;
    };
    union
    {
        __IOM uint32_t REG_1590H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1590H_b;
    };
    union
    {
        __IOM uint32_t REG_1594H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1594H_b;
    };
    union
    {
        __IOM uint32_t REG_1598H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1598H_b;
    };
    union
    {
        __IOM uint32_t REG_159CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_159CH_b;
    };
    union
    {
        __IOM uint32_t REG_15A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15A0H_b;
    };
    union
    {
        __IOM uint32_t REG_15A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15A4H_b;
    };
    union
    {
        __IOM uint32_t REG_15A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15A8H_b;
    };
    union
    {
        __IOM uint32_t REG_15ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15ACH_b;
    };
    union
    {
        __IOM uint32_t REG_15B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15B0H_b;
    };
    union
    {
        __IOM uint32_t REG_15B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15B4H_b;
    };
    union
    {
        __IOM uint32_t REG_15B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15B8H_b;
    };
    union
    {
        __IOM uint32_t REG_15BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15BCH_b;
    };
    union
    {
        __IOM uint32_t REG_15C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15C0H_b;
    };
    union
    {
        __IOM uint32_t REG_15C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15C4H_b;
    };
    union
    {
        __IOM uint32_t REG_15C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15C8H_b;
    };
    union
    {
        __IOM uint32_t REG_15CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15CCH_b;
    };
    union
    {
        __IOM uint32_t REG_15D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15D0H_b;
    };
    union
    {
        __IOM uint32_t REG_15D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15D4H_b;
    };
    union
    {
        __IOM uint32_t REG_15D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15D8H_b;
    };
    union
    {
        __IOM uint32_t REG_15DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15DCH_b;
    };
    union
    {
        __IOM uint32_t REG_15E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15E0H_b;
    };
    union
    {
        __IOM uint32_t REG_15E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15E4H_b;
    };
    union
    {
        __IOM uint32_t REG_15E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15E8H_b;
    };
    union
    {
        __IOM uint32_t REG_15ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15ECH_b;
    };
    union
    {
        __IOM uint32_t REG_15F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15F0H_b;
    };
    union
    {
        __IOM uint32_t REG_15F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15F4H_b;
    };
    union
    {
        __IOM uint32_t REG_15F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15F8H_b;
    };
    union
    {
        __IOM uint32_t REG_15FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_15FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1600H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1600H_b;
    };
    union
    {
        __IOM uint32_t REG_1604H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1604H_b;
    };
    union
    {
        __IOM uint32_t REG_1608H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1608H_b;
    };
    union
    {
        __IOM uint32_t REG_160CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_160CH_b;
    };
    union
    {
        __IOM uint32_t REG_1610H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1610H_b;
    };
    union
    {
        __IOM uint32_t REG_1614H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1614H_b;
    };
    union
    {
        __IOM uint32_t REG_1618H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1618H_b;
    };
    union
    {
        __IOM uint32_t REG_161CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_161CH_b;
    };
    union
    {
        __IOM uint32_t REG_1620H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1620H_b;
    };
    union
    {
        __IOM uint32_t REG_1624H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1624H_b;
    };
    union
    {
        __IOM uint32_t REG_1628H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1628H_b;
    };
    union
    {
        __IOM uint32_t REG_162CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_162CH_b;
    };
    union
    {
        __IOM uint32_t REG_1630H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1630H_b;
    };
    union
    {
        __IOM uint32_t REG_1634H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1634H_b;
    };
    union
    {
        __IOM uint32_t REG_1638H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1638H_b;
    };
    union
    {
        __IOM uint32_t REG_163CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_163CH_b;
    };
    union
    {
        __IOM uint32_t REG_1640H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1640H_b;
    };
    union
    {
        __IOM uint32_t REG_1644H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1644H_b;
    };
    union
    {
        __IOM uint32_t REG_1648H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1648H_b;
    };
    union
    {
        __IOM uint32_t REG_164CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_164CH_b;
    };
    union
    {
        __IOM uint32_t REG_1650H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1650H_b;
    };
    union
    {
        __IOM uint32_t REG_1654H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1654H_b;
    };
    union
    {
        __IOM uint32_t REG_1658H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1658H_b;
    };
    union
    {
        __IOM uint32_t REG_165CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_165CH_b;
    };
    union
    {
        __IOM uint32_t REG_1660H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1660H_b;
    };
    union
    {
        __IOM uint32_t REG_1664H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1664H_b;
    };
    union
    {
        __IOM uint32_t REG_1668H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1668H_b;
    };
    union
    {
        __IOM uint32_t REG_166CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_166CH_b;
    };
    union
    {
        __IOM uint32_t REG_1670H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1670H_b;
    };
    union
    {
        __IOM uint32_t REG_1674H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1674H_b;
    };
    union
    {
        __IOM uint32_t REG_1678H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1678H_b;
    };
    union
    {
        __IOM uint32_t REG_167CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_167CH_b;
    };
    union
    {
        __IOM uint32_t REG_1680H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1680H_b;
    };
    union
    {
        __IOM uint32_t REG_1684H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1684H_b;
    };
    union
    {
        __IOM uint32_t REG_1688H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1688H_b;
    };
    union
    {
        __IOM uint32_t REG_168CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_168CH_b;
    };
    union
    {
        __IOM uint32_t REG_1690H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1690H_b;
    };
    union
    {
        __IOM uint32_t REG_1694H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1694H_b;
    };
    union
    {
        __IOM uint32_t REG_1698H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1698H_b;
    };
    union
    {
        __IOM uint32_t REG_169CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_169CH_b;
    };
    union
    {
        __IOM uint32_t REG_16A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16A0H_b;
    };
    union
    {
        __IOM uint32_t REG_16A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16A4H_b;
    };
    union
    {
        __IOM uint32_t REG_16A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16A8H_b;
    };
    union
    {
        __IOM uint32_t REG_16ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16ACH_b;
    };
    union
    {
        __IOM uint32_t REG_16B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16B0H_b;
    };
    union
    {
        __IOM uint32_t REG_16B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16B4H_b;
    };
    union
    {
        __IOM uint32_t REG_16B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16B8H_b;
    };
    union
    {
        __IOM uint32_t REG_16BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16BCH_b;
    };
    union
    {
        __IOM uint32_t REG_16C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16C0H_b;
    };
    union
    {
        __IOM uint32_t REG_16C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16C4H_b;
    };
    union
    {
        __IOM uint32_t REG_16C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16C8H_b;
    };
    union
    {
        __IOM uint32_t REG_16CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16CCH_b;
    };
    union
    {
        __IOM uint32_t REG_16D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16D0H_b;
    };
    union
    {
        __IOM uint32_t REG_16D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16D4H_b;
    };
    union
    {
        __IOM uint32_t REG_16D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16D8H_b;
    };
    union
    {
        __IOM uint32_t REG_16DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16DCH_b;
    };
    union
    {
        __IOM uint32_t REG_16E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16E0H_b;
    };
    union
    {
        __IOM uint32_t REG_16E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16E4H_b;
    };
    union
    {
        __IOM uint32_t REG_16E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16E8H_b;
    };
    union
    {
        __IOM uint32_t REG_16ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16ECH_b;
    };
    union
    {
        __IOM uint32_t REG_16F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16F0H_b;
    };
    union
    {
        __IOM uint32_t REG_16F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16F4H_b;
    };
    union
    {
        __IOM uint32_t REG_16F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16F8H_b;
    };
    union
    {
        __IOM uint32_t REG_16FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_16FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1700H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1700H_b;
    };
    union
    {
        __IOM uint32_t REG_1704H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1704H_b;
    };
    union
    {
        __IOM uint32_t REG_1708H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1708H_b;
    };
    union
    {
        __IOM uint32_t REG_170CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_170CH_b;
    };
    union
    {
        __IOM uint32_t REG_1710H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1710H_b;
    };
    union
    {
        __IOM uint32_t REG_1714H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1714H_b;
    };
    union
    {
        __IOM uint32_t REG_1718H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1718H_b;
    };
    union
    {
        __IOM uint32_t REG_171CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_171CH_b;
    };
    union
    {
        __IOM uint32_t REG_1720H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1720H_b;
    };
    union
    {
        __IOM uint32_t REG_1724H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1724H_b;
    };
    union
    {
        __IOM uint32_t REG_1728H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1728H_b;
    };
    union
    {
        __IOM uint32_t REG_172CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_172CH_b;
    };
    union
    {
        __IOM uint32_t REG_1730H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1730H_b;
    };
    union
    {
        __IOM uint32_t REG_1734H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1734H_b;
    };
    union
    {
        __IOM uint32_t REG_1738H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1738H_b;
    };
    union
    {
        __IOM uint32_t REG_173CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_173CH_b;
    };
    union
    {
        __IOM uint32_t REG_1740H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1740H_b;
    };
    union
    {
        __IOM uint32_t REG_1744H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1744H_b;
    };
    union
    {
        __IOM uint32_t REG_1748H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1748H_b;
    };
    union
    {
        __IOM uint32_t REG_174CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_174CH_b;
    };
    union
    {
        __IOM uint32_t REG_1750H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1750H_b;
    };
    union
    {
        __IOM uint32_t REG_1754H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1754H_b;
    };
    union
    {
        __IOM uint32_t REG_1758H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1758H_b;
    };
    union
    {
        __IOM uint32_t REG_175CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_175CH_b;
    };
    union
    {
        __IOM uint32_t REG_1760H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1760H_b;
    };
    union
    {
        __IOM uint32_t REG_1764H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1764H_b;
    };
    union
    {
        __IOM uint32_t REG_1768H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1768H_b;
    };
    union
    {
        __IOM uint32_t REG_176CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_176CH_b;
    };
    union
    {
        __IOM uint32_t REG_1770H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1770H_b;
    };
    union
    {
        __IOM uint32_t REG_1774H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1774H_b;
    };
    union
    {
        __IOM uint32_t REG_1778H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1778H_b;
    };
    union
    {
        __IOM uint32_t REG_177CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_177CH_b;
    };
    union
    {
        __IOM uint32_t REG_1780H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1780H_b;
    };
    union
    {
        __IOM uint32_t REG_1784H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1784H_b;
    };
    union
    {
        __IOM uint32_t REG_1788H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1788H_b;
    };
    union
    {
        __IOM uint32_t REG_178CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_178CH_b;
    };
    union
    {
        __IOM uint32_t REG_1790H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1790H_b;
    };
    union
    {
        __IOM uint32_t REG_1794H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1794H_b;
    };
    union
    {
        __IOM uint32_t REG_1798H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1798H_b;
    };
    union
    {
        __IOM uint32_t REG_179CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_179CH_b;
    };
    union
    {
        __IOM uint32_t REG_17A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17A0H_b;
    };
    union
    {
        __IOM uint32_t REG_17A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17A4H_b;
    };
    union
    {
        __IOM uint32_t REG_17A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17A8H_b;
    };
    union
    {
        __IOM uint32_t REG_17ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17ACH_b;
    };
    union
    {
        __IOM uint32_t REG_17B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17B0H_b;
    };
    union
    {
        __IOM uint32_t REG_17B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17B4H_b;
    };
    union
    {
        __IOM uint32_t REG_17B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17B8H_b;
    };
    union
    {
        __IOM uint32_t REG_17BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17BCH_b;
    };
    union
    {
        __IOM uint32_t REG_17C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17C0H_b;
    };
    union
    {
        __IOM uint32_t REG_17C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17C4H_b;
    };
    union
    {
        __IOM uint32_t REG_17C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17C8H_b;
    };
    union
    {
        __IOM uint32_t REG_17CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17CCH_b;
    };
    union
    {
        __IOM uint32_t REG_17D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17D0H_b;
    };
    union
    {
        __IOM uint32_t REG_17D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17D4H_b;
    };
    union
    {
        __IOM uint32_t REG_17D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17D8H_b;
    };
    union
    {
        __IOM uint32_t REG_17DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17DCH_b;
    };
    union
    {
        __IOM uint32_t REG_17E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17E0H_b;
    };
    union
    {
        __IOM uint32_t REG_17E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17E4H_b;
    };
    union
    {
        __IOM uint32_t REG_17E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17E8H_b;
    };
    union
    {
        __IOM uint32_t REG_17ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17ECH_b;
    };
    union
    {
        __IOM uint32_t REG_17F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17F0H_b;
    };
    union
    {
        __IOM uint32_t REG_17F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17F4H_b;
    };
    union
    {
        __IOM uint32_t REG_17F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17F8H_b;
    };
    union
    {
        __IOM uint32_t REG_17FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_17FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1800H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1800H_b;
    };
    union
    {
        __IOM uint32_t REG_1804H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1804H_b;
    };
    union
    {
        __IOM uint32_t REG_1808H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1808H_b;
    };
    union
    {
        __IOM uint32_t REG_180CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_180CH_b;
    };
    union
    {
        __IOM uint32_t REG_1810H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1810H_b;
    };
    union
    {
        __IOM uint32_t REG_1814H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1814H_b;
    };
    union
    {
        __IOM uint32_t REG_1818H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1818H_b;
    };
    union
    {
        __IOM uint32_t REG_181CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_181CH_b;
    };
    union
    {
        __IOM uint32_t REG_1820H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1820H_b;
    };
    union
    {
        __IOM uint32_t REG_1824H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1824H_b;
    };
    union
    {
        __IOM uint32_t REG_1828H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1828H_b;
    };
    union
    {
        __IOM uint32_t REG_182CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_182CH_b;
    };
    union
    {
        __IOM uint32_t REG_1830H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1830H_b;
    };
    union
    {
        __IOM uint32_t REG_1834H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1834H_b;
    };
    union
    {
        __IOM uint32_t REG_1838H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1838H_b;
    };
    union
    {
        __IOM uint32_t REG_183CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_183CH_b;
    };
    union
    {
        __IOM uint32_t REG_1840H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1840H_b;
    };
    union
    {
        __IOM uint32_t REG_1844H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1844H_b;
    };
    union
    {
        __IOM uint32_t REG_1848H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1848H_b;
    };
    union
    {
        __IOM uint32_t REG_184CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_184CH_b;
    };
    union
    {
        __IOM uint32_t REG_1850H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1850H_b;
    };
    union
    {
        __IOM uint32_t REG_1854H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1854H_b;
    };
    union
    {
        __IOM uint32_t REG_1858H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1858H_b;
    };
    union
    {
        __IOM uint32_t REG_185CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_185CH_b;
    };
    union
    {
        __IOM uint32_t REG_1860H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1860H_b;
    };
    union
    {
        __IOM uint32_t REG_1864H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1864H_b;
    };
    union
    {
        __IOM uint32_t REG_1868H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1868H_b;
    };
    union
    {
        __IOM uint32_t REG_186CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_186CH_b;
    };
    union
    {
        __IOM uint32_t REG_1870H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1870H_b;
    };
    union
    {
        __IOM uint32_t REG_1874H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1874H_b;
    };
    union
    {
        __IOM uint32_t REG_1878H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1878H_b;
    };
    union
    {
        __IOM uint32_t REG_187CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_187CH_b;
    };
    union
    {
        __IOM uint32_t REG_1880H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1880H_b;
    };
    union
    {
        __IOM uint32_t REG_1884H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1884H_b;
    };
    union
    {
        __IOM uint32_t REG_1888H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1888H_b;
    };
    union
    {
        __IOM uint32_t REG_188CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_188CH_b;
    };
    union
    {
        __IOM uint32_t REG_1890H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1890H_b;
    };
    union
    {
        __IOM uint32_t REG_1894H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1894H_b;
    };
    union
    {
        __IOM uint32_t REG_1898H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1898H_b;
    };
    union
    {
        __IOM uint32_t REG_189CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_189CH_b;
    };
    union
    {
        __IOM uint32_t REG_18A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18A0H_b;
    };
    union
    {
        __IOM uint32_t REG_18A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18A4H_b;
    };
    union
    {
        __IOM uint32_t REG_18A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18A8H_b;
    };
    union
    {
        __IOM uint32_t REG_18ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18ACH_b;
    };
    union
    {
        __IOM uint32_t REG_18B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18B0H_b;
    };
    union
    {
        __IOM uint32_t REG_18B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18B4H_b;
    };
    union
    {
        __IOM uint32_t REG_18B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18B8H_b;
    };
    union
    {
        __IOM uint32_t REG_18BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18BCH_b;
    };
    union
    {
        __IOM uint32_t REG_18C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18C0H_b;
    };
    union
    {
        __IOM uint32_t REG_18C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18C4H_b;
    };
    union
    {
        __IOM uint32_t REG_18C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18C8H_b;
    };
    union
    {
        __IOM uint32_t REG_18CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18CCH_b;
    };
    union
    {
        __IOM uint32_t REG_18D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18D0H_b;
    };
    union
    {
        __IOM uint32_t REG_18D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18D4H_b;
    };
    union
    {
        __IOM uint32_t REG_18D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18D8H_b;
    };
    union
    {
        __IOM uint32_t REG_18DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18DCH_b;
    };
    union
    {
        __IOM uint32_t REG_18E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18E0H_b;
    };
    union
    {
        __IOM uint32_t REG_18E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18E4H_b;
    };
    union
    {
        __IOM uint32_t REG_18E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18E8H_b;
    };
    union
    {
        __IOM uint32_t REG_18ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18ECH_b;
    };
    union
    {
        __IOM uint32_t REG_18F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18F0H_b;
    };
    union
    {
        __IOM uint32_t REG_18F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18F4H_b;
    };
    union
    {
        __IOM uint32_t REG_18F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18F8H_b;
    };
    union
    {
        __IOM uint32_t REG_18FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_18FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1900H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1900H_b;
    };
    union
    {
        __IOM uint32_t REG_1904H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1904H_b;
    };
    union
    {
        __IOM uint32_t REG_1908H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1908H_b;
    };
    union
    {
        __IOM uint32_t REG_190CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_190CH_b;
    };
    union
    {
        __IOM uint32_t REG_1910H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1910H_b;
    };
    union
    {
        __IOM uint32_t REG_1914H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1914H_b;
    };
    union
    {
        __IOM uint32_t REG_1918H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1918H_b;
    };
    union
    {
        __IOM uint32_t REG_191CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_191CH_b;
    };
    union
    {
        __IOM uint32_t REG_1920H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1920H_b;
    };
    union
    {
        __IOM uint32_t REG_1924H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1924H_b;
    };
    union
    {
        __IOM uint32_t REG_1928H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1928H_b;
    };
    union
    {
        __IOM uint32_t REG_192CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_192CH_b;
    };
    union
    {
        __IOM uint32_t REG_1930H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1930H_b;
    };
    union
    {
        __IOM uint32_t REG_1934H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1934H_b;
    };
    union
    {
        __IOM uint32_t REG_1938H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1938H_b;
    };
    union
    {
        __IOM uint32_t REG_193CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_193CH_b;
    };
    union
    {
        __IOM uint32_t REG_1940H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1940H_b;
    };
    union
    {
        __IOM uint32_t REG_1944H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1944H_b;
    };
    union
    {
        __IOM uint32_t REG_1948H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1948H_b;
    };
    union
    {
        __IOM uint32_t REG_194CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_194CH_b;
    };
    union
    {
        __IOM uint32_t REG_1950H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1950H_b;
    };
    union
    {
        __IOM uint32_t REG_1954H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1954H_b;
    };
    union
    {
        __IOM uint32_t REG_1958H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1958H_b;
    };
    union
    {
        __IOM uint32_t REG_195CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_195CH_b;
    };
    union
    {
        __IOM uint32_t REG_1960H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1960H_b;
    };
    union
    {
        __IOM uint32_t REG_1964H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1964H_b;
    };
    union
    {
        __IOM uint32_t REG_1968H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1968H_b;
    };
    union
    {
        __IOM uint32_t REG_196CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_196CH_b;
    };
    union
    {
        __IOM uint32_t REG_1970H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1970H_b;
    };
    union
    {
        __IOM uint32_t REG_1974H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1974H_b;
    };
    union
    {
        __IOM uint32_t REG_1978H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1978H_b;
    };
    union
    {
        __IOM uint32_t REG_197CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_197CH_b;
    };
    union
    {
        __IOM uint32_t REG_1980H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1980H_b;
    };
    union
    {
        __IOM uint32_t REG_1984H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1984H_b;
    };
    union
    {
        __IOM uint32_t REG_1988H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1988H_b;
    };
    union
    {
        __IOM uint32_t REG_198CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_198CH_b;
    };
    union
    {
        __IOM uint32_t REG_1990H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1990H_b;
    };
    union
    {
        __IOM uint32_t REG_1994H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1994H_b;
    };
    union
    {
        __IOM uint32_t REG_1998H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1998H_b;
    };
    union
    {
        __IOM uint32_t REG_199CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_199CH_b;
    };
    union
    {
        __IOM uint32_t REG_19A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19A0H_b;
    };
    union
    {
        __IOM uint32_t REG_19A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19A4H_b;
    };
    union
    {
        __IOM uint32_t REG_19A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19A8H_b;
    };
    union
    {
        __IOM uint32_t REG_19ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19ACH_b;
    };
    union
    {
        __IOM uint32_t REG_19B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19B0H_b;
    };
    union
    {
        __IOM uint32_t REG_19B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19B4H_b;
    };
    union
    {
        __IOM uint32_t REG_19B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19B8H_b;
    };
    union
    {
        __IOM uint32_t REG_19BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19BCH_b;
    };
    union
    {
        __IOM uint32_t REG_19C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19C0H_b;
    };
    union
    {
        __IOM uint32_t REG_19C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19C4H_b;
    };
    union
    {
        __IOM uint32_t REG_19C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19C8H_b;
    };
    union
    {
        __IOM uint32_t REG_19CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19CCH_b;
    };
    union
    {
        __IOM uint32_t REG_19D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19D0H_b;
    };
    union
    {
        __IOM uint32_t REG_19D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19D4H_b;
    };
    union
    {
        __IOM uint32_t REG_19D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19D8H_b;
    };
    union
    {
        __IOM uint32_t REG_19DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19DCH_b;
    };
    union
    {
        __IOM uint32_t REG_19E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19E0H_b;
    };
    union
    {
        __IOM uint32_t REG_19E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19E4H_b;
    };
    union
    {
        __IOM uint32_t REG_19E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19E8H_b;
    };
    union
    {
        __IOM uint32_t REG_19ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19ECH_b;
    };
    union
    {
        __IOM uint32_t REG_19F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19F0H_b;
    };
    union
    {
        __IOM uint32_t REG_19F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19F4H_b;
    };
    union
    {
        __IOM uint32_t REG_19F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19F8H_b;
    };
    union
    {
        __IOM uint32_t REG_19FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_19FCH_b;
    };
    union
    {
        __IOM uint32_t REG_1A00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A00H_b;
    };
    union
    {
        __IOM uint32_t REG_1A04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A04H_b;
    };
    union
    {
        __IOM uint32_t REG_1A08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A08H_b;
    };
    union
    {
        __IOM uint32_t REG_1A0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A0CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A10H_b;
    };
    union
    {
        __IOM uint32_t REG_1A14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A14H_b;
    };
    union
    {
        __IOM uint32_t REG_1A18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A18H_b;
    };
    union
    {
        __IOM uint32_t REG_1A1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A1CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A20H_b;
    };
    union
    {
        __IOM uint32_t REG_1A24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A24H_b;
    };
    union
    {
        __IOM uint32_t REG_1A28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A28H_b;
    };
    union
    {
        __IOM uint32_t REG_1A2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A2CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A30H_b;
    };
    union
    {
        __IOM uint32_t REG_1A34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A34H_b;
    };
    union
    {
        __IOM uint32_t REG_1A38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A38H_b;
    };
    union
    {
        __IOM uint32_t REG_1A3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A3CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A40H_b;
    };
    union
    {
        __IOM uint32_t REG_1A44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A44H_b;
    };
    union
    {
        __IOM uint32_t REG_1A48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A48H_b;
    };
    union
    {
        __IOM uint32_t REG_1A4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A4CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A50H_b;
    };
    union
    {
        __IOM uint32_t REG_1A54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A54H_b;
    };
    union
    {
        __IOM uint32_t REG_1A58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A58H_b;
    };
    union
    {
        __IOM uint32_t REG_1A5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A5CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A60H_b;
    };
    union
    {
        __IOM uint32_t REG_1A64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A64H_b;
    };
    union
    {
        __IOM uint32_t REG_1A68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A68H_b;
    };
    union
    {
        __IOM uint32_t REG_1A6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A6CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A70H_b;
    };
    union
    {
        __IOM uint32_t REG_1A74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A74H_b;
    };
    union
    {
        __IOM uint32_t REG_1A78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A78H_b;
    };
    union
    {
        __IOM uint32_t REG_1A7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A7CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A80H_b;
    };
    union
    {
        __IOM uint32_t REG_1A84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A84H_b;
    };
    union
    {
        __IOM uint32_t REG_1A88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A88H_b;
    };
    union
    {
        __IOM uint32_t REG_1A8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A8CH_b;
    };
    union
    {
        __IOM uint32_t REG_1A90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A90H_b;
    };
    union
    {
        __IOM uint32_t REG_1A94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A94H_b;
    };
    union
    {
        __IOM uint32_t REG_1A98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A98H_b;
    };
    union
    {
        __IOM uint32_t REG_1A9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1A9CH_b;
    };
    union
    {
        __IOM uint32_t REG_1AA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AA0H_b;
    };
    union
    {
        __IOM uint32_t REG_1AA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AA4H_b;
    };
    union
    {
        __IOM uint32_t REG_1AA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AA8H_b;
    };
    union
    {
        __IOM uint32_t REG_1AACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AACH_b;
    };
    union
    {
        __IOM uint32_t REG_1AB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AB0H_b;
    };
    union
    {
        __IOM uint32_t REG_1AB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AB4H_b;
    };
    union
    {
        __IOM uint32_t REG_1AB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AB8H_b;
    };
    union
    {
        __IOM uint32_t REG_1ABCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ABCH_b;
    };
    union
    {
        __IOM uint32_t REG_1AC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AC0H_b;
    };
    union
    {
        __IOM uint32_t REG_1AC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AC4H_b;
    };
    union
    {
        __IOM uint32_t REG_1AC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AC8H_b;
    };
    union
    {
        __IOM uint32_t REG_1ACCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ACCH_b;
    };
    union
    {
        __IOM uint32_t REG_1AD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AD0H_b;
    };
    union
    {
        __IOM uint32_t REG_1AD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AD4H_b;
    };
    union
    {
        __IOM uint32_t REG_1AD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AD8H_b;
    };
    union
    {
        __IOM uint32_t REG_1ADCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ADCH_b;
    };
    union
    {
        __IOM uint32_t REG_1AE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AE0H_b;
    };
    union
    {
        __IOM uint32_t REG_1AE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AE4H_b;
    };
    union
    {
        __IOM uint32_t REG_1AE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AE8H_b;
    };
    union
    {
        __IOM uint32_t REG_1AECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AECH_b;
    };
    union
    {
        __IOM uint32_t REG_1AF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AF0H_b;
    };
    union
    {
        __IOM uint32_t REG_1AF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AF4H_b;
    };
    union
    {
        __IOM uint32_t REG_1AF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AF8H_b;
    };
    union
    {
        __IOM uint32_t REG_1AFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1AFCH_b;
    };
    union
    {
        __IOM uint32_t REG_1B00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B00H_b;
    };
    union
    {
        __IOM uint32_t REG_1B04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B04H_b;
    };
    union
    {
        __IOM uint32_t REG_1B08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B08H_b;
    };
    union
    {
        __IOM uint32_t REG_1B0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B0CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B10H_b;
    };
    union
    {
        __IOM uint32_t REG_1B14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B14H_b;
    };
    union
    {
        __IOM uint32_t REG_1B18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B18H_b;
    };
    union
    {
        __IOM uint32_t REG_1B1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B1CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B20H_b;
    };
    union
    {
        __IOM uint32_t REG_1B24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B24H_b;
    };
    union
    {
        __IOM uint32_t REG_1B28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B28H_b;
    };
    union
    {
        __IOM uint32_t REG_1B2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B2CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B30H_b;
    };
    union
    {
        __IOM uint32_t REG_1B34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B34H_b;
    };
    union
    {
        __IOM uint32_t REG_1B38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B38H_b;
    };
    union
    {
        __IOM uint32_t REG_1B3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B3CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B40H_b;
    };
    union
    {
        __IOM uint32_t REG_1B44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B44H_b;
    };
    union
    {
        __IOM uint32_t REG_1B48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B48H_b;
    };
    union
    {
        __IOM uint32_t REG_1B4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B4CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B50H_b;
    };
    union
    {
        __IOM uint32_t REG_1B54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B54H_b;
    };
    union
    {
        __IOM uint32_t REG_1B58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B58H_b;
    };
    union
    {
        __IOM uint32_t REG_1B5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B5CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B60H_b;
    };
    union
    {
        __IOM uint32_t REG_1B64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B64H_b;
    };
    union
    {
        __IOM uint32_t REG_1B68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B68H_b;
    };
    union
    {
        __IOM uint32_t REG_1B6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B6CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B70H_b;
    };
    union
    {
        __IOM uint32_t REG_1B74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B74H_b;
    };
    union
    {
        __IOM uint32_t REG_1B78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B78H_b;
    };
    union
    {
        __IOM uint32_t REG_1B7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B7CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B80H_b;
    };
    union
    {
        __IOM uint32_t REG_1B84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B84H_b;
    };
    union
    {
        __IOM uint32_t REG_1B88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B88H_b;
    };
    union
    {
        __IOM uint32_t REG_1B8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B8CH_b;
    };
    union
    {
        __IOM uint32_t REG_1B90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B90H_b;
    };
    union
    {
        __IOM uint32_t REG_1B94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B94H_b;
    };
    union
    {
        __IOM uint32_t REG_1B98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B98H_b;
    };
    union
    {
        __IOM uint32_t REG_1B9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1B9CH_b;
    };
    union
    {
        __IOM uint32_t REG_1BA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BA0H_b;
    };
    union
    {
        __IOM uint32_t REG_1BA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BA4H_b;
    };
    union
    {
        __IOM uint32_t REG_1BA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BA8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BACH_b;
    };
    union
    {
        __IOM uint32_t REG_1BB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BB0H_b;
    };
    union
    {
        __IOM uint32_t REG_1BB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BB4H_b;
    };
    union
    {
        __IOM uint32_t REG_1BB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BB8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BBCH_b;
    };
    union
    {
        __IOM uint32_t REG_1BC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BC0H_b;
    };
    union
    {
        __IOM uint32_t REG_1BC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BC4H_b;
    };
    union
    {
        __IOM uint32_t REG_1BC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BC8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BCCH_b;
    };
    union
    {
        __IOM uint32_t REG_1BD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BD0H_b;
    };
    union
    {
        __IOM uint32_t REG_1BD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BD4H_b;
    };
    union
    {
        __IOM uint32_t REG_1BD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BD8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BDCH_b;
    };
    union
    {
        __IOM uint32_t REG_1BE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BE0H_b;
    };
    union
    {
        __IOM uint32_t REG_1BE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BE4H_b;
    };
    union
    {
        __IOM uint32_t REG_1BE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BE8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BECH_b;
    };
    union
    {
        __IOM uint32_t REG_1BF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BF0H_b;
    };
    union
    {
        __IOM uint32_t REG_1BF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BF4H_b;
    };
    union
    {
        __IOM uint32_t REG_1BF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BF8H_b;
    };
    union
    {
        __IOM uint32_t REG_1BFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1BFCH_b;
    };
    union
    {
        __IOM uint32_t REG_1C00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C00H_b;
    };
    union
    {
        __IOM uint32_t REG_1C04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C04H_b;
    };
    union
    {
        __IOM uint32_t REG_1C08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C08H_b;
    };
    union
    {
        __IOM uint32_t REG_1C0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C0CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C10H_b;
    };
    union
    {
        __IOM uint32_t REG_1C14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C14H_b;
    };
    union
    {
        __IOM uint32_t REG_1C18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C18H_b;
    };
    union
    {
        __IOM uint32_t REG_1C1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C1CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C20H_b;
    };
    union
    {
        __IOM uint32_t REG_1C24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C24H_b;
    };
    union
    {
        __IOM uint32_t REG_1C28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C28H_b;
    };
    union
    {
        __IOM uint32_t REG_1C2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C2CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C30H_b;
    };
    union
    {
        __IOM uint32_t REG_1C34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C34H_b;
    };
    union
    {
        __IOM uint32_t REG_1C38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C38H_b;
    };
    union
    {
        __IOM uint32_t REG_1C3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C3CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C40H_b;
    };
    union
    {
        __IOM uint32_t REG_1C44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C44H_b;
    };
    union
    {
        __IOM uint32_t REG_1C48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C48H_b;
    };
    union
    {
        __IOM uint32_t REG_1C4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C4CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C50H_b;
    };
    union
    {
        __IOM uint32_t REG_1C54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C54H_b;
    };
    union
    {
        __IOM uint32_t REG_1C58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C58H_b;
    };
    union
    {
        __IOM uint32_t REG_1C5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C5CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C60H_b;
    };
    union
    {
        __IOM uint32_t REG_1C64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C64H_b;
    };
    union
    {
        __IOM uint32_t REG_1C68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C68H_b;
    };
    union
    {
        __IOM uint32_t REG_1C6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C6CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C70H_b;
    };
    union
    {
        __IOM uint32_t REG_1C74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C74H_b;
    };
    union
    {
        __IOM uint32_t REG_1C78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C78H_b;
    };
    union
    {
        __IOM uint32_t REG_1C7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C7CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C80H_b;
    };
    union
    {
        __IOM uint32_t REG_1C84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C84H_b;
    };
    union
    {
        __IOM uint32_t REG_1C88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C88H_b;
    };
    union
    {
        __IOM uint32_t REG_1C8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C8CH_b;
    };
    union
    {
        __IOM uint32_t REG_1C90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C90H_b;
    };
    union
    {
        __IOM uint32_t REG_1C94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C94H_b;
    };
    union
    {
        __IOM uint32_t REG_1C98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C98H_b;
    };
    union
    {
        __IOM uint32_t REG_1C9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1C9CH_b;
    };
    union
    {
        __IOM uint32_t REG_1CA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CA0H_b;
    };
    union
    {
        __IOM uint32_t REG_1CA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CA4H_b;
    };
    union
    {
        __IOM uint32_t REG_1CA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CA8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CACH_b;
    };
    union
    {
        __IOM uint32_t REG_1CB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CB0H_b;
    };
    union
    {
        __IOM uint32_t REG_1CB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CB4H_b;
    };
    union
    {
        __IOM uint32_t REG_1CB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CB8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CBCH_b;
    };
    union
    {
        __IOM uint32_t REG_1CC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CC0H_b;
    };
    union
    {
        __IOM uint32_t REG_1CC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CC4H_b;
    };
    union
    {
        __IOM uint32_t REG_1CC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CC8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CCCH_b;
    };
    union
    {
        __IOM uint32_t REG_1CD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CD0H_b;
    };
    union
    {
        __IOM uint32_t REG_1CD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CD4H_b;
    };
    union
    {
        __IOM uint32_t REG_1CD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CD8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CDCH_b;
    };
    union
    {
        __IOM uint32_t REG_1CE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CE0H_b;
    };
    union
    {
        __IOM uint32_t REG_1CE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CE4H_b;
    };
    union
    {
        __IOM uint32_t REG_1CE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CE8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CECH_b;
    };
    union
    {
        __IOM uint32_t REG_1CF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CF0H_b;
    };
    union
    {
        __IOM uint32_t REG_1CF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CF4H_b;
    };
    union
    {
        __IOM uint32_t REG_1CF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CF8H_b;
    };
    union
    {
        __IOM uint32_t REG_1CFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1CFCH_b;
    };
    union
    {
        __IOM uint32_t REG_1D00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D00H_b;
    };
    union
    {
        __IOM uint32_t REG_1D04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D04H_b;
    };
    union
    {
        __IOM uint32_t REG_1D08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D08H_b;
    };
    union
    {
        __IOM uint32_t REG_1D0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D0CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D10H_b;
    };
    union
    {
        __IOM uint32_t REG_1D14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D14H_b;
    };
    union
    {
        __IOM uint32_t REG_1D18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D18H_b;
    };
    union
    {
        __IOM uint32_t REG_1D1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D1CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D20H_b;
    };
    union
    {
        __IOM uint32_t REG_1D24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D24H_b;
    };
    union
    {
        __IOM uint32_t REG_1D28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D28H_b;
    };
    union
    {
        __IOM uint32_t REG_1D2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D2CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D30H_b;
    };
    union
    {
        __IOM uint32_t REG_1D34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D34H_b;
    };
    union
    {
        __IOM uint32_t REG_1D38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D38H_b;
    };
    union
    {
        __IOM uint32_t REG_1D3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D3CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D40H_b;
    };
    union
    {
        __IOM uint32_t REG_1D44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D44H_b;
    };
    union
    {
        __IOM uint32_t REG_1D48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D48H_b;
    };
    union
    {
        __IOM uint32_t REG_1D4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D4CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D50H_b;
    };
    union
    {
        __IOM uint32_t REG_1D54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D54H_b;
    };
    union
    {
        __IOM uint32_t REG_1D58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D58H_b;
    };
    union
    {
        __IOM uint32_t REG_1D5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D5CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D60H_b;
    };
    union
    {
        __IOM uint32_t REG_1D64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D64H_b;
    };
    union
    {
        __IOM uint32_t REG_1D68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D68H_b;
    };
    union
    {
        __IOM uint32_t REG_1D6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D6CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D70H_b;
    };
    union
    {
        __IOM uint32_t REG_1D74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D74H_b;
    };
    union
    {
        __IOM uint32_t REG_1D78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D78H_b;
    };
    union
    {
        __IOM uint32_t REG_1D7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D7CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D80H_b;
    };
    union
    {
        __IOM uint32_t REG_1D84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D84H_b;
    };
    union
    {
        __IOM uint32_t REG_1D88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D88H_b;
    };
    union
    {
        __IOM uint32_t REG_1D8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D8CH_b;
    };
    union
    {
        __IOM uint32_t REG_1D90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D90H_b;
    };
    union
    {
        __IOM uint32_t REG_1D94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D94H_b;
    };
    union
    {
        __IOM uint32_t REG_1D98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D98H_b;
    };
    union
    {
        __IOM uint32_t REG_1D9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1D9CH_b;
    };
    union
    {
        __IOM uint32_t REG_1DA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DA0H_b;
    };
    union
    {
        __IOM uint32_t REG_1DA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DA4H_b;
    };
    union
    {
        __IOM uint32_t REG_1DA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DA8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DACH_b;
    };
    union
    {
        __IOM uint32_t REG_1DB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DB0H_b;
    };
    union
    {
        __IOM uint32_t REG_1DB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DB4H_b;
    };
    union
    {
        __IOM uint32_t REG_1DB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DB8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DBCH_b;
    };
    union
    {
        __IOM uint32_t REG_1DC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DC0H_b;
    };
    union
    {
        __IOM uint32_t REG_1DC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DC4H_b;
    };
    union
    {
        __IOM uint32_t REG_1DC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DC8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DCCH_b;
    };
    union
    {
        __IOM uint32_t REG_1DD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DD0H_b;
    };
    union
    {
        __IOM uint32_t REG_1DD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DD4H_b;
    };
    union
    {
        __IOM uint32_t REG_1DD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DD8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DDCH_b;
    };
    union
    {
        __IOM uint32_t REG_1DE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DE0H_b;
    };
    union
    {
        __IOM uint32_t REG_1DE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DE4H_b;
    };
    union
    {
        __IOM uint32_t REG_1DE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DE8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DECH_b;
    };
    union
    {
        __IOM uint32_t REG_1DF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DF0H_b;
    };
    union
    {
        __IOM uint32_t REG_1DF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DF4H_b;
    };
    union
    {
        __IOM uint32_t REG_1DF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DF8H_b;
    };
    union
    {
        __IOM uint32_t REG_1DFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1DFCH_b;
    };
    union
    {
        __IOM uint32_t REG_1E00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E00H_b;
    };
    union
    {
        __IOM uint32_t REG_1E04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E04H_b;
    };
    union
    {
        __IOM uint32_t REG_1E08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E08H_b;
    };
    union
    {
        __IOM uint32_t REG_1E0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E0CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E10H_b;
    };
    union
    {
        __IOM uint32_t REG_1E14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E14H_b;
    };
    union
    {
        __IOM uint32_t REG_1E18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E18H_b;
    };
    union
    {
        __IOM uint32_t REG_1E1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E1CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E20H_b;
    };
    union
    {
        __IOM uint32_t REG_1E24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E24H_b;
    };
    union
    {
        __IOM uint32_t REG_1E28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E28H_b;
    };
    union
    {
        __IOM uint32_t REG_1E2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E2CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E30H_b;
    };
    union
    {
        __IOM uint32_t REG_1E34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E34H_b;
    };
    union
    {
        __IOM uint32_t REG_1E38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E38H_b;
    };
    union
    {
        __IOM uint32_t REG_1E3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E3CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E40H_b;
    };
    union
    {
        __IOM uint32_t REG_1E44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E44H_b;
    };
    union
    {
        __IOM uint32_t REG_1E48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E48H_b;
    };
    union
    {
        __IOM uint32_t REG_1E4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E4CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E50H_b;
    };
    union
    {
        __IOM uint32_t REG_1E54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E54H_b;
    };
    union
    {
        __IOM uint32_t REG_1E58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E58H_b;
    };
    union
    {
        __IOM uint32_t REG_1E5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E5CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E60H_b;
    };
    union
    {
        __IOM uint32_t REG_1E64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E64H_b;
    };
    union
    {
        __IOM uint32_t REG_1E68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E68H_b;
    };
    union
    {
        __IOM uint32_t REG_1E6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E6CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E70H_b;
    };
    union
    {
        __IOM uint32_t REG_1E74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E74H_b;
    };
    union
    {
        __IOM uint32_t REG_1E78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E78H_b;
    };
    union
    {
        __IOM uint32_t REG_1E7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E7CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E80H_b;
    };
    union
    {
        __IOM uint32_t REG_1E84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E84H_b;
    };
    union
    {
        __IOM uint32_t REG_1E88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E88H_b;
    };
    union
    {
        __IOM uint32_t REG_1E8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E8CH_b;
    };
    union
    {
        __IOM uint32_t REG_1E90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E90H_b;
    };
    union
    {
        __IOM uint32_t REG_1E94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E94H_b;
    };
    union
    {
        __IOM uint32_t REG_1E98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E98H_b;
    };
    union
    {
        __IOM uint32_t REG_1E9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1E9CH_b;
    };
    union
    {
        __IOM uint32_t REG_1EA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EA0H_b;
    };
    union
    {
        __IOM uint32_t REG_1EA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EA4H_b;
    };
    union
    {
        __IOM uint32_t REG_1EA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EA8H_b;
    };
    union
    {
        __IOM uint32_t REG_1EACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EACH_b;
    };
    union
    {
        __IOM uint32_t REG_1EB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EB0H_b;
    };
    union
    {
        __IOM uint32_t REG_1EB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EB4H_b;
    };
    union
    {
        __IOM uint32_t REG_1EB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EB8H_b;
    };
    union
    {
        __IOM uint32_t REG_1EBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EBCH_b;
    };
    union
    {
        __IOM uint32_t REG_1EC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EC0H_b;
    };
    union
    {
        __IOM uint32_t REG_1EC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EC4H_b;
    };
    union
    {
        __IOM uint32_t REG_1EC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EC8H_b;
    };
    union
    {
        __IOM uint32_t REG_1ECCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ECCH_b;
    };
    union
    {
        __IOM uint32_t REG_1ED0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ED0H_b;
    };
    union
    {
        __IOM uint32_t REG_1ED4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ED4H_b;
    };
    union
    {
        __IOM uint32_t REG_1ED8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1ED8H_b;
    };
    union
    {
        __IOM uint32_t REG_1EDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EDCH_b;
    };
    union
    {
        __IOM uint32_t REG_1EE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EE0H_b;
    };
    union
    {
        __IOM uint32_t REG_1EE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EE4H_b;
    };
    union
    {
        __IOM uint32_t REG_1EE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EE8H_b;
    };
    union
    {
        __IOM uint32_t REG_1EECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EECH_b;
    };
    union
    {
        __IOM uint32_t REG_1EF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EF0H_b;
    };
    union
    {
        __IOM uint32_t REG_1EF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EF4H_b;
    };
    union
    {
        __IOM uint32_t REG_1EF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EF8H_b;
    };
    union
    {
        __IOM uint32_t REG_1EFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1EFCH_b;
    };
    union
    {
        __IOM uint32_t REG_1F00H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F00H_b;
    };
    union
    {
        __IOM uint32_t REG_1F04H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F04H_b;
    };
    union
    {
        __IOM uint32_t REG_1F08H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F08H_b;
    };
    union
    {
        __IOM uint32_t REG_1F0CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F0CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F10H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F10H_b;
    };
    union
    {
        __IOM uint32_t REG_1F14H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F14H_b;
    };
    union
    {
        __IOM uint32_t REG_1F18H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F18H_b;
    };
    union
    {
        __IOM uint32_t REG_1F1CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F1CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F20H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F20H_b;
    };
    union
    {
        __IOM uint32_t REG_1F24H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F24H_b;
    };
    union
    {
        __IOM uint32_t REG_1F28H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F28H_b;
    };
    union
    {
        __IOM uint32_t REG_1F2CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F2CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F30H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F30H_b;
    };
    union
    {
        __IOM uint32_t REG_1F34H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F34H_b;
    };
    union
    {
        __IOM uint32_t REG_1F38H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F38H_b;
    };
    union
    {
        __IOM uint32_t REG_1F3CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F3CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F40H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F40H_b;
    };
    union
    {
        __IOM uint32_t REG_1F44H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F44H_b;
    };
    union
    {
        __IOM uint32_t REG_1F48H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F48H_b;
    };
    union
    {
        __IOM uint32_t REG_1F4CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F4CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F50H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F50H_b;
    };
    union
    {
        __IOM uint32_t REG_1F54H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F54H_b;
    };
    union
    {
        __IOM uint32_t REG_1F58H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F58H_b;
    };
    union
    {
        __IOM uint32_t REG_1F5CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F5CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F60H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F60H_b;
    };
    union
    {
        __IOM uint32_t REG_1F64H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F64H_b;
    };
    union
    {
        __IOM uint32_t REG_1F68H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F68H_b;
    };
    union
    {
        __IOM uint32_t REG_1F6CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F6CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F70H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F70H_b;
    };
    union
    {
        __IOM uint32_t REG_1F74H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F74H_b;
    };
    union
    {
        __IOM uint32_t REG_1F78H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F78H_b;
    };
    union
    {
        __IOM uint32_t REG_1F7CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F7CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F80H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F80H_b;
    };
    union
    {
        __IOM uint32_t REG_1F84H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F84H_b;
    };
    union
    {
        __IOM uint32_t REG_1F88H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F88H_b;
    };
    union
    {
        __IOM uint32_t REG_1F8CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F8CH_b;
    };
    union
    {
        __IOM uint32_t REG_1F90H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F90H_b;
    };
    union
    {
        __IOM uint32_t REG_1F94H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F94H_b;
    };
    union
    {
        __IOM uint32_t REG_1F98H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F98H_b;
    };
    union
    {
        __IOM uint32_t REG_1F9CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1F9CH_b;
    };
    union
    {
        __IOM uint32_t REG_1FA0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FA0H_b;
    };
    union
    {
        __IOM uint32_t REG_1FA4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FA4H_b;
    };
    union
    {
        __IOM uint32_t REG_1FA8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FA8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FACH_b;
    };
    union
    {
        __IOM uint32_t REG_1FB0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FB0H_b;
    };
    union
    {
        __IOM uint32_t REG_1FB4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FB4H_b;
    };
    union
    {
        __IOM uint32_t REG_1FB8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FB8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FBCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FBCH_b;
    };
    union
    {
        __IOM uint32_t REG_1FC0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FC0H_b;
    };
    union
    {
        __IOM uint32_t REG_1FC4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FC4H_b;
    };
    union
    {
        __IOM uint32_t REG_1FC8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FC8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FCCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FCCH_b;
    };
    union
    {
        __IOM uint32_t REG_1FD0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FD0H_b;
    };
    union
    {
        __IOM uint32_t REG_1FD4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FD4H_b;
    };
    union
    {
        __IOM uint32_t REG_1FD8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FD8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FDCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FDCH_b;
    };
    union
    {
        __IOM uint32_t REG_1FE0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FE0H_b;
    };
    union
    {
        __IOM uint32_t REG_1FE4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FE4H_b;
    };
    union
    {
        __IOM uint32_t REG_1FE8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FE8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FECH_b;
    };
    union
    {
        __IOM uint32_t REG_1FF0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FF0H_b;
    };
    union
    {
        __IOM uint32_t REG_1FF4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FF4H_b;
    };
    union
    {
        __IOM uint32_t REG_1FF8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FF8H_b;
    };
    union
    {
        __IOM uint32_t REG_1FFCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_1FFCH_b;
    };
    union
    {
        __IOM uint32_t REG_2000H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2000H_b;
    };
    union
    {
        __IOM uint32_t REG_2004H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2004H_b;
    };
    union
    {
        __IOM uint32_t REG_2008H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2008H_b;
    };
    union
    {
        __IOM uint32_t REG_200CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_200CH_b;
    };
    union
    {
        __IOM uint32_t REG_2010H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2010H_b;
    };
    union
    {
        __IOM uint32_t REG_2014H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2014H_b;
    };
    union
    {
        __IOM uint32_t REG_2018H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2018H_b;
    };
    union
    {
        __IOM uint32_t REG_201CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_201CH_b;
    };
    union
    {
        __IOM uint32_t REG_2020H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2020H_b;
    };
    union
    {
        __IOM uint32_t REG_2024H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2024H_b;
    };
    union
    {
        __IOM uint32_t REG_2028H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2028H_b;
    };
    union
    {
        __IOM uint32_t REG_202CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_202CH_b;
    };
    union
    {
        __IOM uint32_t REG_2030H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2030H_b;
    };
    union
    {
        __IOM uint32_t REG_2034H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2034H_b;
    };
    union
    {
        __IOM uint32_t REG_2038H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2038H_b;
    };
    union
    {
        __IOM uint32_t REG_203CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_203CH_b;
    };
    union
    {
        __IOM uint32_t REG_2040H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2040H_b;
    };
    union
    {
        __IOM uint32_t REG_2044H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2044H_b;
    };
    union
    {
        __IOM uint32_t REG_2048H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2048H_b;
    };
    union
    {
        __IOM uint32_t REG_204CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_204CH_b;
    };
    union
    {
        __IOM uint32_t REG_2050H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2050H_b;
    };
    union
    {
        __IOM uint32_t REG_2054H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2054H_b;
    };
    union
    {
        __IOM uint32_t REG_2058H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2058H_b;
    };
    union
    {
        __IOM uint32_t REG_205CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_205CH_b;
    };
    union
    {
        __IOM uint32_t REG_2060H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2060H_b;
    };
    union
    {
        __IOM uint32_t REG_2064H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2064H_b;
    };
    union
    {
        __IOM uint32_t REG_2068H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2068H_b;
    };
    union
    {
        __IOM uint32_t REG_206CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_206CH_b;
    };
    union
    {
        __IOM uint32_t REG_2070H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2070H_b;
    };
    union
    {
        __IOM uint32_t REG_2074H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2074H_b;
    };
    union
    {
        __IOM uint32_t REG_2078H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2078H_b;
    };
    union
    {
        __IOM uint32_t REG_207CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_207CH_b;
    };
    union
    {
        __IOM uint32_t REG_2080H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2080H_b;
    };
    union
    {
        __IOM uint32_t REG_2084H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2084H_b;
    };
    union
    {
        __IOM uint32_t REG_2088H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2088H_b;
    };
    union
    {
        __IOM uint32_t REG_208CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_208CH_b;
    };
    union
    {
        __IOM uint32_t REG_2090H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2090H_b;
    };
    union
    {
        __IOM uint32_t REG_2094H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2094H_b;
    };
    union
    {
        __IOM uint32_t REG_2098H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2098H_b;
    };
    union
    {
        __IOM uint32_t REG_209CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_209CH_b;
    };
    union
    {
        __IOM uint32_t REG_20A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20A0H_b;
    };
    union
    {
        __IOM uint32_t REG_20A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20A4H_b;
    };
    union
    {
        __IOM uint32_t REG_20A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20A8H_b;
    };
    union
    {
        __IOM uint32_t REG_20ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20ACH_b;
    };
    union
    {
        __IOM uint32_t REG_20B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20B0H_b;
    };
    union
    {
        __IOM uint32_t REG_20B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20B4H_b;
    };
    union
    {
        __IOM uint32_t REG_20B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20B8H_b;
    };
    union
    {
        __IOM uint32_t REG_20BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20BCH_b;
    };
    union
    {
        __IOM uint32_t REG_20C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20C0H_b;
    };
    union
    {
        __IOM uint32_t REG_20C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20C4H_b;
    };
    union
    {
        __IOM uint32_t REG_20C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20C8H_b;
    };
    union
    {
        __IOM uint32_t REG_20CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20CCH_b;
    };
    union
    {
        __IOM uint32_t REG_20D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20D0H_b;
    };
    union
    {
        __IOM uint32_t REG_20D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20D4H_b;
    };
    union
    {
        __IOM uint32_t REG_20D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20D8H_b;
    };
    union
    {
        __IOM uint32_t REG_20DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20DCH_b;
    };
    union
    {
        __IOM uint32_t REG_20E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20E0H_b;
    };
    union
    {
        __IOM uint32_t REG_20E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20E4H_b;
    };
    union
    {
        __IOM uint32_t REG_20E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20E8H_b;
    };
    union
    {
        __IOM uint32_t REG_20ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20ECH_b;
    };
    union
    {
        __IOM uint32_t REG_20F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20F0H_b;
    };
    union
    {
        __IOM uint32_t REG_20F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20F4H_b;
    };
    union
    {
        __IOM uint32_t REG_20F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20F8H_b;
    };
    union
    {
        __IOM uint32_t REG_20FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_20FCH_b;
    };
    union
    {
        __IOM uint32_t REG_2100H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2100H_b;
    };
    union
    {
        __IOM uint32_t REG_2104H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2104H_b;
    };
    union
    {
        __IOM uint32_t REG_2108H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2108H_b;
    };
    union
    {
        __IOM uint32_t REG_210CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_210CH_b;
    };
    union
    {
        __IOM uint32_t REG_2110H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2110H_b;
    };
    union
    {
        __IOM uint32_t REG_2114H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2114H_b;
    };
    union
    {
        __IOM uint32_t REG_2118H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2118H_b;
    };
    union
    {
        __IOM uint32_t REG_211CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_211CH_b;
    };
    union
    {
        __IOM uint32_t REG_2120H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2120H_b;
    };
    union
    {
        __IOM uint32_t REG_2124H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2124H_b;
    };
    union
    {
        __IOM uint32_t REG_2128H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2128H_b;
    };
    union
    {
        __IOM uint32_t REG_212CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_212CH_b;
    };
    union
    {
        __IOM uint32_t REG_2130H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2130H_b;
    };
    union
    {
        __IOM uint32_t REG_2134H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2134H_b;
    };
    union
    {
        __IOM uint32_t REG_2138H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2138H_b;
    };
    union
    {
        __IOM uint32_t REG_213CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_213CH_b;
    };
    union
    {
        __IOM uint32_t REG_2140H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2140H_b;
    };
    union
    {
        __IOM uint32_t REG_2144H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2144H_b;
    };
    union
    {
        __IOM uint32_t REG_2148H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2148H_b;
    };
    union
    {
        __IOM uint32_t REG_214CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_214CH_b;
    };
    union
    {
        __IOM uint32_t REG_2150H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2150H_b;
    };
    union
    {
        __IOM uint32_t REG_2154H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2154H_b;
    };
    union
    {
        __IOM uint32_t REG_2158H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2158H_b;
    };
    union
    {
        __IOM uint32_t REG_215CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_215CH_b;
    };
    union
    {
        __IOM uint32_t REG_2160H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2160H_b;
    };
    union
    {
        __IOM uint32_t REG_2164H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2164H_b;
    };
    union
    {
        __IOM uint32_t REG_2168H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2168H_b;
    };
    union
    {
        __IOM uint32_t REG_216CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_216CH_b;
    };
    union
    {
        __IOM uint32_t REG_2170H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2170H_b;
    };
    union
    {
        __IOM uint32_t REG_2174H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2174H_b;
    };
    union
    {
        __IOM uint32_t REG_2178H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2178H_b;
    };
    union
    {
        __IOM uint32_t REG_217CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_217CH_b;
    };
    union
    {
        __IOM uint32_t REG_2180H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2180H_b;
    };
    union
    {
        __IOM uint32_t REG_2184H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2184H_b;
    };
    union
    {
        __IOM uint32_t REG_2188H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2188H_b;
    };
    union
    {
        __IOM uint32_t REG_218CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_218CH_b;
    };
    union
    {
        __IOM uint32_t REG_2190H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2190H_b;
    };
    union
    {
        __IOM uint32_t REG_2194H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2194H_b;
    };
    union
    {
        __IOM uint32_t REG_2198H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2198H_b;
    };
    union
    {
        __IOM uint32_t REG_219CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_219CH_b;
    };
    union
    {
        __IOM uint32_t REG_21A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21A0H_b;
    };
    union
    {
        __IOM uint32_t REG_21A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21A4H_b;
    };
    union
    {
        __IOM uint32_t REG_21A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21A8H_b;
    };
    union
    {
        __IOM uint32_t REG_21ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21ACH_b;
    };
    union
    {
        __IOM uint32_t REG_21B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21B0H_b;
    };
    union
    {
        __IOM uint32_t REG_21B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21B4H_b;
    };
    union
    {
        __IOM uint32_t REG_21B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21B8H_b;
    };
    union
    {
        __IOM uint32_t REG_21BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21BCH_b;
    };
    union
    {
        __IOM uint32_t REG_21C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21C0H_b;
    };
    union
    {
        __IOM uint32_t REG_21C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21C4H_b;
    };
    union
    {
        __IOM uint32_t REG_21C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21C8H_b;
    };
    union
    {
        __IOM uint32_t REG_21CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21CCH_b;
    };
    union
    {
        __IOM uint32_t REG_21D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21D0H_b;
    };
    union
    {
        __IOM uint32_t REG_21D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21D4H_b;
    };
    union
    {
        __IOM uint32_t REG_21D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21D8H_b;
    };
    union
    {
        __IOM uint32_t REG_21DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21DCH_b;
    };
    union
    {
        __IOM uint32_t REG_21E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21E0H_b;
    };
    union
    {
        __IOM uint32_t REG_21E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21E4H_b;
    };
    union
    {
        __IOM uint32_t REG_21E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21E8H_b;
    };
    union
    {
        __IOM uint32_t REG_21ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21ECH_b;
    };
    union
    {
        __IOM uint32_t REG_21F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21F0H_b;
    };
    union
    {
        __IOM uint32_t REG_21F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21F4H_b;
    };
    union
    {
        __IOM uint32_t REG_21F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21F8H_b;
    };
    union
    {
        __IOM uint32_t REG_21FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_21FCH_b;
    };
    union
    {
        __IOM uint32_t REG_2200H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2200H_b;
    };
    union
    {
        __IOM uint32_t REG_2204H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2204H_b;
    };
    union
    {
        __IOM uint32_t REG_2208H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2208H_b;
    };
    union
    {
        __IOM uint32_t REG_220CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_220CH_b;
    };
    union
    {
        __IOM uint32_t REG_2210H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2210H_b;
    };
    union
    {
        __IOM uint32_t REG_2214H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2214H_b;
    };
    union
    {
        __IOM uint32_t REG_2218H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2218H_b;
    };
    union
    {
        __IOM uint32_t REG_221CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_221CH_b;
    };
    union
    {
        __IOM uint32_t REG_2220H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2220H_b;
    };
    union
    {
        __IOM uint32_t REG_2224H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2224H_b;
    };
    union
    {
        __IOM uint32_t REG_2228H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2228H_b;
    };
    union
    {
        __IOM uint32_t REG_222CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_222CH_b;
    };
    union
    {
        __IOM uint32_t REG_2230H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2230H_b;
    };
    union
    {
        __IOM uint32_t REG_2234H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2234H_b;
    };
    union
    {
        __IOM uint32_t REG_2238H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2238H_b;
    };
    union
    {
        __IOM uint32_t REG_223CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_223CH_b;
    };
    union
    {
        __IOM uint32_t REG_2240H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2240H_b;
    };
    union
    {
        __IOM uint32_t REG_2244H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2244H_b;
    };
    union
    {
        __IOM uint32_t REG_2248H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2248H_b;
    };
    union
    {
        __IOM uint32_t REG_224CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_224CH_b;
    };
    union
    {
        __IOM uint32_t REG_2250H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2250H_b;
    };
    union
    {
        __IOM uint32_t REG_2254H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2254H_b;
    };
    union
    {
        __IOM uint32_t REG_2258H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2258H_b;
    };
    union
    {
        __IOM uint32_t REG_225CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_225CH_b;
    };
    union
    {
        __IOM uint32_t REG_2260H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2260H_b;
    };
    union
    {
        __IOM uint32_t REG_2264H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2264H_b;
    };
    union
    {
        __IOM uint32_t REG_2268H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2268H_b;
    };
    union
    {
        __IOM uint32_t REG_226CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_226CH_b;
    };
    union
    {
        __IOM uint32_t REG_2270H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2270H_b;
    };
    union
    {
        __IOM uint32_t REG_2274H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2274H_b;
    };
    union
    {
        __IOM uint32_t REG_2278H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2278H_b;
    };
    union
    {
        __IOM uint32_t REG_227CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_227CH_b;
    };
    union
    {
        __IOM uint32_t REG_2280H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2280H_b;
    };
    union
    {
        __IOM uint32_t REG_2284H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2284H_b;
    };
    union
    {
        __IOM uint32_t REG_2288H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2288H_b;
    };
    union
    {
        __IOM uint32_t REG_228CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_228CH_b;
    };
    union
    {
        __IOM uint32_t REG_2290H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2290H_b;
    };
    union
    {
        __IOM uint32_t REG_2294H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2294H_b;
    };
    union
    {
        __IOM uint32_t REG_2298H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2298H_b;
    };
    union
    {
        __IOM uint32_t REG_229CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_229CH_b;
    };
    union
    {
        __IOM uint32_t REG_22A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22A0H_b;
    };
    union
    {
        __IOM uint32_t REG_22A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22A4H_b;
    };
    union
    {
        __IOM uint32_t REG_22A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22A8H_b;
    };
    union
    {
        __IOM uint32_t REG_22ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22ACH_b;
    };
    union
    {
        __IOM uint32_t REG_22B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22B0H_b;
    };
    union
    {
        __IOM uint32_t REG_22B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22B4H_b;
    };
    union
    {
        __IOM uint32_t REG_22B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22B8H_b;
    };
    union
    {
        __IOM uint32_t REG_22BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22BCH_b;
    };
    union
    {
        __IOM uint32_t REG_22C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22C0H_b;
    };
    union
    {
        __IOM uint32_t REG_22C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22C4H_b;
    };
    union
    {
        __IOM uint32_t REG_22C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22C8H_b;
    };
    union
    {
        __IOM uint32_t REG_22CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22CCH_b;
    };
    union
    {
        __IOM uint32_t REG_22D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22D0H_b;
    };
    union
    {
        __IOM uint32_t REG_22D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22D4H_b;
    };
    union
    {
        __IOM uint32_t REG_22D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22D8H_b;
    };
    union
    {
        __IOM uint32_t REG_22DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22DCH_b;
    };
    union
    {
        __IOM uint32_t REG_22E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22E0H_b;
    };
    union
    {
        __IOM uint32_t REG_22E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22E4H_b;
    };
    union
    {
        __IOM uint32_t REG_22E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22E8H_b;
    };
    union
    {
        __IOM uint32_t REG_22ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22ECH_b;
    };
    union
    {
        __IOM uint32_t REG_22F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22F0H_b;
    };
    union
    {
        __IOM uint32_t REG_22F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22F4H_b;
    };
    union
    {
        __IOM uint32_t REG_22F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22F8H_b;
    };
    union
    {
        __IOM uint32_t REG_22FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_22FCH_b;
    };
    union
    {
        __IOM uint32_t REG_2300H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2300H_b;
    };
    union
    {
        __IOM uint32_t REG_2304H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2304H_b;
    };
    union
    {
        __IOM uint32_t REG_2308H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2308H_b;
    };
    union
    {
        __IOM uint32_t REG_230CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_230CH_b;
    };
    union
    {
        __IOM uint32_t REG_2310H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2310H_b;
    };
    union
    {
        __IOM uint32_t REG_2314H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2314H_b;
    };
    union
    {
        __IOM uint32_t REG_2318H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2318H_b;
    };
    union
    {
        __IOM uint32_t REG_231CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_231CH_b;
    };
    union
    {
        __IOM uint32_t REG_2320H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2320H_b;
    };
    union
    {
        __IOM uint32_t REG_2324H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2324H_b;
    };
    union
    {
        __IOM uint32_t REG_2328H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2328H_b;
    };
    union
    {
        __IOM uint32_t REG_232CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_232CH_b;
    };
    union
    {
        __IOM uint32_t REG_2330H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2330H_b;
    };
    union
    {
        __IOM uint32_t REG_2334H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2334H_b;
    };
    union
    {
        __IOM uint32_t REG_2338H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2338H_b;
    };
    union
    {
        __IOM uint32_t REG_233CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_233CH_b;
    };
    union
    {
        __IOM uint32_t REG_2340H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2340H_b;
    };
    union
    {
        __IOM uint32_t REG_2344H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2344H_b;
    };
    union
    {
        __IOM uint32_t REG_2348H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2348H_b;
    };
    union
    {
        __IOM uint32_t REG_234CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_234CH_b;
    };
    union
    {
        __IOM uint32_t REG_2350H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2350H_b;
    };
    union
    {
        __IOM uint32_t REG_2354H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2354H_b;
    };
    union
    {
        __IOM uint32_t REG_2358H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2358H_b;
    };
    union
    {
        __IOM uint32_t REG_235CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_235CH_b;
    };
    union
    {
        __IOM uint32_t REG_2360H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2360H_b;
    };
    union
    {
        __IOM uint32_t REG_2364H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2364H_b;
    };
    union
    {
        __IOM uint32_t REG_2368H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2368H_b;
    };
    union
    {
        __IOM uint32_t REG_236CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_236CH_b;
    };
    union
    {
        __IOM uint32_t REG_2370H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2370H_b;
    };
    union
    {
        __IOM uint32_t REG_2374H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2374H_b;
    };
    union
    {
        __IOM uint32_t REG_2378H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2378H_b;
    };
    union
    {
        __IOM uint32_t REG_237CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_237CH_b;
    };
    union
    {
        __IOM uint32_t REG_2380H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2380H_b;
    };
    union
    {
        __IOM uint32_t REG_2384H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2384H_b;
    };
    union
    {
        __IOM uint32_t REG_2388H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2388H_b;
    };
    union
    {
        __IOM uint32_t REG_238CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_238CH_b;
    };
    union
    {
        __IOM uint32_t REG_2390H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2390H_b;
    };
    union
    {
        __IOM uint32_t REG_2394H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2394H_b;
    };
    union
    {
        __IOM uint32_t REG_2398H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2398H_b;
    };
    union
    {
        __IOM uint32_t REG_239CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_239CH_b;
    };
    union
    {
        __IOM uint32_t REG_23A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23A0H_b;
    };
    union
    {
        __IOM uint32_t REG_23A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23A4H_b;
    };
    union
    {
        __IOM uint32_t REG_23A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23A8H_b;
    };
    union
    {
        __IOM uint32_t REG_23ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23ACH_b;
    };
    union
    {
        __IOM uint32_t REG_23B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23B0H_b;
    };
    union
    {
        __IOM uint32_t REG_23B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23B4H_b;
    };
    union
    {
        __IOM uint32_t REG_23B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23B8H_b;
    };
    union
    {
        __IOM uint32_t REG_23BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23BCH_b;
    };
    union
    {
        __IOM uint32_t REG_23C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23C0H_b;
    };
    union
    {
        __IOM uint32_t REG_23C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23C4H_b;
    };
    union
    {
        __IOM uint32_t REG_23C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23C8H_b;
    };
    union
    {
        __IOM uint32_t REG_23CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23CCH_b;
    };
    union
    {
        __IOM uint32_t REG_23D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23D0H_b;
    };
    union
    {
        __IOM uint32_t REG_23D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23D4H_b;
    };
    union
    {
        __IOM uint32_t REG_23D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23D8H_b;
    };
    union
    {
        __IOM uint32_t REG_23DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23DCH_b;
    };
    union
    {
        __IOM uint32_t REG_23E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23E0H_b;
    };
    union
    {
        __IOM uint32_t REG_23E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23E4H_b;
    };
    union
    {
        __IOM uint32_t REG_23E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23E8H_b;
    };
    union
    {
        __IOM uint32_t REG_23ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23ECH_b;
    };
    union
    {
        __IOM uint32_t REG_23F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23F0H_b;
    };
    union
    {
        __IOM uint32_t REG_23F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23F4H_b;
    };
    union
    {
        __IOM uint32_t REG_23F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23F8H_b;
    };
    union
    {
        __IOM uint32_t REG_23FCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_23FCH_b;
    };
    union
    {
        __IOM uint32_t REG_2400H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2400H_b;
    };
    union
    {
        __IOM uint32_t REG_2404H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2404H_b;
    };
    union
    {
        __IOM uint32_t REG_2408H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2408H_b;
    };
    union
    {
        __IOM uint32_t REG_240CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_240CH_b;
    };
    union
    {
        __IOM uint32_t REG_2410H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2410H_b;
    };
    union
    {
        __IOM uint32_t REG_2414H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2414H_b;
    };
    union
    {
        __IOM uint32_t REG_2418H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2418H_b;
    };
    union
    {
        __IOM uint32_t REG_241CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_241CH_b;
    };
    union
    {
        __IOM uint32_t REG_2420H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2420H_b;
    };
    union
    {
        __IOM uint32_t REG_2424H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2424H_b;
    };
    union
    {
        __IOM uint32_t REG_2428H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2428H_b;
    };
    union
    {
        __IOM uint32_t REG_242CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_242CH_b;
    };
    union
    {
        __IOM uint32_t REG_2430H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2430H_b;
    };
    union
    {
        __IOM uint32_t REG_2434H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2434H_b;
    };
    union
    {
        __IOM uint32_t REG_2438H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2438H_b;
    };
    union
    {
        __IOM uint32_t REG_243CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_243CH_b;
    };
    union
    {
        __IOM uint32_t REG_2440H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2440H_b;
    };
    union
    {
        __IOM uint32_t REG_2444H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2444H_b;
    };
    union
    {
        __IOM uint32_t REG_2448H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2448H_b;
    };
    union
    {
        __IOM uint32_t REG_244CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_244CH_b;
    };
    union
    {
        __IOM uint32_t REG_2450H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2450H_b;
    };
    union
    {
        __IOM uint32_t REG_2454H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2454H_b;
    };
    union
    {
        __IOM uint32_t REG_2458H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2458H_b;
    };
    union
    {
        __IOM uint32_t REG_245CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_245CH_b;
    };
    union
    {
        __IOM uint32_t REG_2460H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2460H_b;
    };
    union
    {
        __IOM uint32_t REG_2464H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2464H_b;
    };
    union
    {
        __IOM uint32_t REG_2468H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2468H_b;
    };
    union
    {
        __IOM uint32_t REG_246CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_246CH_b;
    };
    union
    {
        __IOM uint32_t REG_2470H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2470H_b;
    };
    union
    {
        __IOM uint32_t REG_2474H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2474H_b;
    };
    union
    {
        __IOM uint32_t REG_2478H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2478H_b;
    };
    union
    {
        __IOM uint32_t REG_247CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_247CH_b;
    };
    union
    {
        __IOM uint32_t REG_2480H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2480H_b;
    };
    union
    {
        __IOM uint32_t REG_2484H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2484H_b;
    };
    union
    {
        __IOM uint32_t REG_2488H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2488H_b;
    };
    union
    {
        __IOM uint32_t REG_248CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_248CH_b;
    };
    union
    {
        __IOM uint32_t REG_2490H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2490H_b;
    };
    union
    {
        __IOM uint32_t REG_2494H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2494H_b;
    };
    union
    {
        __IOM uint32_t REG_2498H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_2498H_b;
    };
    union
    {
        __IOM uint32_t REG_249CH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_249CH_b;
    };
    union
    {
        __IOM uint32_t REG_24A0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24A0H_b;
    };
    union
    {
        __IOM uint32_t REG_24A4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24A4H_b;
    };
    union
    {
        __IOM uint32_t REG_24A8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24A8H_b;
    };
    union
    {
        __IOM uint32_t REG_24ACH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24ACH_b;
    };
    union
    {
        __IOM uint32_t REG_24B0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24B0H_b;
    };
    union
    {
        __IOM uint32_t REG_24B4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24B4H_b;
    };
    union
    {
        __IOM uint32_t REG_24B8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24B8H_b;
    };
    union
    {
        __IOM uint32_t REG_24BCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24BCH_b;
    };
    union
    {
        __IOM uint32_t REG_24C0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24C0H_b;
    };
    union
    {
        __IOM uint32_t REG_24C4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24C4H_b;
    };
    union
    {
        __IOM uint32_t REG_24C8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24C8H_b;
    };
    union
    {
        __IOM uint32_t REG_24CCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24CCH_b;
    };
    union
    {
        __IOM uint32_t REG_24D0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24D0H_b;
    };
    union
    {
        __IOM uint32_t REG_24D4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24D4H_b;
    };
    union
    {
        __IOM uint32_t REG_24D8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24D8H_b;
    };
    union
    {
        __IOM uint32_t REG_24DCH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24DCH_b;
    };
    union
    {
        __IOM uint32_t REG_24E0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24E0H_b;
    };
    union
    {
        __IOM uint32_t REG_24E4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24E4H_b;
    };
    union
    {
        __IOM uint32_t REG_24E8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24E8H_b;
    };
    union
    {
        __IOM uint32_t REG_24ECH;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24ECH_b;
    };
    union
    {
        __IOM uint32_t REG_24F0H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24F0H_b;
    };
    union
    {
        __IOM uint32_t REG_24F4H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24F4H_b;
    };
    union
    {
        __IOM uint32_t REG_24F8H;
        struct
        {
            __IOM uint32_t B0  : 1;
            __IOM uint32_t B1  : 1;
            __IOM uint32_t B2  : 1;
            __IOM uint32_t B3  : 1;
            __IOM uint32_t B4  : 1;
            __IOM uint32_t B5  : 1;
            __IOM uint32_t B6  : 1;
            __IOM uint32_t B7  : 1;
            __IOM uint32_t B8  : 1;
            __IOM uint32_t B9  : 1;
            __IOM uint32_t B10 : 1;
            __IOM uint32_t B11 : 1;
            __IOM uint32_t B12 : 1;
            __IOM uint32_t B13 : 1;
            __IOM uint32_t B14 : 1;
            __IOM uint32_t B15 : 1;
            __IOM uint32_t B16 : 1;
            __IOM uint32_t B17 : 1;
            __IOM uint32_t B18 : 1;
            __IOM uint32_t B19 : 1;
            __IOM uint32_t B20 : 1;
            __IOM uint32_t B21 : 1;
            __IOM uint32_t B22 : 1;
            __IOM uint32_t B23 : 1;
            __IOM uint32_t B24 : 1;
            __IOM uint32_t B25 : 1;
            __IOM uint32_t B26 : 1;
            __IOM uint32_t B27 : 1;
            __IOM uint32_t B28 : 1;
            __IOM uint32_t B29 : 1;
            __IOM uint32_t B30 : 1;
            __IOM uint32_t B31 : 1;
        } REG_24F8H_b;
    };
} SCE_Type;

// for bit access

/* --------------------------------  SCE.REG_xxxH  -------------------------------- */
#define SCE_REG_0_Pos     0                          /*!< SCE REG_xxxH: bit 0 Position       */
#define SCE_REG_0_Msk     (0x01UL << SCE_REG_0_Pos)  /*!< SCE REG_xxxH: bit 0 Mask           */
#define SCE_REG_1_Pos     1                          /*!< SCE REG_xxxH: bit 1 Position       */
#define SCE_REG_1_Msk     (0x01UL << SCE_REG_1_Pos)  /*!< SCE REG_xxxH: bit 1 Mask           */
#define SCE_REG_2_Pos     2                          /*!< SCE REG_xxxH: bit 2 Position       */
#define SCE_REG_2_Msk     (0x01UL << SCE_REG_2_Pos)  /*!< SCE REG_xxxH: bit 2 Mask           */
#define SCE_REG_3_Pos     3                          /*!< SCE REG_xxxH: bit 3 Position       */
#define SCE_REG_3_Msk     (0x01UL << SCE_REG_3_Pos)  /*!< SCE REG_xxxH: bit 3 Mask           */
#define SCE_REG_4_Pos     4                          /*!< SCE REG_xxxH: bit 4 Position       */
#define SCE_REG_4_Msk     (0x01UL << SCE_REG_4_Pos)  /*!< SCE REG_xxxH: bit 4 Mask           */
#define SCE_REG_5_Pos     5                          /*!< SCE REG_xxxH: bit 5 Position       */
#define SCE_REG_5_Msk     (0x01UL << SCE_REG_5_Pos)  /*!< SCE REG_xxxH: bit 5 Mask           */
#define SCE_REG_6_Pos     6                          /*!< SCE REG_xxxH: bit 6 Position       */
#define SCE_REG_6_Msk     (0x01UL << SCE_REG_6_Pos)  /*!< SCE REG_xxxH: bit 6 Mask           */
#define SCE_REG_7_Pos     7                          /*!< SCE REG_xxxH: bit 7 Position       */
#define SCE_REG_7_Msk     (0x01UL << SCE_REG_7_Pos)  /*!< SCE REG_xxxH: bit 7 Mask           */
#define SCE_REG_8_Pos     8                          /*!< SCE REG_xxxH: bit 8 Position       */
#define SCE_REG_8_Msk     (0x01UL << SCE_REG_8_Pos)  /*!< SCE REG_xxxH: bit 8 Mask           */
#define SCE_REG_9_Pos     9                          /*!< SCE REG_xxxH: bit 9 Position       */
#define SCE_REG_9_Msk     (0x01UL << SCE_REG_9_Pos)  /*!< SCE REG_xxxH: bit 9 Mask           */
#define SCE_REG_10_Pos    10                         /*!< SCE REG_xxxH: bit 10 Position       */
#define SCE_REG_10_Msk    (0x01UL << SCE_REG_10_Pos) /*!< SCE REG_xxxH: bit 10 Mask           */
#define SCE_REG_11_Pos    11                         /*!< SCE REG_xxxH: bit 11 Position       */
#define SCE_REG_11_Msk    (0x01UL << SCE_REG_11_Pos) /*!< SCE REG_xxxH: bit 11 Mask           */
#define SCE_REG_12_Pos    12                         /*!< SCE REG_xxxH: bit 12 Position       */
#define SCE_REG_12_Msk    (0x01UL << SCE_REG_12_Pos) /*!< SCE REG_xxxH: bit 12 Mask           */
#define SCE_REG_13_Pos    13                         /*!< SCE REG_xxxH: bit 13 Position       */
#define SCE_REG_13_Msk    (0x01UL << SCE_REG_13_Pos) /*!< SCE REG_xxxH: bit 13 Mask           */
#define SCE_REG_14_Pos    14                         /*!< SCE REG_xxxH: bit 14 Position       */
#define SCE_REG_14_Msk    (0x01UL << SCE_REG_14_Pos) /*!< SCE REG_xxxH: bit 14 Mask           */
#define SCE_REG_15_Pos    15                         /*!< SCE REG_xxxH: bit 15 Position       */
#define SCE_REG_15_Msk    (0x01UL << SCE_REG_15_Pos) /*!< SCE REG_xxxH: bit 15 Mask           */
#define SCE_REG_16_Pos    16                         /*!< SCE REG_xxxH: bit 16 Position       */
#define SCE_REG_16_Msk    (0x01UL << SCE_REG_16_Pos) /*!< SCE REG_xxxH: bit 16 Mask           */
#define SCE_REG_17_Pos    17                         /*!< SCE REG_xxxH: bit 17 Position       */
#define SCE_REG_17_Msk    (0x01UL << SCE_REG_17_Pos) /*!< SCE REG_xxxH: bit 17 Mask           */
#define SCE_REG_18_Pos    18                         /*!< SCE REG_xxxH: bit 18 Position       */
#define SCE_REG_18_Msk    (0x01UL << SCE_REG_18_Pos) /*!< SCE REG_xxxH: bit 18 Mask           */
#define SCE_REG_19_Pos    19                         /*!< SCE REG_xxxH: bit 19 Position       */
#define SCE_REG_19_Msk    (0x01UL << SCE_REG_19_Pos) /*!< SCE REG_xxxH: bit 19 Mask           */
#define SCE_REG_20_Pos    20                         /*!< SCE REG_xxxH: bit 20 Position       */
#define SCE_REG_20_Msk    (0x01UL << SCE_REG_20_Pos) /*!< SCE REG_xxxH: bit 20 Mask           */
#define SCE_REG_21_Pos    21                         /*!< SCE REG_xxxH: bit 21 Position       */
#define SCE_REG_21_Msk    (0x01UL << SCE_REG_21_Pos) /*!< SCE REG_xxxH: bit 21 Mask           */
#define SCE_REG_22_Pos    22                         /*!< SCE REG_xxxH: bit 22 Position       */
#define SCE_REG_22_Msk    (0x01UL << SCE_REG_22_Pos) /*!< SCE REG_xxxH: bit 22 Mask           */
#define SCE_REG_23_Pos    23                         /*!< SCE REG_xxxH: bit 23 Position       */
#define SCE_REG_23_Msk    (0x01UL << SCE_REG_23_Pos) /*!< SCE REG_xxxH: bit 23 Mask           */
#define SCE_REG_24_Pos    24                         /*!< SCE REG_xxxH: bit 24 Position       */
#define SCE_REG_24_Msk    (0x01UL << SCE_REG_24_Pos) /*!< SCE REG_xxxH: bit 24 Mask           */
#define SCE_REG_25_Pos    25                         /*!< SCE REG_xxxH: bit 25 Position       */
#define SCE_REG_25_Msk    (0x01UL << SCE_REG_25_Pos) /*!< SCE REG_xxxH: bit 25 Mask           */
#define SCE_REG_26_Pos    26                         /*!< SCE REG_xxxH: bit 26 Position       */
#define SCE_REG_26_Msk    (0x01UL << SCE_REG_26_Pos) /*!< SCE REG_xxxH: bit 26 Mask           */
#define SCE_REG_27_Pos    27                         /*!< SCE REG_xxxH: bit 27 Position       */
#define SCE_REG_27_Msk    (0x01UL << SCE_REG_27_Pos) /*!< SCE REG_xxxH: bit 27 Mask           */
#define SCE_REG_28_Pos    28                         /*!< SCE REG_xxxH: bit 28 Position       */
#define SCE_REG_28_Msk    (0x01UL << SCE_REG_28_Pos) /*!< SCE REG_xxxH: bit 28 Mask           */
#define SCE_REG_29_Pos    29                         /*!< SCE REG_xxxH: bit 29 Position       */
#define SCE_REG_29_Msk    (0x01UL << SCE_REG_29_Pos) /*!< SCE REG_xxxH: bit 29 Mask           */
#define SCE_REG_30_Pos    30                         /*!< SCE REG_xxxH: bit 30 Position       */
#define SCE_REG_30_Msk    (0x01UL << SCE_REG_30_Pos) /*!< SCE REG_xxxH: bit 30 Mask           */
#define SCE_REG_31_Pos    31                         /*!< SCE REG_xxxH: bit 31 Position       */
#define SCE_REG_31_Msk    (0x01UL << SCE_REG_31_Pos) /*!< SCE REG_xxxH: bit 31 Mask           */

/* ================================================================================ */
/* ================             Peripheral declaration             ================ */
/* ================================================================================ */

#define SCE    ((SCE_Type *) SCE_BASE)

// macro definishion

#define SCE_DELAY(delay)                                      \
    for (volatile uint32_t count = 0; count < delay; count++) \
    {                                                         \
        ;                                                     \
    }

// [R RD 1 B]
#define  RD1_PROG(regName) \
    (SCE->regName)

// [R WR 1 B]
#define  WR1_PROG(regName, value) \
    SCE->regName = value

// [R WR 2 B0 B1]
#define  WR2_PROG(regName, value0, value1) \
    WR1_PROG(regName, value0);             \
    WR1_PROG(regName, value1);

// [R WR 3 B0 B1 B2]
#define  WR3_PROG(regName, value0, value1, value2) \
    WR1_PROG(regName, value0);                     \
    WR1_PROG(regName, value1);                     \
    WR1_PROG(regName, value2)

// [R WR 4 B0 B1 B2 B3]
#define  WR4_PROG(regName, value0, value1, value2, value3) \
    WR1_PROG(regName, value0);                             \
    WR1_PROG(regName, value1);                             \
    WR1_PROG(regName, value2);                             \
    WR1_PROG(regName, value3)

// [R WR 16 B0 B1 B2 B3 B4 B5 B6 B7 B8 B9 B10 B11 B12 B13 B14 B15]
#define  WR16_PROG(regName,     \
                   value0,      \
                   value1,      \
                   value2,      \
                   value3,      \
                   value4,      \
                   value5,      \
                   value6,      \
                   value7,      \
                   value8,      \
                   value9,      \
                   value10,     \
                   value11,     \
                   value12,     \
                   value13,     \
                   value14,     \
                   value15)     \
    WR1_PROG(regName, value0);  \
    WR1_PROG(regName, value1);  \
    WR1_PROG(regName, value2);  \
    WR1_PROG(regName, value3);  \
    WR1_PROG(regName, value4);  \
    WR1_PROG(regName, value5);  \
    WR1_PROG(regName, value6);  \
    WR1_PROG(regName, value7);  \
    WR1_PROG(regName, value8);  \
    WR1_PROG(regName, value9);  \
    WR1_PROG(regName, value10); \
    WR1_PROG(regName, value11); \
    WR1_PROG(regName, value12); \
    WR1_PROG(regName, value13); \
    WR1_PROG(regName, value14); \
    WR1_PROG(regName, value15)

// [R RD 1 B]
#define  RD1_EVAL(regName, value) \
    if (SCE->regname != value)    \
        return FSP_ERR_CRYPTO_SCE_FAIL

// [R CHK_STATUS A B]
#define  CHCK_STS(regName, bitPos, value) \
    (((SCE->regName & (0x01UL << bitPos)) >> bitPos) == value)

// [R CHK_REG A (!= B)]
#define  RD1_MASK(regName, maskValue) \
    (SCE->regName & maskValue)

// [R WAIT_STATUS A B]
#define  WAIT_STS(regName, bitPos, value) \
    while (!CHCK_STS(regName, bitPos, value))

// [R WR 1 MEM[Ofs]]
#define  WR1_ADDR(regName, addr) \
    SCE->regName = *(addr)

// [R WR 2 MEM[Ofs]]
#define  WR2_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1);

// [R WR 3 MEM[Ofs]]
#define  WR3_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1); \
    WR1_ADDR(regName, (addr) + 2);

// [R WR 4 MEM[Ofs]]
#define  WR4_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1); \
    WR1_ADDR(regName, (addr) + 2); \
    WR1_ADDR(regName, (addr) + 3)

// [R WR 5 MEM[Ofs]]
#define  WR5_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1); \
    WR1_ADDR(regName, (addr) + 2); \
    WR1_ADDR(regName, (addr) + 3); \
    WR1_ADDR(regName, (addr) + 4)

// [R WR 6 MEM[Ofs]]
#define  WR6_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1); \
    WR1_ADDR(regName, (addr) + 2); \
    WR1_ADDR(regName, (addr) + 3); \
    WR1_ADDR(regName, (addr) + 4); \
    WR1_ADDR(regName, (addr) + 5)

// [R WR 7 MEM[Ofs]]
#define  WR7_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1); \
    WR1_ADDR(regName, (addr) + 2); \
    WR1_ADDR(regName, (addr) + 3); \
    WR1_ADDR(regName, (addr) + 4); \
    WR1_ADDR(regName, (addr) + 5); \
    WR1_ADDR(regName, (addr) + 6)

// [R WR 8 MEM[Ofs]]
#define  WR8_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0); \
    WR1_ADDR(regName, (addr) + 1); \
    WR1_ADDR(regName, (addr) + 2); \
    WR1_ADDR(regName, (addr) + 3); \
    WR1_ADDR(regName, (addr) + 4); \
    WR1_ADDR(regName, (addr) + 5); \
    WR1_ADDR(regName, (addr) + 6); \
    WR1_ADDR(regName, (addr) + 7)

// [R WR 12 MEM[Ofs]]
#define  WR12_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0);  \
    WR1_ADDR(regName, (addr) + 1);  \
    WR1_ADDR(regName, (addr) + 2);  \
    WR1_ADDR(regName, (addr) + 3);  \
    WR1_ADDR(regName, (addr) + 4);  \
    WR1_ADDR(regName, (addr) + 5);  \
    WR1_ADDR(regName, (addr) + 6);  \
    WR1_ADDR(regName, (addr) + 7);  \
    WR1_ADDR(regName, (addr) + 8);  \
    WR1_ADDR(regName, (addr) + 9);  \
    WR1_ADDR(regName, (addr) + 10); \
    WR1_ADDR(regName, (addr) + 11)

// [R WR 16 MEM[Ofs]]
#define  WR16_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0);  \
    WR1_ADDR(regName, (addr) + 1);  \
    WR1_ADDR(regName, (addr) + 2);  \
    WR1_ADDR(regName, (addr) + 3);  \
    WR1_ADDR(regName, (addr) + 4);  \
    WR1_ADDR(regName, (addr) + 5);  \
    WR1_ADDR(regName, (addr) + 6);  \
    WR1_ADDR(regName, (addr) + 7);  \
    WR1_ADDR(regName, (addr) + 8);  \
    WR1_ADDR(regName, (addr) + 9);  \
    WR1_ADDR(regName, (addr) + 10); \
    WR1_ADDR(regName, (addr) + 11); \
    WR1_ADDR(regName, (addr) + 12); \
    WR1_ADDR(regName, (addr) + 13); \
    WR1_ADDR(regName, (addr) + 14); \
    WR1_ADDR(regName, (addr) + 15)

// [R WR 32 MEM[Ofs]]
#define  WR32_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0);  \
    WR1_ADDR(regName, (addr) + 1);  \
    WR1_ADDR(regName, (addr) + 2);  \
    WR1_ADDR(regName, (addr) + 3);  \
    WR1_ADDR(regName, (addr) + 4);  \
    WR1_ADDR(regName, (addr) + 5);  \
    WR1_ADDR(regName, (addr) + 6);  \
    WR1_ADDR(regName, (addr) + 7);  \
    WR1_ADDR(regName, (addr) + 8);  \
    WR1_ADDR(regName, (addr) + 9);  \
    WR1_ADDR(regName, (addr) + 10); \
    WR1_ADDR(regName, (addr) + 11); \
    WR1_ADDR(regName, (addr) + 12); \
    WR1_ADDR(regName, (addr) + 13); \
    WR1_ADDR(regName, (addr) + 14); \
    WR1_ADDR(regName, (addr) + 15); \
    WR1_ADDR(regName, (addr) + 16); \
    WR1_ADDR(regName, (addr) + 17); \
    WR1_ADDR(regName, (addr) + 18); \
    WR1_ADDR(regName, (addr) + 19); \
    WR1_ADDR(regName, (addr) + 20); \
    WR1_ADDR(regName, (addr) + 21); \
    WR1_ADDR(regName, (addr) + 22); \
    WR1_ADDR(regName, (addr) + 23); \
    WR1_ADDR(regName, (addr) + 24); \
    WR1_ADDR(regName, (addr) + 25); \
    WR1_ADDR(regName, (addr) + 26); \
    WR1_ADDR(regName, (addr) + 27); \
    WR1_ADDR(regName, (addr) + 28); \
    WR1_ADDR(regName, (addr) + 29); \
    WR1_ADDR(regName, (addr) + 30); \
    WR1_ADDR(regName, (addr) + 31)

// [R WR 64 MEM[Ofs]]
#define  WR64_ADDR(regName, addr)   \
    WR1_ADDR(regName, (addr) + 0);  \
    WR1_ADDR(regName, (addr) + 1);  \
    WR1_ADDR(regName, (addr) + 2);  \
    WR1_ADDR(regName, (addr) + 3);  \
    WR1_ADDR(regName, (addr) + 4);  \
    WR1_ADDR(regName, (addr) + 5);  \
    WR1_ADDR(regName, (addr) + 6);  \
    WR1_ADDR(regName, (addr) + 7);  \
    WR1_ADDR(regName, (addr) + 8);  \
    WR1_ADDR(regName, (addr) + 9);  \
    WR1_ADDR(regName, (addr) + 10); \
    WR1_ADDR(regName, (addr) + 11); \
    WR1_ADDR(regName, (addr) + 12); \
    WR1_ADDR(regName, (addr) + 13); \
    WR1_ADDR(regName, (addr) + 14); \
    WR1_ADDR(regName, (addr) + 15); \
    WR1_ADDR(regName, (addr) + 16); \
    WR1_ADDR(regName, (addr) + 17); \
    WR1_ADDR(regName, (addr) + 18); \
    WR1_ADDR(regName, (addr) + 19); \
    WR1_ADDR(regName, (addr) + 20); \
    WR1_ADDR(regName, (addr) + 21); \
    WR1_ADDR(regName, (addr) + 22); \
    WR1_ADDR(regName, (addr) + 23); \
    WR1_ADDR(regName, (addr) + 24); \
    WR1_ADDR(regName, (addr) + 25); \
    WR1_ADDR(regName, (addr) + 26); \
    WR1_ADDR(regName, (addr) + 27); \
    WR1_ADDR(regName, (addr) + 28); \
    WR1_ADDR(regName, (addr) + 29); \
    WR1_ADDR(regName, (addr) + 30); \
    WR1_ADDR(regName, (addr) + 31); \
    WR1_ADDR(regName, (addr) + 32); \
    WR1_ADDR(regName, (addr) + 33); \
    WR1_ADDR(regName, (addr) + 34); \
    WR1_ADDR(regName, (addr) + 35); \
    WR1_ADDR(regName, (addr) + 36); \
    WR1_ADDR(regName, (addr) + 37); \
    WR1_ADDR(regName, (addr) + 38); \
    WR1_ADDR(regName, (addr) + 39); \
    WR1_ADDR(regName, (addr) + 40); \
    WR1_ADDR(regName, (addr) + 41); \
    WR1_ADDR(regName, (addr) + 42); \
    WR1_ADDR(regName, (addr) + 43); \
    WR1_ADDR(regName, (addr) + 44); \
    WR1_ADDR(regName, (addr) + 45); \
    WR1_ADDR(regName, (addr) + 46); \
    WR1_ADDR(regName, (addr) + 47); \
    WR1_ADDR(regName, (addr) + 48); \
    WR1_ADDR(regName, (addr) + 49); \
    WR1_ADDR(regName, (addr) + 50); \
    WR1_ADDR(regName, (addr) + 51); \
    WR1_ADDR(regName, (addr) + 52); \
    WR1_ADDR(regName, (addr) + 53); \
    WR1_ADDR(regName, (addr) + 54); \
    WR1_ADDR(regName, (addr) + 55); \
    WR1_ADDR(regName, (addr) + 56); \
    WR1_ADDR(regName, (addr) + 57); \
    WR1_ADDR(regName, (addr) + 58); \
    WR1_ADDR(regName, (addr) + 59); \
    WR1_ADDR(regName, (addr) + 60); \
    WR1_ADDR(regName, (addr) + 61); \
    WR1_ADDR(regName, (addr) + 62); \
    WR1_ADDR(regName, (addr) + 63)

// [R RD 1 MEM[Ofs]]
#define  RD1_ADDR(regName, addr) \
    *((addr) + 0) = SCE->regName;

// [R RD 2 MEM[Ofs]]
#define  RD2_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName;

// [R RD 3 MEM[Ofs]]
#define  RD3_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName; \
    *((addr) + 2) = SCE->regName;

// [R RD 4 MEM[Ofs]]
#define  RD4_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName; \
    *((addr) + 2) = SCE->regName; \
    *((addr) + 3) = SCE->regName

// [R RD 5 MEM[Ofs]]
#define  RD5_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName; \
    *((addr) + 2) = SCE->regName; \
    *((addr) + 3) = SCE->regName; \
    *((addr) + 4) = SCE->regName

// [R RD 6 MEM[Ofs]]
#define  RD6_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName; \
    *((addr) + 2) = SCE->regName; \
    *((addr) + 3) = SCE->regName; \
    *((addr) + 4) = SCE->regName; \
    *((addr) + 5) = SCE->regName

// [R RD 7 MEM[Ofs]]
#define  RD7_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName; \
    *((addr) + 2) = SCE->regName; \
    *((addr) + 3) = SCE->regName; \
    *((addr) + 4) = SCE->regName; \
    *((addr) + 5) = SCE->regName; \
    *((addr) + 6) = SCE->regName

// [R RD 8 MEM[Ofs]]
#define  RD8_ADDR(regName, addr)  \
    *((addr) + 0) = SCE->regName; \
    *((addr) + 1) = SCE->regName; \
    *((addr) + 2) = SCE->regName; \
    *((addr) + 3) = SCE->regName; \
    *((addr) + 4) = SCE->regName; \
    *((addr) + 5) = SCE->regName; \
    *((addr) + 6) = SCE->regName; \
    *((addr) + 7) = SCE->regName

// [R RD 12 MEM[Ofs]]
#define  RD12_ADDR(regName, addr)  \
    *((addr) + 0)  = SCE->regName; \
    *((addr) + 1)  = SCE->regName; \
    *((addr) + 2)  = SCE->regName; \
    *((addr) + 3)  = SCE->regName; \
    *((addr) + 4)  = SCE->regName; \
    *((addr) + 5)  = SCE->regName; \
    *((addr) + 6)  = SCE->regName; \
    *((addr) + 7)  = SCE->regName; \
    *((addr) + 8)  = SCE->regName; \
    *((addr) + 9)  = SCE->regName; \
    *((addr) + 10) = SCE->regName; \
    *((addr) + 11) = SCE->regName

// [R RD 16 MEM[Ofs]]
#define  RD16_ADDR(regName, addr)  \
    *((addr) + 0)  = SCE->regName; \
    *((addr) + 1)  = SCE->regName; \
    *((addr) + 2)  = SCE->regName; \
    *((addr) + 3)  = SCE->regName; \
    *((addr) + 4)  = SCE->regName; \
    *((addr) + 5)  = SCE->regName; \
    *((addr) + 6)  = SCE->regName; \
    *((addr) + 7)  = SCE->regName; \
    *((addr) + 8)  = SCE->regName; \
    *((addr) + 9)  = SCE->regName; \
    *((addr) + 10) = SCE->regName; \
    *((addr) + 11) = SCE->regName; \
    *((addr) + 12) = SCE->regName; \
    *((addr) + 13) = SCE->regName; \
    *((addr) + 14) = SCE->regName; \
    *((addr) + 15) = SCE->regName

// [R RD 32 MEM[Ofs]]
#define  RD32_ADDR(regName, addr)  \
    *((addr) + 0)  = SCE->regName; \
    *((addr) + 1)  = SCE->regName; \
    *((addr) + 2)  = SCE->regName; \
    *((addr) + 3)  = SCE->regName; \
    *((addr) + 4)  = SCE->regName; \
    *((addr) + 5)  = SCE->regName; \
    *((addr) + 6)  = SCE->regName; \
    *((addr) + 7)  = SCE->regName; \
    *((addr) + 8)  = SCE->regName; \
    *((addr) + 9)  = SCE->regName; \
    *((addr) + 10) = SCE->regName; \
    *((addr) + 11) = SCE->regName; \
    *((addr) + 12) = SCE->regName; \
    *((addr) + 13) = SCE->regName; \
    *((addr) + 14) = SCE->regName; \
    *((addr) + 15) = SCE->regName; \
    *((addr) + 16) = SCE->regName; \
    *((addr) + 17) = SCE->regName; \
    *((addr) + 18) = SCE->regName; \
    *((addr) + 19) = SCE->regName; \
    *((addr) + 20) = SCE->regName; \
    *((addr) + 21) = SCE->regName; \
    *((addr) + 22) = SCE->regName; \
    *((addr) + 23) = SCE->regName; \
    *((addr) + 24) = SCE->regName; \
    *((addr) + 25) = SCE->regName; \
    *((addr) + 26) = SCE->regName; \
    *((addr) + 27) = SCE->regName; \
    *((addr) + 28) = SCE->regName; \
    *((addr) + 29) = SCE->regName; \
    *((addr) + 30) = SCE->regName; \
    *((addr) + 31) = SCE->regName

// [R RD 64 MEM[Ofs]]
#define  RD64_ADDR(regName, addr)  \
    *((addr) + 0)  = SCE->regName; \
    *((addr) + 1)  = SCE->regName; \
    *((addr) + 2)  = SCE->regName; \
    *((addr) + 3)  = SCE->regName; \
    *((addr) + 4)  = SCE->regName; \
    *((addr) + 5)  = SCE->regName; \
    *((addr) + 6)  = SCE->regName; \
    *((addr) + 7)  = SCE->regName; \
    *((addr) + 8)  = SCE->regName; \
    *((addr) + 9)  = SCE->regName; \
    *((addr) + 10) = SCE->regName; \
    *((addr) + 11) = SCE->regName; \
    *((addr) + 12) = SCE->regName; \
    *((addr) + 13) = SCE->regName; \
    *((addr) + 14) = SCE->regName; \
    *((addr) + 15) = SCE->regName; \
    *((addr) + 16) = SCE->regName; \
    *((addr) + 17) = SCE->regName; \
    *((addr) + 18) = SCE->regName; \
    *((addr) + 19) = SCE->regName; \
    *((addr) + 20) = SCE->regName; \
    *((addr) + 21) = SCE->regName; \
    *((addr) + 22) = SCE->regName; \
    *((addr) + 23) = SCE->regName; \
    *((addr) + 24) = SCE->regName; \
    *((addr) + 25) = SCE->regName; \
    *((addr) + 26) = SCE->regName; \
    *((addr) + 27) = SCE->regName; \
    *((addr) + 28) = SCE->regName; \
    *((addr) + 29) = SCE->regName; \
    *((addr) + 30) = SCE->regName; \
    *((addr) + 31) = SCE->regName; \
    *((addr) + 32) = SCE->regName; \
    *((addr) + 33) = SCE->regName; \
    *((addr) + 34) = SCE->regName; \
    *((addr) + 35) = SCE->regName; \
    *((addr) + 36) = SCE->regName; \
    *((addr) + 37) = SCE->regName; \
    *((addr) + 38) = SCE->regName; \
    *((addr) + 39) = SCE->regName; \
    *((addr) + 40) = SCE->regName; \
    *((addr) + 41) = SCE->regName; \
    *((addr) + 42) = SCE->regName; \
    *((addr) + 43) = SCE->regName; \
    *((addr) + 44) = SCE->regName; \
    *((addr) + 45) = SCE->regName; \
    *((addr) + 46) = SCE->regName; \
    *((addr) + 47) = SCE->regName; \
    *((addr) + 48) = SCE->regName; \
    *((addr) + 49) = SCE->regName; \
    *((addr) + 50) = SCE->regName; \
    *((addr) + 51) = SCE->regName; \
    *((addr) + 52) = SCE->regName; \
    *((addr) + 53) = SCE->regName; \
    *((addr) + 54) = SCE->regName; \
    *((addr) + 55) = SCE->regName; \
    *((addr) + 56) = SCE->regName; \
    *((addr) + 57) = SCE->regName; \
    *((addr) + 58) = SCE->regName; \
    *((addr) + 59) = SCE->regName; \
    *((addr) + 60) = SCE->regName; \
    *((addr) + 61) = SCE->regName; \
    *((addr) + 62) = SCE->regName; \
    *((addr) + 63) = SCE->regName

// [TEST_BUSY WAIT A]
#define  WAI_BUSY(value) \
    WAIT_STS(SCE->REG_00H, 31, value)

void SC32_function001(uint32_t ARG1, uint32_t ARG2, uint32_t ARG3, uint32_t ARG4);
void SC32_function002(uint32_t ARG1, uint32_t ARG2, uint32_t ARG3, uint32_t ARG4);
void SC32_function003(uint32_t ARG1, uint32_t ARG2, uint32_t ARG3, uint32_t ARG4);

void SC327_function001(uint32_t ARG1, uint32_t ARG2, uint32_t ARG3, uint32_t ARG4);
void SC327_function002(uint32_t ARG1, uint32_t ARG2, uint32_t ARG3, uint32_t ARG4);
void SC327_function003(uint32_t ARG1, uint32_t ARG2, uint32_t ARG3, uint32_t ARG4);

// [R WR 32 B0 B1 B2 B3 B4 .... B31]
#define  WR32_PROG(regName,                                                                                  \
                   value0, value1, value2, value3, value4, value5, value6, value7, value8, value9,           \
                   value10, value11, value12, value13, value14, value15, value16, value17, value18, value19, \
                   value20, value21, value22, value23, value24, value25, value26, value27, value28, value29, \
                   value30, value31                                                                          \
                   )                                                                                         \
    WR1_PROG(regName, value0);                                                                               \
    WR1_PROG(regName, value1);                                                                               \
    WR1_PROG(regName, value2);                                                                               \
    WR1_PROG(regName, value3);                                                                               \
    WR1_PROG(regName, value4);                                                                               \
    WR1_PROG(regName, value5);                                                                               \
    WR1_PROG(regName, value6);                                                                               \
    WR1_PROG(regName, value7);                                                                               \
    WR1_PROG(regName, value8);                                                                               \
    WR1_PROG(regName, value9);                                                                               \
    WR1_PROG(regName, value10);                                                                              \
    WR1_PROG(regName, value11);                                                                              \
    WR1_PROG(regName, value12);                                                                              \
    WR1_PROG(regName, value13);                                                                              \
    WR1_PROG(regName, value14);                                                                              \
    WR1_PROG(regName, value15);                                                                              \
    WR1_PROG(regName, value16);                                                                              \
    WR1_PROG(regName, value17);                                                                              \
    WR1_PROG(regName, value18);                                                                              \
    WR1_PROG(regName, value19);                                                                              \
    WR1_PROG(regName, value20);                                                                              \
    WR1_PROG(regName, value21);                                                                              \
    WR1_PROG(regName, value22);                                                                              \
    WR1_PROG(regName, value23);                                                                              \
    WR1_PROG(regName, value24);                                                                              \
    WR1_PROG(regName, value25);                                                                              \
    WR1_PROG(regName, value26);                                                                              \
    WR1_PROG(regName, value27);                                                                              \
    WR1_PROG(regName, value28);                                                                              \
    WR1_PROG(regName, value29);                                                                              \
    WR1_PROG(regName, value30);                                                                              \
    WR1_PROG(regName, value31)

// [R WR 48 B0 B1 B2 B3 B4 .... B47]
#define  WR48_PROG(regName,                                                                                  \
                   value0, value1, value2, value3, value4, value5, value6, value7, value8, value9,           \
                   value10, value11, value12, value13, value14, value15, value16, value17, value18, value19, \
                   value20, value21, value22, value23, value24, value25, value26, value27, value28, value29, \
                   value30, value31, value32, value33, value34, value35, value36, value37, value38, value39, \
                   value40, value41, value42, value43, value44, value45, value46, value47                    \
                   )                                                                                         \
    WR1_PROG(regName, value0);                                                                               \
    WR1_PROG(regName, value1);                                                                               \
    WR1_PROG(regName, value2);                                                                               \
    WR1_PROG(regName, value3);                                                                               \
    WR1_PROG(regName, value4);                                                                               \
    WR1_PROG(regName, value5);                                                                               \
    WR1_PROG(regName, value6);                                                                               \
    WR1_PROG(regName, value7);                                                                               \
    WR1_PROG(regName, value8);                                                                               \
    WR1_PROG(regName, value9);                                                                               \
    WR1_PROG(regName, value10);                                                                              \
    WR1_PROG(regName, value11);                                                                              \
    WR1_PROG(regName, value12);                                                                              \
    WR1_PROG(regName, value13);                                                                              \
    WR1_PROG(regName, value14);                                                                              \
    WR1_PROG(regName, value15);                                                                              \
    WR1_PROG(regName, value16);                                                                              \
    WR1_PROG(regName, value17);                                                                              \
    WR1_PROG(regName, value18);                                                                              \
    WR1_PROG(regName, value19);                                                                              \
    WR1_PROG(regName, value20);                                                                              \
    WR1_PROG(regName, value21);                                                                              \
    WR1_PROG(regName, value22);                                                                              \
    WR1_PROG(regName, value23);                                                                              \
    WR1_PROG(regName, value24);                                                                              \
    WR1_PROG(regName, value25);                                                                              \
    WR1_PROG(regName, value26);                                                                              \
    WR1_PROG(regName, value27);                                                                              \
    WR1_PROG(regName, value28);                                                                              \
    WR1_PROG(regName, value29);                                                                              \
    WR1_PROG(regName, value30);                                                                              \
    WR1_PROG(regName, value31);                                                                              \
    WR1_PROG(regName, value32);                                                                              \
    WR1_PROG(regName, value33);                                                                              \
    WR1_PROG(regName, value34);                                                                              \
    WR1_PROG(regName, value35);                                                                              \
    WR1_PROG(regName, value36);                                                                              \
    WR1_PROG(regName, value37);                                                                              \
    WR1_PROG(regName, value38);                                                                              \
    WR1_PROG(regName, value39);                                                                              \
    WR1_PROG(regName, value40);                                                                              \
    WR1_PROG(regName, value41);                                                                              \
    WR1_PROG(regName, value42);                                                                              \
    WR1_PROG(regName, value43);                                                                              \
    WR1_PROG(regName, value44);                                                                              \
    WR1_PROG(regName, value45);                                                                              \
    WR1_PROG(regName, value46);                                                                              \
    WR1_PROG(regName, value47)

// [R WR 48 B0 B1 B2 B3 B4 .... B47]
#define  WR64_PROG(regName,                                                                                  \
                   value0, value1, value2, value3, value4, value5, value6, value7, value8, value9,           \
                   value10, value11, value12, value13, value14, value15, value16, value17, value18, value19, \
                   value20, value21, value22, value23, value24, value25, value26, value27, value28, value29, \
                   value30, value31, value32, value33, value34, value35, value36, value37, value38, value39, \
                   value40, value41, value42, value43, value44, value45, value46, value47, value48, value49, \
                   value50, value51, value52, value53, value54, value55, value56, value57, value58, value59, \
                   value60, value61, value62, value63                                                        \
                   )                                                                                         \
    WR1_PROG(regName, value0);                                                                               \
    WR1_PROG(regName, value1);                                                                               \
    WR1_PROG(regName, value2);                                                                               \
    WR1_PROG(regName, value3);                                                                               \
    WR1_PROG(regName, value4);                                                                               \
    WR1_PROG(regName, value5);                                                                               \
    WR1_PROG(regName, value6);                                                                               \
    WR1_PROG(regName, value7);                                                                               \
    WR1_PROG(regName, value8);                                                                               \
    WR1_PROG(regName, value9);                                                                               \
    WR1_PROG(regName, value10);                                                                              \
    WR1_PROG(regName, value11);                                                                              \
    WR1_PROG(regName, value12);                                                                              \
    WR1_PROG(regName, value13);                                                                              \
    WR1_PROG(regName, value14);                                                                              \
    WR1_PROG(regName, value15);                                                                              \
    WR1_PROG(regName, value16);                                                                              \
    WR1_PROG(regName, value17);                                                                              \
    WR1_PROG(regName, value18);                                                                              \
    WR1_PROG(regName, value19);                                                                              \
    WR1_PROG(regName, value20);                                                                              \
    WR1_PROG(regName, value21);                                                                              \
    WR1_PROG(regName, value22);                                                                              \
    WR1_PROG(regName, value23);                                                                              \
    WR1_PROG(regName, value24);                                                                              \
    WR1_PROG(regName, value25);                                                                              \
    WR1_PROG(regName, value26);                                                                              \
    WR1_PROG(regName, value27);                                                                              \
    WR1_PROG(regName, value28);                                                                              \
    WR1_PROG(regName, value29);                                                                              \
    WR1_PROG(regName, value30);                                                                              \
    WR1_PROG(regName, value31);                                                                              \
    WR1_PROG(regName, value32);                                                                              \
    WR1_PROG(regName, value33);                                                                              \
    WR1_PROG(regName, value34);                                                                              \
    WR1_PROG(regName, value35);                                                                              \
    WR1_PROG(regName, value36);                                                                              \
    WR1_PROG(regName, value37);                                                                              \
    WR1_PROG(regName, value38);                                                                              \
    WR1_PROG(regName, value39);                                                                              \
    WR1_PROG(regName, value40);                                                                              \
    WR1_PROG(regName, value41);                                                                              \
    WR1_PROG(regName, value42);                                                                              \
    WR1_PROG(regName, value43);                                                                              \
    WR1_PROG(regName, value44);                                                                              \
    WR1_PROG(regName, value45);                                                                              \
    WR1_PROG(regName, value46);                                                                              \
    WR1_PROG(regName, value47);                                                                              \
    WR1_PROG(regName, value48);                                                                              \
    WR1_PROG(regName, value49);                                                                              \
    WR1_PROG(regName, value50);                                                                              \
    WR1_PROG(regName, value51);                                                                              \
    WR1_PROG(regName, value52);                                                                              \
    WR1_PROG(regName, value53);                                                                              \
    WR1_PROG(regName, value54);                                                                              \
    WR1_PROG(regName, value55);                                                                              \
    WR1_PROG(regName, value56);                                                                              \
    WR1_PROG(regName, value57);                                                                              \
    WR1_PROG(regName, value58);                                                                              \
    WR1_PROG(regName, value59);                                                                              \
    WR1_PROG(regName, value60);                                                                              \
    WR1_PROG(regName, value61);                                                                              \
    WR1_PROG(regName, value62);                                                                              \
    WR1_PROG(regName, value63)

#endif                                 // __SCE_ProcCommon_h__
